@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
@@ -38,13 +38,13 @@
 #include <string.h>
 #include <unistd.h>
 
+#include <glib.h>
+#include <glib/gi18n-lib.h>
+
 #ifndef G_OS_WIN32
 #include <sys/wait.h>
 #endif
 
-#include <glib.h>
-#include <glib/gi18n-lib.h>
-
 #include <libedataserver/e-iconv.h>
 #include <libedataserver/e-sexp.h>
 
@@ -58,6 +58,7 @@
 #include "camel-session.h"
 #include "camel-stream-fs.h"
 #include "camel-stream-mem.h"
+#include "camel-string-utils.h"
 #include "camel-url.h"
 
 #define d(x)
@@ -93,7 +94,10 @@ static ESExpResult *get_current_date (struct _ESExp *f, int argc, struct _ESExpR
 static ESExpResult *header_source (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms);
 static ESExpResult *get_size (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms);
 static ESExpResult *pipe_message (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms);
+
+#ifdef NON_TINYMAIL_FEATURES
 static ESExpResult *junk_test (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms);
+#endif
 
 /* builtin functions */
 static struct {
@@ -122,7 +126,9 @@ static struct {
 	{ "header-source",      (ESExpFunc *) header_source,      0 },
 	{ "get-size",           (ESExpFunc *) get_size,           0 },
 	{ "pipe-message",       (ESExpFunc *) pipe_message,       0 },
+#ifdef NON_TINYMAIL_FEATURES
 	{ "junk-test",          (ESExpFunc *) junk_test,          0 },
+#endif
 };
 
 
@@ -131,12 +137,12 @@ camel_filter_search_get_message (FilterMessageSearch *fms, struct _ESExp *sexp)
 {
 	if (fms->message)
 		return fms->message;
-	
+
 	fms->message = fms->get_message (fms->get_message_data, fms->ex);
-	
+
 	if (fms->message == NULL)
 		e_sexp_fatal_error (sexp, _("Failed to retrieve message"));
-	
+
 	return fms->message;
 }
 
@@ -146,13 +152,14 @@ check_header (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMess
 	gboolean matched = FALSE;
 	ESExpResult *r;
 	int i;
-	
+
 	if (argc > 1 && argv[0]->type == ESEXP_RES_STRING) {
 		char *name = argv[0]->value.string;
 
 		/* shortcut: a match for "" against any header always matches */
 		for (i=1; i<argc && !matched; i++)
 			matched = argv[i]->type == ESEXP_RES_STRING && argv[i]->value.string[0] == 0;
+
 #if NON_TINYMAIL_FEATURES
 		if (g_ascii_strcasecmp(name, "x-camel-mlist") == 0) {
 			const char *list = camel_message_info_mlist(fms->info);
@@ -165,16 +172,19 @@ check_header (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMess
 			}
 		} else {
 #endif
-			CamelMimeMessage *message = camel_filter_search_get_message (fms, f);
+			{
+			CamelMimeMessage *message;
 			struct _camel_header_raw *header;
 			const char *charset = NULL;
 			camel_search_t type = CAMEL_SEARCH_TYPE_ENCODED;
 			CamelContentType *ct;
 
+			message = camel_filter_search_get_message (fms, f);
+
 			/* FIXME: what about Resent-To, Resent-Cc and Resent-From? */
 			if (g_ascii_strcasecmp("to", name) == 0 || g_ascii_strcasecmp("cc", name) == 0 || g_ascii_strcasecmp("from", name) == 0)
 				type = CAMEL_SEARCH_TYPE_ADDRESS_ENCODED;
-			else {
+			else if (message) {
 				ct = camel_mime_part_get_content_type (CAMEL_MIME_PART (message));
 				if (ct) {
 					charset = camel_content_type_param (ct, "charset");
@@ -193,11 +203,12 @@ check_header (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMess
 #ifdef NON_TINYMAIL_FEATURES
 		}
 #endif
+		}
 	}
-	
+
 	r = e_sexp_result_new (f, ESEXP_RES_BOOL);
 	r->value.bool = matched;
-	
+
 	return r;
 }
 
@@ -239,17 +250,17 @@ header_exists (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMes
 	gboolean matched = FALSE;
 	ESExpResult *r;
 	int i;
-	
+
 	message = camel_filter_search_get_message (fms, f);
-	
+
 	for (i = 0; i < argc && !matched; i++) {
 		if (argv[i]->type == ESEXP_RES_STRING)
 			matched = camel_medium_get_header (CAMEL_MEDIUM (message), argv[i]->value.string) != NULL;
 	}
-	
+
 	r = e_sexp_result_new (f, ESEXP_RES_BOOL);
 	r->value.bool = matched;
-	
+
 	return r;
 }
 
@@ -260,9 +271,9 @@ header_regex (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMess
 	CamelMimeMessage *message;
 	regex_t pattern;
 	const char *contents;
-	
+
 	message = camel_filter_search_get_message (fms, f);
-	
+
 	if (argc > 1 && argv[0]->type == ESEXP_RES_STRING
 	    && (contents = camel_medium_get_header (CAMEL_MEDIUM (message), argv[0]->value.string))
 	    && camel_search_build_match_regex(&pattern, CAMEL_SEARCH_MATCH_REGEX|CAMEL_SEARCH_MATCH_ICASE, argc-1, argv+1, fms->ex) == 0) {
@@ -270,7 +281,7 @@ header_regex (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMess
 		regfree (&pattern);
 	} else
 		r->value.bool = FALSE;
-	
+
 	return r;
 }
 
@@ -281,7 +292,7 @@ get_full_header (CamelMimeMessage *message)
 	GString *str = g_string_new ("");
 	char   *ret;
 	struct _camel_header_raw *h;
-	
+
 	for (h = mp->headers; h; h = h->next) {
 		if (h->value != NULL) {
 			g_string_append (str, h->name);
@@ -293,10 +304,10 @@ get_full_header (CamelMimeMessage *message)
 			g_string_append_c(str, '\n');
 		}
 	}
-	
+
 	ret = str->str;
 	g_string_free (str, FALSE);
-	
+
 	return ret;
 }
 
@@ -307,7 +318,7 @@ header_full_regex (struct _ESExp *f, int argc, struct _ESExpResult **argv, Filte
 	CamelMimeMessage *message;
 	regex_t pattern;
 	char *contents;
-	
+
 	if (camel_search_build_match_regex(&pattern, CAMEL_SEARCH_MATCH_REGEX|CAMEL_SEARCH_MATCH_ICASE|CAMEL_SEARCH_MATCH_NEWLINE,
 					   argc, argv, fms->ex) == 0) {
 		message = camel_filter_search_get_message (fms, f);
@@ -317,7 +328,7 @@ header_full_regex (struct _ESExp *f, int argc, struct _ESExpResult **argv, Filte
 		regfree (&pattern);
 	} else
 		r->value.bool = FALSE;
-	
+
 	return r;
 }
 
@@ -326,13 +337,13 @@ match_all (struct _ESExp *f, int argc, struct _ESExpTerm **argv, FilterMessageSe
 {
 	/* match-all: when dealing with single messages is a no-op */
 	ESExpResult *r;
-	
+
 	if (argc > 0)
 		return e_sexp_term_eval (f, argv[0]);
-	
+
 	r = e_sexp_result_new (f, ESEXP_RES_BOOL);
 	r->value.bool = TRUE;
-	
+
 	return r;
 }
 
@@ -342,14 +353,14 @@ body_contains (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMes
 	ESExpResult *r = e_sexp_result_new (f, ESEXP_RES_BOOL);
 	CamelMimeMessage *message;
 	regex_t pattern;
-	
+
 	if (camel_search_build_match_regex (&pattern, CAMEL_SEARCH_MATCH_ICASE, argc, argv, fms->ex) == 0) {
 		message = camel_filter_search_get_message (fms, f);
 		r->value.bool = camel_search_message_body_contains ((CamelDataWrapper *) message, &pattern);
 		regfree (&pattern);
 	} else
 		r->value.bool = FALSE;
-	
+
 	return r;
 }
 
@@ -359,7 +370,7 @@ body_regex (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessag
 	ESExpResult *r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 	CamelMimeMessage *message;
 	regex_t pattern;
-	
+
 	if (camel_search_build_match_regex(&pattern, CAMEL_SEARCH_MATCH_ICASE|CAMEL_SEARCH_MATCH_REGEX|CAMEL_SEARCH_MATCH_NEWLINE,
 					   argc, argv, fms->ex) == 0) {
 		message = camel_filter_search_get_message (fms, f);
@@ -367,7 +378,7 @@ body_regex (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessag
 		regfree (&pattern);
 	} else
 		r->value.bool = FALSE;
-	
+
 	return r;
 }
 
@@ -377,7 +388,7 @@ user_flag (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessage
 	ESExpResult *r;
 	gboolean truth = FALSE;
 	int i;
-	
+
 	/* performs an OR of all words */
 	for (i = 0; i < argc && !truth; i++) {
 		if (argv[i]->type == ESEXP_RES_STRING
@@ -386,10 +397,10 @@ user_flag (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessage
 			break;
 		}
 	}
-	
+
 	r = e_sexp_result_new (f, ESEXP_RES_BOOL);
 	r->value.bool = truth;
-	
+
 	return r;
 }
 
@@ -397,13 +408,13 @@ static ESExpResult *
 system_flag (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms)
 {
 	ESExpResult *r;
-	
+
 	if (argc != 1 || argv[0]->type != ESEXP_RES_STRING)
 		e_sexp_fatal_error(f, _("Invalid arguments to (system-flag)"));
-	
+
 	r = e_sexp_result_new (f, ESEXP_RES_BOOL);
 	r->value.bool = camel_system_flag_get (camel_message_info_flags(fms->info), argv[0]->value.string);
-	
+
 	return r;
 }
 
@@ -415,12 +426,12 @@ user_tag (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageS
 
 	if (argc != 1 || argv[0]->type != ESEXP_RES_STRING)
 		e_sexp_fatal_error(f, _("Invalid arguments to (user-tag)"));
-	
+
 	tag = camel_message_info_user_tag(fms->info, argv[0]->value.string);
-	
+
 	r = e_sexp_result_new (f, ESEXP_RES_STRING);
 	r->value.string = g_strdup (tag ? tag : "");
-	
+
 	return r;
 }
 
@@ -429,11 +440,11 @@ get_sent_date (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMes
 {
 	CamelMimeMessage *message;
 	ESExpResult *r;
-	
+
 	message = camel_filter_search_get_message (fms, f);
 	r = e_sexp_result_new (f, ESEXP_RES_INT);
 	r->value.number = camel_mime_message_get_date (message, NULL);
-	
+
 	return r;
 }
 
@@ -442,11 +453,11 @@ get_received_date (struct _ESExp *f, int argc, struct _ESExpResult **argv, Filte
 {
 	CamelMimeMessage *message;
 	ESExpResult *r;
-	
+
 	message = camel_filter_search_get_message (fms, f);
 	r = e_sexp_result_new (f, ESEXP_RES_INT);
 	r->value.number = camel_mime_message_get_date_received (message, NULL);
-	
+
 	return r;
 }
 
@@ -454,10 +465,10 @@ static ESExpResult *
 get_current_date (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms)
 {
 	ESExpResult *r;
-	
+
 	r = e_sexp_result_new (f, ESEXP_RES_INT);
 	r->value.number = time (NULL);
-	
+
 	return r;
 }
 
@@ -496,7 +507,7 @@ header_source (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMes
 
 	r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 	r->value.bool = truth;
-	
+
 	return r;
 }
 
@@ -505,10 +516,9 @@ static ESExpResult *
 get_size (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms)
 {
 	ESExpResult *r;
-	
-	r = e_sexp_result_new(f, ESEXP_RES_INT);
 
-	r->value.number = camel_message_info_size(fms->info);
+	r = e_sexp_result_new(f, ESEXP_RES_INT);
+	r->value.number = camel_message_info_size(fms->info) / 1024;
 
 	return r;
 }
@@ -554,15 +564,15 @@ run_command (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessa
 	child_watch_data_t child_watch_data;
 	GSource *source;
 	GMainContext *context;
-	
+
 	if (argc < 1 || argv[0]->value.string[0] == '\0')
 		return 0;
-	
+
 	args = g_ptr_array_new ();
 	for (i = 0; i < argc; i++)
 		g_ptr_array_add (args, argv[i]->value.string);
 	g_ptr_array_add (args, NULL);
-	
+
 	if (!g_spawn_async_with_pipes (NULL,
 				       (gchar **) args->pdata,
 				       NULL,
@@ -585,16 +595,16 @@ run_command (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessa
 		g_error_free (error);
 		return -1;
 	}
-	
+
 	g_ptr_array_free (args, TRUE);
-	
+
 	message = camel_filter_search_get_message (fms, f);
-	
+
 	stream = camel_stream_fs_new_with_fd (pipe_to_child);
 	camel_data_wrapper_write_to_stream (CAMEL_DATA_WRAPPER (message), stream);
 	camel_stream_flush (stream);
 	camel_object_unref (stream);
-	
+
 	context = g_main_context_new ();
 	child_watch_data.loop = g_main_loop_new (context, FALSE);
 	g_main_context_unref (context);
@@ -622,7 +632,7 @@ pipe_message (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMess
 {
 	ESExpResult *r;
 	int retval, i;
-	
+
 	/* make sure all args are strings */
 	for (i = 0; i < argc; i++) {
 		if (argv[i]->type != ESEXP_RES_STRING) {
@@ -630,25 +640,52 @@ pipe_message (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMess
 			goto done;
 		}
 	}
-	
+
 	retval = run_command (f, argc, argv, fms);
-	
+
  done:
 	r = e_sexp_result_new (f, ESEXP_RES_INT);
 	r->value.number = retval;
-	
+
 	return r;
 }
 
+#ifdef NON_TINYMAIL_FEATURES
 static ESExpResult *
 junk_test (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms)
 {
 	ESExpResult *r;
 	gboolean retval = FALSE;
-
+	
+	d(printf("doing junk test for message from '%s'\n", camel_message_info_from (fms->info)));
 	if (fms->session->junk_plugin != NULL) {
-		retval = camel_junk_plugin_check_junk (fms->session->junk_plugin, camel_filter_search_get_message (fms, f));
+		CamelMessageInfo *info = fms->info;
+		const GHashTable *ht = camel_session_get_junk_headers (fms->session);
+		struct _camel_header_param *node = ((CamelMessageInfoBase *)info)->headers;
+
+		while (node && !retval) {
+			if (node->name) {
+				char *value = (char *) g_hash_table_lookup ((GHashTable *) ht, node->name);
+				d(printf("JunkCheckMatch: %s %s %s\n", node->name, node->value, value));
+				if (value)
+					retval = camel_strstrcase(node->value, value) != NULL;
 		
+			}
+			node = node->next;
+		}
+		if (camel_debug ("junk"))
+			printf("filtered based on junk header ? %d\n", retval);
+		if (!retval) {
+			retval = camel_session_lookup_addressbook (fms->session, camel_message_info_from (info)) != TRUE;
+			if (camel_debug ("junk"))
+				printf("Sender '%s' in book? %d\n", camel_message_info_from (info), !retval);
+			
+			if (retval) /* Not in book. Could be spam. So check for it*/ {
+				d(printf("filtering message\n"));
+				retval = camel_junk_plugin_check_junk (fms->session->junk_plugin, camel_filter_search_get_message (fms, f));
+			}
+		}
+
 		if (camel_debug ("junk"))
 			printf("junk filter => %s\n", retval ? "*JUNK*" : "clean");
 	}
@@ -658,6 +695,7 @@ junk_test (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessage
 
 	return r;
 }
+#endif
 
 /**
  * camel_filter_search_match:
@@ -690,16 +728,16 @@ camel_filter_search_match (CamelSession *session,
 	fms.info = info;
 	fms.source = source;
 	fms.ex = ex;
-	
+
 	sexp = e_sexp_new ();
-	
+
 	for (i = 0; i < sizeof (symbols) / sizeof (symbols[0]); i++) {
 		if (symbols[i].type == 1)
 			e_sexp_add_ifunction (sexp, 0, symbols[i].name, (ESExpIFunc *)symbols[i].func, &fms);
 		else
 			e_sexp_add_function (sexp, 0, symbols[i].name, symbols[i].func, &fms);
 	}
-	
+
 	e_sexp_input_text (sexp, expression, strlen (expression));
 	if (e_sexp_parse (sexp) == -1) {
 		if (!camel_exception_is_set (ex))
@@ -708,7 +746,7 @@ camel_filter_search_match (CamelSession *session,
 					      e_sexp_error (sexp), expression);
 		goto error;
 	}
-	
+
 	result = e_sexp_eval (sexp);
 	if (result == NULL) {
 		if (!camel_exception_is_set (ex))
@@ -716,25 +754,25 @@ camel_filter_search_match (CamelSession *session,
 					      e_sexp_error (sexp), expression);
 		goto error;
 	}
-	
+
 	if (result->type == ESEXP_RES_BOOL)
 		retval = result->value.bool ? CAMEL_SEARCH_MATCHED : CAMEL_SEARCH_NOMATCH;
 	else
 		retval = CAMEL_SEARCH_NOMATCH;
-	
+
 	e_sexp_result_free (sexp, result);
 	e_sexp_unref (sexp);
-	
+
 	if (fms.message)
 		camel_object_unref (fms.message);
-	
+
 	return retval;
-	
+
  error:
 	if (fms.message)
 		camel_object_unref (fms.message);
-	
+
 	e_sexp_unref (sexp);
-	
+
 	return CAMEL_SEARCH_ERROR;
 }
