@@ -306,8 +306,9 @@ static void free_o_name(void *key, void *value, void *data)
 static void
 foreach_msginfo (gpointer data, gpointer user_data)
 {
+	//CamelMessageInfoBase *base = data;
 	camel_message_info_free (data);
-	/* g_slice_free1 ((gint)user_data, data); */
+	//base->summary = NULL;
 }
 
 static inline
@@ -338,6 +339,8 @@ camel_folder_summary_finalize (CamelObject *obj)
 
 	p = _PRIVATE(obj);
 
+	g_static_rec_mutex_lock (&global_lock);
+
 	g_static_rec_mutex_lock (s->dump_lock);
 
 	g_ptr_array_foreach (s->messages, foreach_msginfo, (gpointer)s->message_info_size);
@@ -350,6 +353,9 @@ camel_folder_summary_finalize (CamelObject *obj)
 	camel_folder_summary_unload_mmap (s);
 
 	g_static_rec_mutex_unlock (s->dump_lock);
+
+	g_static_rec_mutex_unlock (&global_lock);
+
 	/**/
 	g_free (s->dump_lock);
 
@@ -729,6 +735,7 @@ camel_folder_summary_load(CamelFolderSummary *s)
 	int i;
 	CamelMessageInfo *mi;
 	GError *err = NULL;
+	gboolean ul = FALSE;
 
 	if (s->summary_path == NULL || !g_file_test (s->summary_path, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR))
 		return -1;
@@ -768,15 +775,17 @@ printf ("Removes %s\n", ri->uid);
 		}
 	}
 
+	g_static_rec_mutex_lock (&global_lock);
+
 	/* now read in each message ... */
 	for (i=0; i < s->saved_count; i++)
 	{
 		gboolean must_add = FALSE;
 		s->idx = i;
 
-		g_static_rec_mutex_lock (&global_lock);
+		ul = TRUE;
+
 		mi = ((CamelFolderSummaryClass *)(CAMEL_OBJECT_GET_CLASS(s)))->message_info_load(s, &must_add);
-		g_static_rec_mutex_unlock (&global_lock);
 
 		camel_operation_progress (NULL, i , s->saved_count);
 
@@ -799,6 +808,10 @@ printf ("Removes %s\n", ri->uid);
 			camel_folder_summary_mmap_add(s, mi);
 	}
 
+	g_static_rec_mutex_unlock (&global_lock);
+
+	ul = FALSE;
+
 	if (s->saved_count <= 0) {
 		g_mapped_file_free (s->file);
 		s->file = NULL;
@@ -814,6 +827,9 @@ printf ("Removes %s\n", ri->uid);
 	return 0;
 
 error:
+	if (ul)
+		g_static_rec_mutex_unlock (&global_lock);
+
 	camel_operation_end (NULL);
 
 	if (errno != EINVAL)
@@ -854,6 +870,16 @@ perform_content_info_save(CamelFolderSummary *s, FILE *out, CamelMessageContentI
 	return 0;
 }
 
+static void 
+flush_for_reload (CamelFolderSummary *s, CamelMessageInfoBase *mi)
+{
+	if (!(mi->flags & CAMEL_MESSAGE_INFO_NEEDS_FREE)) {
+		mi->subject = "...";
+		mi->to = "...";
+		mi->from = "...";
+		mi->cc = "...";
+	}
+} 
 
 /**
  * camel_folder_summary_save:
@@ -903,7 +929,7 @@ camel_folder_summary_save_append (CamelFolderSummary *s, CamelException *ex)
 		i = errno;
 		g_unlink(path);
 		errno = i;
- 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
+ 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM_IO_WRITE,
 			"Error storing the summary");
 		g_static_rec_mutex_unlock (s->dump_lock);
 		return -1;
@@ -943,6 +969,7 @@ camel_folder_summary_save_append (CamelFolderSummary *s, CamelException *ex)
 				}
 			}
 		}
+
 	}
 
 	if (fflush (out) != 0 || fsync (fileno (out)) == -1)
@@ -972,6 +999,11 @@ haerror:
 	g_static_rec_mutex_lock (&global_lock);
 	g_static_mutex_lock (&global_lock2);
 
+	for (i = 0; i < count; i++) {
+		mi = s->messages->pdata[i];
+		flush_for_reload (s, (CamelMessageInfoBase *) mi);
+	}
+
 	/* Remap the appended file :). Moeha! */
 	camel_folder_summary_unload_mmap (s);
 
@@ -998,7 +1030,7 @@ haerror:
 
 exception:
 
-	camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
+	camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM_IO_WRITE,
 		"Error storing the summary");
 	i = errno;
 	fclose (out);
@@ -1038,7 +1070,7 @@ camel_folder_summary_save_rewrite (CamelFolderSummary *s, CamelException *ex)
 
 	if (fd == -1) {
 	  g_static_rec_mutex_unlock (s->dump_lock);
-	  camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
+	  camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM_IO_WRITE,
 		"Error storing the summary");
 	  return -1;
 	}
@@ -1049,7 +1081,7 @@ camel_folder_summary_save_rewrite (CamelFolderSummary *s, CamelException *ex)
 		g_unlink(path);
 		close(fd);
 		errno = i;
- 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
+ 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM_IO_WRITE,
 			"Error storing the summary");
 		g_static_rec_mutex_unlock (s->dump_lock);
 		return -1;
@@ -1121,7 +1153,7 @@ haerror:
 		i = errno;
 		g_unlink(path);
 		errno = i;
-		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
+		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM_IO_WRITE,
 			"Error storing the summary");
 		g_static_mutex_unlock (&global_lock2);
 		g_static_rec_mutex_unlock (&global_lock);
@@ -1146,7 +1178,7 @@ haerror:
 
 exception:
 
-	camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
+	camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM_IO_WRITE,
 		"Error storing the summary");
 	i = errno;
 	fclose (out);
@@ -2194,6 +2226,8 @@ message_info_new_from_header(CamelFolderSummary *s, struct _camel_header_raw *h)
 	prio = camel_header_raw_find(&h, "X-Priority", NULL);
 	if (!prio)
 		prio = camel_header_raw_find(&h, "X-MSMail-Priority", NULL);
+	if (!prio)
+		prio = camel_header_raw_find(&h, "Importance", NULL);
 
 	if (prio)
 	{
@@ -2312,11 +2346,9 @@ message_info_load(CamelFolderSummary *s, gboolean *must_add)
 		mi = (CamelMessageInfoBase *) camel_message_info_new (s);
 		*must_add = TRUE;
 
-		g_static_rec_mutex_lock (&global_lock);
 		if (mi->uid)
 			g_free (mi->uid);
 		mi->uid = g_strdup (theuid);
-		g_static_rec_mutex_unlock (&global_lock);
 
 	} else
 	{
@@ -2330,12 +2362,10 @@ message_info_load(CamelFolderSummary *s, gboolean *must_add)
 			destroy_possible_pstring_stuff (s, (CamelMessageInfo*) mi, FALSE);
 			*must_add = FALSE;
 
-			g_static_rec_mutex_lock (&global_lock);
 
 			if (mi->uid)
 				g_free (mi->uid);
 			mi->uid = g_strdup (theuid);
-			g_static_rec_mutex_unlock (&global_lock);
 
 			CAMEL_SUMMARY_UNLOCK(s, ref_lock);
 		} else
@@ -2351,11 +2381,9 @@ message_info_load(CamelFolderSummary *s, gboolean *must_add)
 		mi = (CamelMessageInfoBase *) camel_message_info_new(s);
 		*must_add = TRUE;
 
-		g_static_rec_mutex_lock (&global_lock);
 		if (mi->uid)
 			g_free (mi->uid);
 		mi->uid = g_strdup (theuid);
-		g_static_rec_mutex_unlock (&global_lock);
 
 	}
 
@@ -3404,6 +3432,7 @@ camel_message_info_free(void *o)
 	g_return_if_fail(mi != NULL);
 
 	if (mi->summary) {
+	 // if (((CamelObject *)mi->summary)->ref_count > 0) {
 		CAMEL_SUMMARY_LOCK(mi->summary, ref_lock);
 
 		if (mi->refcount >= 1)
@@ -3422,6 +3451,7 @@ camel_message_info_free(void *o)
 		}
 
 		((CamelFolderSummaryClass *)(CAMEL_OBJECT_GET_CLASS(mi->summary)))->message_info_free(mi->summary, mi);
+	 // }
 	} else {
 		GLOBAL_INFO_LOCK(info);
 		mi->refcount--;
@@ -4007,6 +4037,17 @@ camel_message_info_dump (CamelMessageInfo *mi)
 	/*camel_content_info_dump(mi->content, 0);*/
 }
 
+void
+camel_folder_summary_lock ()
+{
+	g_static_rec_mutex_lock (&global_lock);	
+}
+
+void
+camel_folder_summary_unlock ()
+{
+	g_static_rec_mutex_unlock (&global_lock);	
+}
 
 static void
 camel_folder_summary_class_init (CamelFolderSummaryClass *klass)
