@@ -98,6 +98,15 @@ struct _CamelTcpStreamSSLPrivate {
 	CamelService *service;
 };
 
+
+static void 
+ssl_enable_compress (CamelTcpStream *stream)
+{
+	/* TODO */ 
+	g_print ("NSS COMPRESS enable request: not yet supported");
+	return;
+}
+
 static void
 camel_tcp_stream_ssl_class_init (CamelTcpStreamSSLClass *camel_tcp_stream_ssl_class)
 {
@@ -113,6 +122,7 @@ camel_tcp_stream_ssl_class_init (CamelTcpStreamSSLClass *camel_tcp_stream_ssl_cl
 	camel_stream_class->flush = stream_flush;
 	camel_stream_class->close = stream_close;
 
+	camel_tcp_stream_class->enable_compress = ssl_enable_compress;
 	camel_tcp_stream_class->gettimeout = stream_gettimeout;
 	camel_tcp_stream_class->read_nb = stream_read_nb;
 	camel_tcp_stream_class->connect = stream_connect;
@@ -190,6 +200,9 @@ camel_tcp_stream_ssl_get_type (void)
  *
  * Returns a new #CamelTcpStreamSSL stream preset in SSL mode
  **/
+
+static gboolean has_init = FALSE;
+
 CamelStream *
 camel_tcp_stream_ssl_new (CamelService *service, const char *expected_host, guint32 flags)
 {
@@ -200,6 +213,15 @@ camel_tcp_stream_ssl_new (CamelService *service, const char *expected_host, guin
 	stream = CAMEL_TCP_STREAM_SSL (camel_object_new (camel_tcp_stream_ssl_get_type ()));
 
 	stream->priv->session = service->session;
+
+	if (!has_init) {
+		char *str = camel_session_get_storage_path (stream->priv->session, service, NULL);
+		if (!str)
+			str = g_strdup (g_get_tmp_dir ());
+		NSS_InitReadWrite (str);
+		has_init = TRUE;
+	}
+
 	camel_object_ref(service->session);
 	stream->priv->expected_host = g_strdup (expected_host);
 	stream->priv->ssl_mode = TRUE;
@@ -822,9 +844,9 @@ ssl_auth_cert (void *data, PRFileDesc *sockfd, PRBool checksig, PRBool is_server
 }
 #endif
 
-CamelCert *camel_certdb_nss_cert_get(CamelCertDB *certdb, CERTCertificate *cert);
-CamelCert *camel_certdb_nss_cert_add(CamelCertDB *certdb, CERTCertificate *cert);
-void camel_certdb_nss_cert_set(CamelCertDB *certdb, CamelCert *ccert, CERTCertificate *cert);
+static CamelCert *camel_certdb_nss_cert_get(CamelCertDB *certdb, CERTCertificate *cert, CamelSession *session);
+static CamelCert *camel_certdb_nss_cert_add(CamelCertDB *certdb, CERTCertificate *cert, CamelSession *session);
+static void camel_certdb_nss_cert_set(CamelCertDB *certdb, CamelCert *ccert, CERTCertificate *cert, CamelSession *session);
 
 static char *
 cert_fingerprint(CERTCertificate *cert)
@@ -851,8 +873,8 @@ cert_fingerprint(CERTCertificate *cert)
 }
 
 /* lookup a cert uses fingerprint to index an on-disk file */
-CamelCert *
-camel_certdb_nss_cert_get(CamelCertDB *certdb, CERTCertificate *cert)
+static CamelCert *
+camel_certdb_nss_cert_get(CamelCertDB *certdb, CERTCertificate *cert, CamelSession *session)
 {
 	char *fingerprint;
 	CamelCert *ccert;
@@ -872,7 +894,7 @@ camel_certdb_nss_cert_get(CamelCertDB *certdb, CERTCertificate *cert)
 		GError *error = NULL;
 
 		filename = g_build_filename (
-			g_get_home_dir (), ".camel_certs", fingerprint, NULL);
+			session->storage_path, ".camel_certs", fingerprint, NULL);
 		g_file_get_contents (filename, &contents, &length, &error);
 		if (error != NULL) {
 			g_warning (
@@ -880,8 +902,6 @@ camel_certdb_nss_cert_get(CamelCertDB *certdb, CERTCertificate *cert)
 				filename, error->message);
 			g_error_free (error);
 
-			camel_cert_set_trust (
-				certdb, ccert, CAMEL_CERT_TRUST_UNKNOWN);
 			camel_certdb_touch (certdb);
 			g_free (fingerprint);
 			g_free (filename);
@@ -901,7 +921,6 @@ camel_certdb_nss_cert_get(CamelCertDB *certdb, CERTCertificate *cert)
 	if (ccert->rawcert->len != cert->derCert.len
 	    || memcmp(ccert->rawcert->data, cert->derCert.data, cert->derCert.len) != 0) {
 		g_warning("rawcert != derCer");
-		camel_cert_set_trust(certdb, ccert, CAMEL_CERT_TRUST_UNKNOWN);
 		camel_certdb_touch(certdb);
 	}
 
@@ -909,8 +928,8 @@ camel_certdb_nss_cert_get(CamelCertDB *certdb, CERTCertificate *cert)
 }
 
 /* add a cert to the certdb */
-CamelCert *
-camel_certdb_nss_cert_add(CamelCertDB *certdb, CERTCertificate *cert)
+static CamelCert *
+camel_certdb_nss_cert_add(CamelCertDB *certdb, CERTCertificate *cert, CamelSession *session)
 {
 	CamelCert *ccert;
 	char *fingerprint;
@@ -923,10 +942,9 @@ camel_certdb_nss_cert_add(CamelCertDB *certdb, CERTCertificate *cert)
 	/* hostname is set in caller */
 	/*camel_cert_set_hostname(certdb, ccert, ssl->priv->expected_host);*/
 	camel_cert_set_fingerprint(certdb, ccert, fingerprint);
-	camel_cert_set_trust(certdb, ccert, CAMEL_CERT_TRUST_UNKNOWN);
 	g_free(fingerprint);
 
-	camel_certdb_nss_cert_set(certdb, ccert, cert);
+	camel_certdb_nss_cert_set(certdb, ccert, cert, session);
 
 	camel_certdb_add(certdb, ccert);
 
@@ -934,8 +952,8 @@ camel_certdb_nss_cert_add(CamelCertDB *certdb, CERTCertificate *cert)
 }
 
 /* set the 'raw' cert (& save it) */
-void
-camel_certdb_nss_cert_set(CamelCertDB *certdb, CamelCert *ccert, CERTCertificate *cert)
+static void
+camel_certdb_nss_cert_set(CamelCertDB *certdb, CamelCert *ccert, CERTCertificate *cert, CamelSession *session)
 {
 	char *dir, *path, *fingerprint;
 	CamelStream *stream;
@@ -949,12 +967,9 @@ camel_certdb_nss_cert_set(CamelCertDB *certdb, CamelCert *ccert, CERTCertificate
 	g_byte_array_set_size (ccert->rawcert, cert->derCert.len);
 	memcpy (ccert->rawcert->data, cert->derCert.data, cert->derCert.len);
 
-#ifndef G_OS_WIN32
-	dir = g_strdup_printf ("%s/.camel_certs", getenv ("HOME"));
-#else
-	dir = g_build_filename (g_get_home_dir (), ".camel_certs", NULL);
-#endif
-	if (g_stat (dir, &st) == -1 && g_mkdir (dir, 0700) == -1) {
+	dir = g_build_filename (session->storage_path, ".camel_certs", NULL);
+
+	if (g_stat (dir, &st) == -1 && g_mkdir_with_parents (dir, 0700) == -1) {
 		g_warning ("Could not create cert directory '%s': %s", dir, strerror (errno));
 		g_free (dir);
 		return;
@@ -1017,6 +1032,7 @@ ssl_bad_cert (void *data, PRFileDesc *sockfd)
 	CERTCertificate *cert;
 	SECStatus status = SECFailure;
 	struct _CamelTcpStreamSSLPrivate *priv;
+	CamelCertTrust trust;
 
 	g_return_val_if_fail (data != NULL, SECFailure);
 	g_return_val_if_fail (CAMEL_IS_TCP_STREAM_SSL (data), SECFailure);
@@ -1029,21 +1045,22 @@ ssl_bad_cert (void *data, PRFileDesc *sockfd)
 		return SECFailure;
 
 	certdb = camel_certdb_get_default();
-	ccert = camel_certdb_nss_cert_get(certdb, cert);
+	ccert = camel_certdb_nss_cert_get(certdb, cert, priv->session);
 	if (ccert == NULL) {
-		ccert = camel_certdb_nss_cert_add(certdb, cert);
+		ccert = camel_certdb_nss_cert_add(certdb, cert, priv->session);
 		camel_cert_set_hostname(certdb, ccert, ssl->priv->expected_host);
 	}
 
-	if (ccert->trust == CAMEL_CERT_TRUST_UNKNOWN) {
+	trust = camel_cert_get_trust (certdb, ccert);
+	if (trust == CAMEL_CERT_TRUST_UNKNOWN) {
 		status = CERT_VerifyCertNow(cert->dbhandle, cert, TRUE, certUsageSSLClient, NULL);
 		fingerprint = cert_fingerprint(cert);
 		cert_str = g_strdup_printf (_("Issuer:            %s\n"
 					      "Subject:           %s\n"
 					      "Fingerprint:       %s\n"
 					      "Signature:         %s"),
-					    CERT_NameToAscii (&cert->issuer),
-					    CERT_NameToAscii (&cert->subject),
+					    ccert?camel_cert_get_issuer (certdb, ccert):CERT_NameToAscii (&cert->issuer),
+					    ccert?camel_cert_get_subject (certdb, ccert):CERT_NameToAscii (&cert->subject),
 					    fingerprint, status == SECSuccess?_("GOOD"):_("BAD"));
 		g_free(fingerprint);
 
@@ -1056,12 +1073,12 @@ ssl_bad_cert (void *data, PRFileDesc *sockfd)
 		accept = camel_session_alert_user_with_id (ssl->priv->session, CAMEL_SESSION_ALERT_WARNING, CAMEL_EXCEPTION_SERVICE_CERTIFICATE, prompt, TRUE, priv->service);
 		g_free(prompt);
 		if (accept) {
-			camel_certdb_nss_cert_set(certdb, ccert, cert);
+			camel_certdb_nss_cert_set(certdb, ccert, cert, ssl->priv->session);
 			camel_cert_set_trust(certdb, ccert, CAMEL_CERT_TRUST_FULLY);
 			camel_certdb_touch(certdb);
 		}
 	} else {
-		accept = ccert->trust != CAMEL_CERT_TRUST_NEVER;
+		accept = trust != CAMEL_CERT_TRUST_NEVER;
 	}
 
 	camel_certdb_cert_unref(certdb, ccert);
