@@ -108,6 +108,7 @@ typedef struct {
 	struct sockaddr * (*get_remote_address) (CamelTcpStream *stream, socklen_t *len);
 	ssize_t (*read_nb)   (CamelTcpStream *stream, char *buffer, size_t n);
 	int (*gettimeout) (CamelTcpStream *stream);
+	void (*enable_compress) (CamelTcpStream *stream);
 
 } CamelTcpStreamClass;
 
@@ -126,6 +127,8 @@ int         camel_tcp_stream_read_nb    (CamelTcpStream *stream, char *buffer, s
 
 int         camel_tcp_stream_gettimeout (CamelTcpStream *stream);
 
+void        camel_tcp_stream_enable_compress (CamelTcpStream *stream);
+
 G_END_DECLS
 
 #endif /* CAMEL_TCP_STREAM_H */
