@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,14 +42,16 @@
 #define d(x)
 #define io(x)			/* io debug */
 
-#define CAMEL_IMAP_STORE_SUMMARY_VERSION_0 (0)
+#define CAMEL_IMAP_STORE_SUMMARY_VERSION_0 (1)
 
-#define CAMEL_IMAP_STORE_SUMMARY_VERSION (0)
+#define CAMEL_IMAP_STORE_SUMMARY_VERSION (1)
 
 #define _PRIVATE(o) (((CamelImapStoreSummary *)(o))->priv)
 
 static void namespace_clear(CamelStoreSummary *s);
 
+static void namespace_free(CamelStoreSummary *s, CamelImapStoreNamespace *ns);
+
 static int summary_header_load(CamelStoreSummary *, FILE *);
 static int summary_header_save(CamelStoreSummary *, FILE *);
 
@@ -93,13 +95,20 @@ camel_imap_store_summary_init (CamelImapStoreSummary *s)
 
 	((CamelStoreSummary *)s)->store_info_size = sizeof(CamelImapStoreInfo);
 	s->version = CAMEL_IMAP_STORE_SUMMARY_VERSION;
+	s->namespaces = NULL;
+	s->namespace = NULL;
 }
 
 static void
 camel_imap_store_summary_finalise (CamelObject *obj)
 {
 	/*struct _CamelImapStoreSummaryPrivate *p;*/
-	/* CamelImapStoreSummary *s = (CamelImapStoreSummary *)obj; */
+	CamelImapStoreSummary *is = (CamelImapStoreSummary *)obj;
+	CamelStoreSummary *s = (CamelStoreSummary *) is;
+
+	if (is->namespaces)
+		namespace_clear (s);
+
 	/*p = _PRIVATE(obj);
 	  g_free(p);*/
 }
@@ -108,7 +117,7 @@ CamelType
 camel_imap_store_summary_get_type (void)
 {
 	static CamelType type = CAMEL_INVALID_TYPE;
-	
+
 	if (type == CAMEL_INVALID_TYPE) {
 		camel_imap_store_summary_parent = (CamelStoreSummaryClass *)camel_store_summary_get_type();
 		type = camel_type_register((CamelType)camel_imap_store_summary_parent, "CamelImapStoreSummary",
@@ -119,7 +128,7 @@ camel_imap_store_summary_get_type (void)
 					   (CamelObjectInitFunc) camel_imap_store_summary_init,
 					   (CamelObjectFinalizeFunc) camel_imap_store_summary_finalise);
 	}
-	
+
 	return type;
 }
 
@@ -127,7 +136,7 @@ camel_imap_store_summary_get_type (void)
  * camel_imap_store_summary_new:
  *
  * Create a new CamelImapStoreSummary object.
- * 
+ *
  * Return value: A new CamelImapStoreSummary widget.
  **/
 CamelImapStoreSummary *
@@ -140,14 +149,14 @@ camel_imap_store_summary_new (void)
 
 /**
  * camel_imap_store_summary_full_name:
- * @s: 
- * @path: 
- * 
+ * @s:
+ * @path:
+ *
  * Retrieve a summary item by full name.
  *
  * A referenced to the summary item is returned, which may be
  * ref'd or free'd as appropriate.
- * 
+ *
  * Return value: The summary item, or NULL if the @full_name name
  * is not available.
  * It must be freed using camel_store_summary_info_free().
@@ -162,7 +171,7 @@ camel_imap_store_summary_full_name(CamelImapStoreSummary *s, const char *full_na
 	for (i=0;i<count;i++) {
 		info = (CamelImapStoreInfo *)camel_store_summary_index((CamelStoreSummary *)s, i);
 		if (info) {
-			if (strcmp(info->full_name, full_name) == 0)
+			if (info->full_name && strcmp(info->full_name, full_name) == 0)
 				return info;
 			camel_store_summary_info_free((CamelStoreSummary *)s, (CamelStoreInfo *)info);
 		}
@@ -286,6 +295,40 @@ camel_imap_store_summary_path_to_full(CamelImapStoreSummary *s, const char *path
 	return f;
 }
 
+static void 
+create_parents (CamelImapStoreSummary *s, const char *path)
+{
+	gchar *p = g_strdup (path);
+	int i = 1, len = strlen (p);
+
+	while (i < len) {
+		char tmp;
+		if (p[i] == '/') {
+			tmp = p[i];
+			p[i]=0;
+			if (!(!strcmp (path, p))) {
+				CamelImapStoreInfo *info;
+
+				info = camel_imap_store_summary_full_name(s, p);
+
+				if (!info) {
+					info = (CamelImapStoreInfo *)camel_store_summary_add_from_path((CamelStoreSummary *)s, p);
+					if (info)
+						camel_store_info_set_string((CamelStoreSummary *)s, (CamelStoreInfo *)info, CAMEL_IMAP_STORE_INFO_FULL_NAME, p);
+				}
+
+				/* if (info) this is a leak, but for some reason it otherwise crashes ... :(
+				 *	camel_store_summary_info_free((CamelStoreSummary *)s, (CamelStoreInfo *)info);
+				 */
+			}
+			p[i]=tmp;
+		}
+		i++;
+	}
+	g_free (p);
+}
+
+
 CamelImapStoreInfo *
 camel_imap_store_summary_add_from_full(CamelImapStoreSummary *s, const char *full, char dir_sep)
 {
@@ -319,8 +362,8 @@ camel_imap_store_summary_add_from_full(CamelImapStoreSummary *s, const char *ful
 		} else {
 			if (full_name[len] == ns->sep)
 				len++;
-			
-			prefix = camel_imap_store_summary_full_to_path(s, full_name+len, ns->sep);
+
+			prefix = camel_imap_store_summary_full_to_path(s, full_name+len, ns->sep?ns->sep:dir_sep);
 			if (*ns->path) {
 				pathu8 = g_strdup_printf ("%s/%s", ns->path, prefix);
 				g_free (prefix);
@@ -334,7 +377,10 @@ camel_imap_store_summary_add_from_full(CamelImapStoreSummary *s, const char *ful
 		pathu8 = camel_imap_store_summary_full_to_path(s, full_name, dir_sep);
 	}
 
+	create_parents (s, pathu8);
+
 	info = (CamelImapStoreInfo *)camel_store_summary_add_from_path((CamelStoreSummary *)s, pathu8);
+
 	if (info) {
 		d(printf("  '%s' -> '%s'\n", pathu8, full_name));
 		camel_store_info_set_string((CamelStoreSummary *)s, (CamelStoreInfo *)info, CAMEL_IMAP_STORE_INFO_FULL_NAME, full_name);
@@ -344,6 +390,9 @@ camel_imap_store_summary_add_from_full(CamelImapStoreSummary *s, const char *ful
 	} else
 		d(printf("  failed\n"));
 
+	if (pathu8)
+		g_free (pathu8);
+
 	return info;
 }
 
@@ -372,7 +421,7 @@ CamelImapStoreNamespace *camel_imap_store_summary_namespace_new(CamelImapStoreSu
 	int len;
 
 	ns = g_malloc0(sizeof(*ns));
-	ns->full_name = g_strdup(full_name);
+	ns->full_name = full_name?g_strdup(full_name):g_strdup ("");
 	len = strlen(ns->full_name)-1;
 	if (len >= 0 && ns->full_name[len] == dir_sep)
 		ns->full_name[len] = 0;
@@ -391,57 +440,87 @@ CamelImapStoreNamespace *camel_imap_store_summary_namespace_new(CamelImapStoreSu
 	return ns;
 }
 
+CamelImapStoreNamespace * camel_imap_store_summary_namespace_add(CamelImapStoreSummary *s, CamelImapStoreNamespace *ns)
+{
+	gboolean add = TRUE;
+	GList *lst = s->namespaces;
+	CamelImapStoreNamespace *ret = NULL;
+
+	while (lst) {
+		CamelImapStoreNamespace *n = lst->data;
+		if (n->full_name && ns->full_name && !strcmp (n->full_name, ns->full_name)) {
+			add = FALSE;
+			ret = n;
+			break;
+		}
+		lst = lst->next;
+	}
+
+	if (add) {
+		s->namespaces = g_list_prepend (s->namespaces, ns);
+		ret = ns;
+	} else
+		namespace_free((CamelStoreSummary*)s, ns);
+
+	return ret;
+}
+
 void camel_imap_store_summary_namespace_set(CamelImapStoreSummary *s, CamelImapStoreNamespace *ns)
 {
-	d(printf("Setting namesapce to '%s' '%c' -> '%s'\n", ns->full_name, ns->sep, ns->path));
-	namespace_clear((CamelStoreSummary *)s);
 	s->namespace = ns;
-	camel_store_summary_touch((CamelStoreSummary *)s);
+	return;
 }
 
 CamelImapStoreNamespace *
 camel_imap_store_summary_namespace_find_path(CamelImapStoreSummary *s, const char *path)
 {
 	int len;
-	CamelImapStoreNamespace *ns;
+	GList *list;
+	CamelImapStoreNamespace *nsf = NULL;
 
-	/* NB: this currently only compares against 1 namespace, in future compare against others */
-	ns = s->namespace;
-	while (ns) {
+	list = s->namespaces;
+	while (list) {
+		CamelImapStoreNamespace *ns = list->data;
 		len = strlen(ns->path);
 		if (len == 0
 		    || (strncmp(ns->path, path, len) == 0
-			&& (path[len] == '/' || path[len] == 0)))
+			&& (path[len] == '/' || path[len] == 0))) {
+			nsf = ns;
 			break;
-		ns = NULL;
+		}
+		list = list->next;
 	}
 
-	/* have a default? */
-	return ns;
+	return nsf;
 }
 
 CamelImapStoreNamespace *
 camel_imap_store_summary_namespace_find_full(CamelImapStoreSummary *s, const char *full)
 {
 	int len;
-	CamelImapStoreNamespace *ns;
+	GList *list;
+	CamelImapStoreNamespace *nsf = NULL;
 
 	/* NB: this currently only compares against 1 namespace, in future compare against others */
-	ns = s->namespace;
-	while (ns) {
+	list = s->namespaces;
+	while (list) {
+		CamelImapStoreNamespace *ns = list->data;
 		len = strlen(ns->full_name);
 		d(printf("find_full: comparing namespace '%s' to name '%s'\n", ns->full_name, full));
 		if (len == 0
 		    || (strncmp(ns->full_name, full, len) == 0
-			&& (full[len] == ns->sep || full[len] == 0)))
+			&& (full[len] == ns->sep || full[len] == 0))) {
+			nsf = ns;
 			break;
-		ns = NULL;
+		}
+		list = list->next;
 	}
 
 	/* have a default? */
-	return ns;
+	return nsf;
 }
 
+
 static void
 namespace_free(CamelStoreSummary *s, CamelImapStoreNamespace *ns)
 {
@@ -454,55 +533,88 @@ static void
 namespace_clear(CamelStoreSummary *s)
 {
 	CamelImapStoreSummary *is = (CamelImapStoreSummary *)s;
+	GList *list = is->namespaces;
 
-	if (is->namespace)
-		namespace_free(s, is->namespace);
-	is->namespace = NULL;
-}
-
-static CamelImapStoreNamespace *
-namespace_load(CamelStoreSummary *s, FILE *in)
-{
-	CamelImapStoreNamespace *ns;
-	guint32 sep = '/';
-
-	ns = g_malloc0(sizeof(*ns));
-
-	if (camel_file_util_decode_string(in, &ns->path) == -1)
-		goto nserror;
-
-	if (camel_file_util_decode_string(in, &ns->full_name) == -1)
-		goto nserror;
-	
-	if (camel_file_util_decode_uint32(in, &sep) == -1)
-		goto nserror;
-
-	ns->sep = sep;
-
-	return ns;
-
- nserror:
-
-	namespace_free(s, ns);
-	return NULL;
+	while (list) {
+		CamelImapStoreNamespace *ns = list->data;
+		namespace_free(s, ns);
+		list=list->next;
+	}
+	g_list_free (is->namespaces);
+	is->namespaces = NULL;
 }
 
 static int
-namespace_save(CamelStoreSummary *s, FILE *in, CamelImapStoreNamespace *ns)
+namespace_load(CamelStoreSummary *s, FILE *in)
 {
-	if (camel_file_util_encode_string(in, ns->path) == -1)
+	CamelImapStoreSummary *is = (CamelImapStoreSummary *) s;
+
+	CamelImapStoreNamespace *ns;
+	int count = 0, i = 0;
+
+	if (is->namespaces)
+		namespace_clear (s);
+
+	if (camel_file_util_decode_uint32(in, (guint32 *) &count) == -1)
+		goto nserror;
+
+	for (i=0 ; i< count; i++) {
+		guint32 sep = '/';
+		ns = g_malloc0(sizeof(*ns));
+		if (camel_file_util_decode_string(in, &ns->path) == -1) {
+			namespace_free(s, ns);
+			goto nserror;
+		}
+		if (camel_file_util_decode_string(in, &ns->full_name) == -1) {
+			namespace_free(s, ns);
+			goto nserror;
+		}
+		if (camel_file_util_decode_uint32(in, &sep) == -1) {
+			namespace_free(s, ns);
+			goto nserror;
+		}
+		ns->sep = sep;
+		is->namespaces = g_list_prepend (is->namespaces, ns);
+	}
+
+	return 0;
+
+ nserror:
+
+	return -1;
+}
+
+static int
+namespace_save(CamelStoreSummary *s, FILE *in)
+{
+	CamelImapStoreSummary *is = (CamelImapStoreSummary *) s;
+
+	GList *list;
+
+	if (camel_file_util_encode_uint32(in, (guint32) g_list_length (is->namespaces)) == -1)
 		goto serr;
 
-	if (camel_file_util_encode_string(in, ns->full_name) == -1)
-		goto serr;
+	list = is->namespaces;
 
-	if (camel_file_util_encode_uint32(in, (guint32)ns->sep) == -1)
-		goto serr;
+	while (list) {
+		CamelImapStoreNamespace *ns = list->data;
+
+		if (camel_file_util_encode_string(in, ns->path) == -1)
+			goto serr;
+
+		if (camel_file_util_encode_string(in, ns->full_name) == -1)
+			goto serr;
+
+		if (camel_file_util_encode_uint32(in, (guint32)ns->sep) == -1)
+			goto serr;
+
+		list = list->next;
+	}
 
 	return 0;
 
  serr:
-	printf ("Error happend while writing\n");
+	printf ("Error happened while writing\n");
 	return -1;
 
 }
@@ -511,7 +623,7 @@ static int
 summary_header_load(CamelStoreSummary *s, FILE *in)
 {
 	CamelImapStoreSummary *is = (CamelImapStoreSummary *)s;
-	gint32 version, capabilities, count;
+	gint32 version, capabilities;
 
 	namespace_clear(s);
 
@@ -527,36 +639,26 @@ summary_header_load(CamelStoreSummary *s, FILE *in)
 	}
 
 	/* note file format can be expanded to contain more namespaces, but only 1 at the moment */
-	if (camel_file_util_decode_fixed_int32(in, &capabilities) == -1
-	    || camel_file_util_decode_fixed_int32(in, &count) == -1
-	    || count > 1)
+	if (camel_file_util_decode_fixed_int32(in, &capabilities) == -1)
 		return -1;
 
 	is->capabilities = capabilities;
-	if (count == 1) {
-		if ((is->namespace = namespace_load(s, in)) == NULL)
-			return -1;
-	}
 
-	return 0;
+	return namespace_load (s, in);
 }
 
 static int
 summary_header_save(CamelStoreSummary *s, FILE *out)
 {
 	CamelImapStoreSummary *is = (CamelImapStoreSummary *)s;
-	guint32 count;
-
-	count = is->namespace?1:0;
 
 	/* always write as latest version */
 	if (camel_imap_store_summary_parent->summary_header_save((CamelStoreSummary *)s, out) == -1
 	    || camel_file_util_encode_fixed_int32(out, CAMEL_IMAP_STORE_SUMMARY_VERSION) == -1
-	    || camel_file_util_encode_fixed_int32(out, is->capabilities) == -1
-	    || camel_file_util_encode_fixed_int32(out, count) == -1)	    
+	    || camel_file_util_encode_fixed_int32(out, is->capabilities) == -1)
 		return -1;
 
-	if (is->namespace && namespace_save(s, out, is->namespace) == -1)
+	if (namespace_save(s, out) == -1)
 		return -1;
 
 	return 0;
