@@ -124,6 +124,7 @@ typedef enum {
 #define IMAP_CAPABILITY_ESEARCH			(1 << 17)
 #define IMAP_CAPABILITY_CONVERT			(1 << 18)
 #define IMAP_CAPABILITY_LISTEXT			(1 << 19)
+#define IMAP_CAPABILITY_COMPRESS		(1 << 20)
 
 #define IMAP_PARAM_OVERRIDE_NAMESPACE		(1 << 0)
 #define IMAP_PARAM_CHECK_ALL			(1 << 1)
@@ -172,7 +173,7 @@ struct _CamelImapStore {
 	guint idle_sleep, getsrv_sleep;
 	gboolean courier_crap, idle_sleep_set;
 	gboolean going_online, got_online, clean_exit;
-	gboolean not_recon;
+	gboolean not_recon, needs_lsub;
 
 	struct addrinfo *addrinfo;
 };
