@@ -387,7 +387,7 @@ imap_parse_list_response (CamelImapStore *store, const char *buf, int *flags, ch
 		word = imap_next_word (word);
 		astring = imap_parse_astring (&word, &len);
 
-		if (!astring && strlen (astring) == 0)
+		if (!astring || strlen (astring) == 0)
 			return FALSE;
 
 		*folder = astring;
