@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -41,6 +41,7 @@
 #include "camel-mime-message.h"
 #include "camel-operation.h"
 #include "camel-private.h"
+#include "camel-stream-null.h"
 
 #include "camel-maildir-summary.h"
 
@@ -67,6 +68,7 @@ static void camel_maildir_summary_finalise	(CamelObject *obj);
 
 #define _PRIVATE(x) (((CamelMaildirSummary *)(x))->priv)
 
+
 struct _CamelMaildirSummaryPrivate {
 	char *current_file;
 	char *hostname;
@@ -80,7 +82,7 @@ CamelType
 camel_maildir_summary_get_type (void)
 {
 	static CamelType type = CAMEL_INVALID_TYPE;
-	
+
 	if (type == CAMEL_INVALID_TYPE) {
 		type = camel_type_register(camel_local_summary_get_type (), "CamelMaildirSummary",
 					   sizeof(CamelMaildirSummary),
@@ -90,7 +92,7 @@ camel_maildir_summary_get_type (void)
 					   (CamelObjectInitFunc)camel_maildir_summary_init,
 					   (CamelObjectFinalizeFunc)camel_maildir_summary_finalise);
 	}
-	
+
 	return type;
 }
 
@@ -156,16 +158,22 @@ camel_maildir_summary_finalise(CamelObject *obj)
  * @index: Index if one is reqiured.
  *
  * Create a new CamelMaildirSummary object.
- * 
+ *
  * Return value: A new #CamelMaildirSummary object.
  **/
-CamelMaildirSummary	*camel_maildir_summary_new(struct _CamelFolder *folder, const char *filename, const char *maildirdir, CamelIndex *index)
+CamelMaildirSummary
+*camel_maildir_summary_new(struct _CamelFolder *folder, const char *filename, const char *maildirdir, CamelIndex *index)
 {
 	CamelMaildirSummary *o = (CamelMaildirSummary *)camel_object_new(camel_maildir_summary_get_type ());
+	CamelException ex = CAMEL_EXCEPTION_INITIALISER;
 
 	((CamelFolderSummary *)o)->folder = folder;
 
 	camel_local_summary_construct((CamelLocalSummary *)o, filename, maildirdir, index);
+
+	maildir_summary_load ((CamelLocalSummary *)o, FALSE, &ex);
+	maildir_summary_check ((CamelLocalSummary *) o, NULL, &ex);
+
 	return o;
 }
 
@@ -174,47 +182,48 @@ static struct {
 	char flag;
 	guint32 flagbit;
 } flagbits[] = {
-	{ 'D', CAMEL_MESSAGE_DRAFT },
-	{ 'F', CAMEL_MESSAGE_FLAGGED },
-	/*{ 'P', CAMEL_MESSAGE_FORWARDED },*/
-	{ 'R', CAMEL_MESSAGE_ANSWERED },
-	{ 'S', CAMEL_MESSAGE_SEEN },
-	{ 'T', CAMEL_MESSAGE_DELETED },
+/* 0 */ { 'D', CAMEL_MESSAGE_DRAFT },
+/* 1 */ { 'F', CAMEL_MESSAGE_FLAGGED },
+/* -    { 'P', CAMEL_MESSAGE_FORWARDED },*/
+/* 2 */ { 'R', CAMEL_MESSAGE_ANSWERED },
+/* 3 */ { 'S', CAMEL_MESSAGE_SEEN },
+/* 4 */ { 'T', CAMEL_MESSAGE_DELETED },
 
 	/* Non-standard flags */
-	{ 'A', CAMEL_MESSAGE_ATTACHMENTS },
-	{ 'I', CAMEL_MESSAGE_PARTIAL },
-	{ 'H', CAMEL_MESSAGE_HIGH_PRIORITY },
-	{ 'N', CAMEL_MESSAGE_NORMAL_PRIORITY },
-	{ 'L', CAMEL_MESSAGE_LOW_PRIORITY },
-	{ 'O', CAMEL_MESSAGE_SUSPENDED_PRIORITY },
+/* 5 */ { 'A', CAMEL_MESSAGE_ATTACHMENTS },
+/* 6 */ { 'I', CAMEL_MESSAGE_PARTIAL },
+/* 7 */ { 'O', CAMEL_MESSAGE_SUSPENDED },
+
+	/* Non-standard priority flags */
+/* 8 */ { 'H', CAMEL_MESSAGE_HIGH_PRIORITY },
+/* 9 */ { 'N', CAMEL_MESSAGE_NORMAL_PRIORITY },
+/* 10*/ { 'L', CAMEL_MESSAGE_LOW_PRIORITY }
 };
 
 /* convert the uid + flags into a unique:info maildir format */
 char *camel_maildir_summary_info_to_name(const CamelMaildirMessageInfo *info)
 {
 	const char *uid;
-	guint32 priority_flag = 0;	/* 2 bits flags */
 	char *p, *buf;
 	int i;
 
 	uid = camel_message_info_uid (info);
 	/* TNY CHANGE: This used to he ":2,", but VFAT does not allow those characters */
-	buf = g_alloca (strlen (uid) + strlen ("_2_") +  (sizeof (flagbits) / sizeof (flagbits[0])) + 1);
-	p = buf + sprintf (buf, "%s_2_", uid);
+	buf = g_alloca (strlen (uid) + strlen ("!2,") +  (sizeof (flagbits) / sizeof (flagbits[0])) + 1);
+	p = buf + sprintf (buf, "%s!2,", uid);
+
 	for (i = 0; i < sizeof (flagbits) / sizeof (flagbits[0]); i++) {
-		/* Priority flags */
-		if ((info->info.info.flags & CAMEL_MESSAGE_HIGH_PRIORITY) != 0) {
-			priority_flag = info->info.info.flags & CAMEL_MESSAGE_HIGH_PRIORITY;
-			if (priority_flag == flagbits[i].flagbit)
+		if (i > 7) {
+			int flags = info->info.info.flags;
+			flags &= CAMEL_MESSAGE_PRIORITY_MASK;
+			if (flags == flagbits[i].flagbit)
 				*p++ = flagbits[i].flag;
-		}
-		/* Standard flags*/
-		else if (info->info.info.flags & flagbits[i].flagbit)		
+		} else if (info->info.info.flags & flagbits[i].flagbit)
 			*p++ = flagbits[i].flag;
 	}
+
 	*p = 0;
-	
+
 	return g_strdup(buf);
 }
 
@@ -222,29 +231,32 @@ char *camel_maildir_summary_info_to_name(const CamelMaildirMessageInfo *info)
 int camel_maildir_summary_name_to_info(CamelMaildirMessageInfo *info, const char *name)
 {
 	char *p, c;
-	guint32 priority_flag = 0;	/* 2 bits flags */
 	guint32 set = 0;	/* what we set */
-	/*guint32 all = 0;*/	/* all flags */
 	int i;
 
-	p = strstr(name, "_2_");
+	p = strstr (name, "!2,");
+
+	if (!p)
+		p = strstr (name, ":2,");
+
 	if (p) {
 		p+=3;
 		while ((c = *p++)) {
 			/* we could assume that the flags are in order, but its just as easy not to require */
-			for (i=0;i<sizeof(flagbits)/sizeof(flagbits[0]);i++) {
-				/* Priority flags */
-				if (info->info.info.flags & CAMEL_MESSAGE_HIGH_PRIORITY) {
-					priority_flag = info->info.info.flags & CAMEL_MESSAGE_HIGH_PRIORITY;					
-					if (flagbits[i].flag == c && (priority_flag != flagbits[i].flagbit))
+			for (i=0; i < sizeof(flagbits)/sizeof(flagbits[0]);i++) {
+
+				if (i > 7) {
+					if (c == flagbits[i].flag) {
+						set &= ~CAMEL_MESSAGE_PRIORITY_MASK;
+						set |= flagbits[i].flagbit;
+					}
+				} else {
+					if (flagbits[i].flag == c && (info->info.info.flags & flagbits[i].flagbit) == 0)
 						set |= flagbits[i].flagbit;
 				}
-				/* Standard flags */
-				else if (flagbits[i].flag == c && (info->info.info.flags & flagbits[i].flagbit) == 0) {
-					set |= flagbits[i].flagbit;
-				}
-				/*all |= flagbits[i].flagbit;*/
+
 			}
+
 		}
 
 		/* changed? */
@@ -274,15 +286,95 @@ static char *maildir_summary_encode_x_evolution(CamelLocalSummary *cls, const Ca
 /* FIXME:
    both 'new' and 'add' will try and set the filename, this is not ideal ...
 */
+
+
+static CamelMessageInfo *
+adapted_local_summary_add (CamelLocalSummary *cls, CamelMimeMessage *msg, const CamelMessageInfo *info, CamelFolderChangeInfo *ci, CamelException *ex)
+{
+	CamelLocalMessageInfo *mi;
+	char *xev;
+
+	d(printf("Adding message to summary\n"));
+
+	mi = (CamelLocalMessageInfo *)camel_folder_summary_add_from_message((CamelFolderSummary *)cls, msg);
+	if (mi) {
+		d(printf("Added, uid = %s\n", mi->uid));
+		if (info) {
+#ifdef NON_TINYMAIL_FEATURES
+
+			const CamelTag *tag = camel_message_info_user_tags(info);
+			const CamelFlag *flag = camel_message_info_user_flags(info);
+
+			while (flag) {
+				camel_message_info_set_user_flag((CamelMessageInfo *)mi, flag->name, TRUE);
+				flag = flag->next;
+			}
+
+			while (tag) {
+				camel_message_info_set_user_tag((CamelMessageInfo *)mi, tag->name, tag->value);
+				tag = tag->next;
+			}
+#endif
+			mi->info.flags = camel_message_info_flags(info);
+			/*mi->info.flags |= (camel_message_info_flags(info) & 0x1fff);*/
+			mi->info.size = ((CamelMessageInfoBase*)info)->size;
+		}
+
+		/* we need to calculate the size ourselves */
+
+		/* this is terribly slow, of course!
+
+		if (mi->info.size == 0) {
+			CamelStreamNull *sn = (CamelStreamNull *)camel_stream_null_new();
+			camel_data_wrapper_write_to_stream((CamelDataWrapper *)msg, (CamelStream *)sn);
+			mi->info.size = (sn->written);
+			camel_object_unref((CamelObject *)sn);
+		} */
+
+		mi->info.flags &= ~(CAMEL_MESSAGE_FOLDER_NOXEV|CAMEL_MESSAGE_FOLDER_FLAGGED);
+		xev = camel_local_summary_encode_x_evolution(cls, mi);
+		camel_medium_set_header((CamelMedium *)msg, "X-Evolution", xev);
+		g_free(xev);
+		if (camel_message_info_uid(mi))
+			camel_folder_change_info_add_uid(ci, camel_message_info_uid(mi));
+	} else {
+		d(printf("Failed!\n"));
+		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM_MEMORY,
+				     _("Unable to add message to summary: unknown reason"));
+	}
+	return (CamelMessageInfo *)mi;
+}
+
 static CamelMessageInfo *maildir_summary_add(CamelLocalSummary *cls, CamelMimeMessage *msg, const CamelMessageInfo *info, CamelFolderChangeInfo *changes, CamelException *ex)
 {
 	CamelMaildirMessageInfo *mi;
 
-	mi = (CamelMaildirMessageInfo *)((CamelLocalSummaryClass *) parent_class)->add(cls, msg, info, changes, ex);
+	/* mi = (CamelMaildirMessageInfo *)((CamelLocalSummaryClass *) parent_class)->add(cls, msg, info, changes, ex); */
+
+	mi = (CamelMaildirMessageInfo *) adapted_local_summary_add  (cls, msg, info, changes, ex);
+
 	if (mi) {
-		if (info) {
+		if (camel_message_info_uid(mi))
+		{
+			/* struct stat sbuf; */
+			CamelMessageInfoBase *mii = (CamelMessageInfoBase *)mi;
+			/* gchar *name = NULL; */
+
 			camel_maildir_info_set_filename(mi, camel_maildir_summary_info_to_name(mi));
 			d(printf("Setting filename to %s\n", camel_maildir_info_filename(mi)));
+
+
+			if (mii->size == 0) {
+				CamelStreamNull *sn = (CamelStreamNull *)camel_stream_null_new();
+				camel_data_wrapper_write_to_stream((CamelDataWrapper *)msg, (CamelStream *)sn);
+				mii->size = (sn->written);
+				camel_object_unref((CamelObject *)sn);
+			} 
+
+			/* name = g_strdup_printf("%s/cur/%s", cls->folder_path, camel_maildir_info_filename (mi));
+			if (stat (name, &sbuf) == 0)
+				mii->size = sbuf.st_size;
+			g_free (name); */
 		}
 	}
 
@@ -293,17 +385,22 @@ static CamelMessageInfo *message_info_new_from_header(CamelFolderSummary * s, st
 {
 	CamelMessageInfo *mi, *info;
 	CamelMaildirSummary *mds = (CamelMaildirSummary *)s;
-	CamelMaildirMessageInfo *mdi;
-	const char *uid;
 
 	mi = ((CamelFolderSummaryClass *) parent_class)->message_info_new_from_header(s, h);
+
 	/* assign the uid and new filename */
+
+	/* with maildir we know the real received date, from the filename
+	   Bulls. We just use what we have in the header of the message!
+	mdi->info.info.date_received = strtoul(camel_message_info_uid(mi), NULL, 10);  */
+
 	if (mi) {
-		mdi = (CamelMaildirMessageInfo *)mi;
+		CamelMaildirMessageInfo *mdi = (CamelMaildirMessageInfo *)mi;
+		const gchar *uid;
 
 		uid = camel_message_info_uid(mi);
-		if (uid==NULL || uid[0] == 0)
-			mdi->info.info.uid = camel_folder_summary_next_uid_string(s);
+		if (uid == NULL || uid[0] == 0)
+			mdi->info.info.uid = camel_folder_summary_next_uid_string (s);
 
 		/* handle 'duplicates' */
 		info = camel_folder_summary_uid(s, uid);
@@ -313,34 +410,12 @@ static CamelMessageInfo *message_info_new_from_header(CamelFolderSummary * s, st
 			mdi = (CamelMaildirMessageInfo *)(mi = info);
 		}
 
-		/* with maildir we know the real received date, from the filename */
-		mdi->info.info.date_received = strtoul(camel_message_info_uid(mi), NULL, 10);
 
 		if (mds->priv->current_file) {
-#if 0
-			char *p1, *p2, *p3;
-			unsigned long uid;
-#endif
 			/* if setting from a file, grab the flags from it */
 			camel_maildir_info_set_filename(mi, g_strdup(mds->priv->current_file));
 			camel_maildir_summary_name_to_info(mdi, mds->priv->current_file);
 
-#if 0
-			/* Actually, I dont think all this effort is worth it at all ... */
-
-			/* also, see if we can extract the next-id from tne name, and safe-if-fy ourselves against collisions */
-			/* we check for something.something_number.something */
-			p1 = strchr(mdi->filename, '.');
-			if (p1) {
-				p2 = strchr(p1+1, '.');
-				p3 = strchr(p1+1, '_');
-				if (p2 && p3 && p3<p2) {
-					uid = strtoul(p3+1, &p1, 10);
-					if (p1 == p2 && uid>0)
-						camel_folder_summary_set_uid(s, uid);
-				}
-			}
-#endif
 		} else {
 			/* if creating a file, set its name from the flags we have */
 			camel_maildir_info_set_filename(mdi, camel_maildir_summary_info_to_name(mdi));
@@ -352,7 +427,8 @@ static CamelMessageInfo *message_info_new_from_header(CamelFolderSummary * s, st
 }
 
 
-static void message_info_free(CamelFolderSummary *s, CamelMessageInfo *mi)
+static void
+message_info_free(CamelFolderSummary *s, CamelMessageInfo *mi)
 {
 #if !defined (DOEPOOLV) && !defined (DOESTRV)
 	CamelMaildirMessageInfo *mdi = (CamelMaildirMessageInfo *)mi;
@@ -373,16 +449,16 @@ static char *maildir_summary_next_uid_string(CamelFolderSummary *s)
 	if (mds->priv->current_file) {
 		char *cln;
 
-		cln = strchr(mds->priv->current_file, ':');
+		cln = strchr(mds->priv->current_file, '!');
+
+		if (!cln)
+			cln = strchr(mds->priv->current_file, ':');
+
 		if (cln)
 			return g_strndup(mds->priv->current_file, cln-mds->priv->current_file);
 		else
 			return g_strdup(mds->priv->current_file);
 	} else {
-		/* the first would probably work, but just to be safe, check for collisions */
-#if 0
-		return g_strdup_printf("%ld.%d_%u.%s", time(0), getpid(), camel_folder_summary_next_uid(s), mds->priv->hostname);
-#else
 		CamelLocalSummary *cls = (CamelLocalSummary *)s;
 		char *name = NULL, *uid = NULL;
 		struct stat st;
@@ -396,7 +472,7 @@ static char *maildir_summary_next_uid_string(CamelFolderSummary *s)
 				g_free(uid);
 				sleep(2);
 			}
-			uid = g_strdup_printf("%ld.%d_%u.%s", time(0), getpid(), nextuid, mds->priv->hostname);
+			uid = g_strdup_printf("%ld.%d_%u.%s", time(NULL), getpid(), nextuid, mds->priv->hostname);
 			name = g_strdup_printf("%s/tmp/%s", cls->folder_path, uid);
 			retry++;
 		} while (stat(name, &st) == 0 && retry<3);
@@ -405,7 +481,6 @@ static char *maildir_summary_next_uid_string(CamelFolderSummary *s)
 
 		g_free(name);
 		return uid;
-#endif
 	}
 }
 
@@ -446,9 +521,9 @@ static int maildir_summary_load(CamelLocalSummary *cls, int forceindex, CamelExc
 
 	dir = opendir(cur);
 	if (dir == NULL) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot open maildir directory path: %s: %s"),
-				      cls->folder_path, g_strerror (errno));
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM_IO_READ,
+			_("Cannot open maildir directory path: %s: %s"),
+			cls->folder_path, g_strerror (errno));
 		g_free(cur);
 		return -1;
 	}
@@ -463,8 +538,11 @@ static int maildir_summary_load(CamelLocalSummary *cls, int forceindex, CamelExc
 		if (!strcmp (d->d_name, "core"))
 			continue;
 
-		/* map the filename -> uid */
-		uid = strchr(d->d_name, ':');
+		uid = strchr(d->d_name, '!');
+
+		if (!uid)
+			uid = strchr(d->d_name, ':');
+
 		if (uid) {
 			int len = uid-d->d_name;
 			uid = e_mempool_alloc(pool, len+1);
@@ -475,6 +553,7 @@ static int maildir_summary_load(CamelLocalSummary *cls, int forceindex, CamelExc
 			uid = e_mempool_strdup(pool, d->d_name);
 			g_hash_table_insert(mds->priv->load_map, uid, uid);
 		}
+
 	}
 	closedir(dir);
 	g_free(cur);
@@ -488,12 +567,15 @@ static int maildir_summary_load(CamelLocalSummary *cls, int forceindex, CamelExc
 	return ret;
 }
 
-static int camel_maildir_summary_add(CamelLocalSummary *cls, const char *name, int forceindex)
+static int
+camel_maildir_summary_add (CamelLocalSummary *cls, const char *name, char *uid)
 {
 	CamelMaildirSummary *maildirs = (CamelMaildirSummary *)cls;
 	char *filename = g_strdup_printf("%s/cur/%s", cls->folder_path, name);
 	int fd;
 	CamelMimeParser *mp;
+	CamelMessageInfo *info;
+	struct stat sbuf;
 
 	d(printf("summarising: %s\n", name));
 
@@ -507,44 +589,17 @@ static int camel_maildir_summary_add(CamelLocalSummary *cls, const char *name, i
 	camel_mime_parser_scan_from(mp, FALSE);
 	camel_mime_parser_init_with_fd(mp, fd);
 	maildirs->priv->current_file = (char *)name;
-	camel_folder_summary_add_from_parser((CamelFolderSummary *)maildirs, mp);
+	info = camel_folder_summary_add_from_parser((CamelFolderSummary *)maildirs, mp, uid);
+
+	if (stat (filename, &sbuf) == 0)
+		((CamelMessageInfoBase *)info)->size = sbuf.st_size;
+
 	camel_object_unref((CamelObject *)mp);
 	maildirs->priv->current_file = NULL;
 	g_free(filename);
 	return 0;
 }
 
-struct _remove_data {
-	CamelLocalSummary *cls;
-	CamelFolderChangeInfo *changes;
-};
-
-static void
-remove_summary(char *key, CamelMessageInfo *info, struct _remove_data *rd)
-{
-	d(printf("removing message %s from summary\n", key));
-	if (rd->cls->index)
-		camel_index_delete_name(rd->cls->index, camel_message_info_uid(info));
-	if (rd->changes)
-		camel_folder_change_info_remove_uid(rd->changes, key);
-	camel_folder_summary_remove((CamelFolderSummary *)rd->cls, info);
-	camel_message_info_free(info);
-}
-
-static int
-sort_receive_cmp(const void *ap, const void *bp)
-{
-	const CamelMaildirMessageInfo
-		*a = *((CamelMaildirMessageInfo **)ap),
-		*b = *((CamelMaildirMessageInfo **)bp);
-
-	if (a->info.info.date_received < b->info.info.date_received)
-		return -1;
-	else if (a->info.info.date_received > b->info.info.date_received)
-		return 1;
-
-	return 0;
-}
 
 static int
 maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changes, CamelException *ex)
@@ -553,54 +608,39 @@ maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changes, Ca
 	struct dirent *d;
 	char *p;
 	CamelMessageInfo *info;
-	CamelMaildirMessageInfo *mdi;
 	CamelFolderSummary *s = (CamelFolderSummary *)cls;
-	GHashTable *left;
-	int i, count, total;
-	int forceindex;
 	char *new, *cur;
 	char *uid;
-	struct _remove_data rd = { cls, changes };
+	int count, total;
 
 	new = g_strdup_printf("%s/new", cls->folder_path);
 	cur = g_strdup_printf("%s/cur", cls->folder_path);
 
-	d(printf("checking summary ...\n"));
-
 	camel_operation_start(NULL, _("Checking folder consistency"));
 
 	/* scan the directory, check for mail files not in the index, or index entries that
 	   no longer exist */
 	dir = opendir(cur);
 	if (dir == NULL) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot open maildir directory path: %s: %s"),
-				      cls->folder_path, g_strerror (errno));
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM_IO_READ,
+			_("Cannot open maildir directory path: %s: %s"),
+			cls->folder_path, g_strerror (errno));
 		g_free(cur);
 		g_free(new);
 		camel_operation_end(NULL);
 		return -1;
 	}
 
-	/* keeps track of all uid's that have not been processed */
-	left = g_hash_table_new(g_str_hash, g_str_equal);
-	count = camel_folder_summary_count((CamelFolderSummary *)cls);
-	forceindex = count == 0;
-	for (i=0;i<count;i++) {
-		info = camel_folder_summary_index((CamelFolderSummary *)cls, i);
-		if (info) {
-			g_hash_table_insert(left, (char *)camel_message_info_uid(info), info);
-		}
-	}
-
-	/* joy, use this to pre-count the total, so we can report progress meaningfully */
 	total = 0;
 	count = 0;
 	while ( (d = readdir(dir)) )
 		total++;
 	rewinddir(dir);
 
+	camel_folder_summary_prepare_hash ((CamelFolderSummary *)cls);
+
 	while ( (d = readdir(dir)) ) {
+		gboolean mfree = FALSE;
 
 		camel_operation_progress(NULL, count, total);
 		count++;
@@ -610,62 +650,40 @@ maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changes, Ca
 		if (p[0] == '.')
 			continue;
 
-		/* map the filename -> uid */
-		uid = strchr(d->d_name, ':');
-		if (uid)
-			uid = g_strndup(d->d_name, uid-d->d_name);
-		else
-			uid = g_strdup(d->d_name);
-		
-		info = g_hash_table_lookup(left, uid);
-		if (info) {
-			camel_message_info_free(info);
-			g_hash_table_remove(left, uid);
-		}
+		if (!strcmp (d->d_name, "core"))
+			continue;
 
-		info = camel_folder_summary_uid((CamelFolderSummary *)cls, uid);
+		/* map the filename -> uid */
+		uid = strchr(d->d_name, '!');
+
+		if (!uid)
+			uid = strchr(d->d_name, ':');
+
+		if (uid) {
+			uid =  g_strndup(d->d_name, uid - d->d_name);
+			mfree = TRUE;
+		} else
+			uid = d->d_name;
+
+
+		info = camel_folder_summary_uid ((CamelFolderSummary *)cls, uid);
 		if (info == NULL) {
 			/* must be a message incorporated by another client, this is not a 'recent' uid */
-			if (camel_maildir_summary_add(cls, d->d_name, forceindex) == 0)
+			if (camel_maildir_summary_add (cls, d->d_name, uid) == 0)
 				if (changes)
 					camel_folder_change_info_add_uid(changes, uid);
+
+
 		} else {
-			const char *filename;
-
-			/*if (cls->index && (!camel_index_has_name(cls->index, uid))) {
-				 message_info_new will handle duplicates 
-				camel_maildir_summary_add(cls, d->d_name, forceindex);
-			}*/
-
-			mdi = (CamelMaildirMessageInfo *)info;
-			filename = camel_maildir_info_filename(mdi);
-			/* TODO: only store the extension in the mdi->filename struct, not the whole lot */
-			if (filename == NULL || strcmp(filename, d->d_name) != 0) {
-#ifdef DOESTRV
-#warning "cannot modify the estrv after its been setup, for mt-safe code"
-				CAMEL_SUMMARY_LOCK(s, summary_lock);
-				/* need to update the summary hash ref */
-				g_hash_table_remove(s->messages_uid, camel_message_info_uid(info));
-				info->strings = e_strv_set_ref(info->strings, CAMEL_MAILDIR_INFO_FILENAME, d->d_name);
-				info->strings = e_strv_pack(info->strings);
-				g_hash_table_insert(s->messages_uid, (char *)camel_message_info_uid(info), info);
-				CAMEL_SUMMARY_UNLOCK(s, summary_lock);
-#else
-# ifdef DOEPOOLV
-				info->strings = e_poolv_set(info->strings, CAMEL_MAILDIR_INFO_FILENAME, d->d_name, FALSE);
-# else	
-				g_free(mdi->filename);
-				mdi->filename = g_strdup(d->d_name);
-# endif
-#endif
-			}
+			// CamelMaildirMessageInfo *mdi = (CamelMaildirMessageInfo *) info;
+			// mdi->filename = g_strdup (d->d_name);
 			camel_message_info_free(info);
 		}
-		g_free(uid);
+
+		if (mfree)
+			g_free(uid);
 	}
 	closedir(dir);
-	g_hash_table_foreach(left, (GHFunc)remove_summary, &rd);
-	g_hash_table_destroy(left);
 
 	camel_operation_end(NULL);
 
@@ -680,12 +698,10 @@ maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changes, Ca
 			total++;
 		rewinddir(dir);
 
-		camel_folder_summary_prepare_hash (s);
-
 		while ( (d = readdir(dir)) ) {
 			char *name, *newname, *destname, *destfilename;
 			char *src, *dest;
-			
+
 			camel_operation_progress(NULL, count, total);
 			count++;
 
@@ -693,6 +709,9 @@ maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changes, Ca
 			if (name[0] == '.')
 				continue;
 
+			if (!strcmp (d->d_name, "core"))
+				continue;
+
 			/* already in summary?  shouldn't happen, but just incase ... */
 			if ((info = camel_folder_summary_uid((CamelFolderSummary *)cls, name))) {
 				camel_message_info_free(info);
@@ -704,13 +723,13 @@ maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changes, Ca
 
 			/* copy this to the destination folder, use 'standard' semantics for maildir info field */
 			src = g_strdup_printf("%s/%s", new, name);
-			destfilename = g_strdup_printf("%s_2_", destname);
+			destfilename = g_strdup_printf("%s!2,", destname);
 			dest = g_strdup_printf("%s/%s", cur, destfilename);
 
 			/* FIXME: This should probably use link/unlink */
 
 			if (rename(src, dest) == 0) {
-				camel_maildir_summary_add(cls, destfilename, forceindex);
+				camel_maildir_summary_add (cls, destfilename, destname);
 				if (changes) {
 					camel_folder_change_info_add_uid(changes, destname);
 					camel_folder_change_info_recent_uid(changes, destname);
@@ -726,7 +745,6 @@ maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changes, Ca
 			g_free(src);
 			g_free(dest);
 		}
-		camel_folder_summary_kill_hash (s);
 
 		camel_operation_end(NULL);
 	}
@@ -735,16 +753,15 @@ maildir_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changes, Ca
 	g_free(new);
 	g_free(cur);
 
-	/* sort the summary based on receive time, since the directory order is not useful */
-	/* This is unneeded, the sorting order is irrelevant
+	camel_folder_summary_kill_hash ((CamelFolderSummary *)cls);
 
-	CAMEL_SUMMARY_LOCK(s, summary_lock);
-	qsort(s->messages->pdata, s->messages->len, sizeof(CamelMessageInfo *), sort_receive_cmp);
-	CAMEL_SUMMARY_UNLOCK(s, summary_lock); */
+	camel_folder_summary_save ((CamelFolderSummary *) cls, ex);
 
 	return 0;
 }
 
+
+
 /* sync the summary with the ondisk files. */
 static int
 maildir_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChangeInfo *changes, CamelException *ex)
@@ -752,9 +769,6 @@ maildir_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChange
 	int count, i;
 	CamelMessageInfo *info;
 	CamelMaildirMessageInfo *mdi;
-#ifdef DOESTRV
-	CamelFolderSummary *s = (CamelFolderSummary *)cls;
-#endif
 	char *name;
 	struct stat st;
 
@@ -772,7 +786,7 @@ maildir_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChange
 		info = camel_folder_summary_index((CamelFolderSummary *)cls, i);
 		mdi = (CamelMaildirMessageInfo *)info;
 		if (mdi && (mdi->info.info.flags & CAMEL_MESSAGE_DELETED) && expunge) {
-			name = g_strdup_printf("%s/cur/%s", cls->folder_path, camel_maildir_info_filename(mdi));
+			name = camel_maildir_get_filename (cls->folder_path, mdi, info->uid);
 			d(printf("deleting %s\n", name));
 			if (unlink(name) == 0 || errno==ENOENT) {
 
@@ -793,43 +807,24 @@ maildir_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChange
 			/* probably should all go in the filename? */
 
 			/* have our flags/ i.e. name changed? */
-			if (strcmp(newname, camel_maildir_info_filename(mdi))) {
+			if (newname && strcmp(newname, camel_maildir_info_filename (mdi))) {
 				name = g_strdup_printf("%s/cur/%s", cls->folder_path, camel_maildir_info_filename(mdi));
 				dest = g_strdup_printf("%s/cur/%s", cls->folder_path, newname);
 				rename(name, dest);
 				if (stat(dest, &st) == -1) {
 					/* we'll assume it didn't work, but dont change anything else */
 					g_free(newname);
-				} else {
-					/* TODO: If this is made mt-safe, then this code could be a problem, since
-					   the estrv is being modified.
-					   Sigh, this may mean the maildir name has to be cached another way */
-#ifdef DOESTRV
-#warning "cannot modify the estrv after its been setup, for mt-safe code"
-					CAMEL_SUMMARY_LOCK(s, summary_lock);
-					/* need to update the summary hash ref */
-					g_hash_table_remove(s->messages_uid, camel_message_info_uid(info));
-					info->strings = e_strv_set_ref_free(info->strings, CAMEL_MAILDIR_INFO_FILENAME, newname);
-					info->strings = e_strv_pack(info->strings);
-					g_hash_table_insert(s->messages_uid, (char *)camel_message_info_uid(info), info);
-					CAMEL_SUMMARY_UNLOCK(s, summary_lock);
-#else
-# ifdef DOEPOOLV
-					info->strings = e_poolv_set(info->strings, CAMEL_MAILDIR_INFO_FILENAME, newname, TRUE);
-# else
-					g_free(mdi->filename);
-					mdi->filename = newname;
-# endif
-#endif
+					newname = NULL;
 				}
 				g_free(name);
 				g_free(dest);
-			} else {
+			} 
+
+			if (newname)
 				g_free(newname);
-			}
 
 			/* strip FOLDER_MESSAGE_FLAGED, etc */
-			mdi->info.info.flags &= 0xffff;
+			mdi->info.info.flags &= 0x1fff;
 		}
 		camel_message_info_free(info);
 	}
@@ -839,3 +834,54 @@ maildir_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChange
 	return ((CamelLocalSummaryClass *)parent_class)->sync(cls, expunge, changes, ex);
 }
 
+
+char *camel_maildir_get_filename (const gchar *fpath, CamelMaildirMessageInfo *mdi, const gchar *uid)
+{
+	char *filen = mdi->filename;
+	gboolean nfree = FALSE;
+	char *name;
+
+	if (!filen || strlen (filen) == 0) {
+		filen = camel_maildir_summary_info_to_name (mdi);
+		if (filen)
+			nfree = TRUE;
+	}
+
+	if (filen)
+		name = g_strdup_printf("%s/cur/%s", fpath, filen);
+	else 
+		name = (char *) uid;
+
+	if (nfree)
+		g_free (filen);
+
+	if (!g_file_test (name, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) {
+		gint len = strlen (name);
+		gchar *cur = g_strdup_printf("%s/cur", fpath);
+		DIR *dir; struct dirent *d;
+		dir = opendir(cur);
+		
+		if (dir) {
+			while ( (d = readdir(dir)) ) {
+				char *nname = g_strdup_printf ("%s/%s", cur, d->d_name);
+				char *ptr = strstr (nname, "!");
+
+				if (!ptr)
+					strstr (nname, ":");
+				if (ptr)
+					len = (ptr - nname);
+
+				if (!g_ascii_strncasecmp (nname, name, len)) {
+					g_free (name);
+					name = nname;
+					break;
+				}
+				g_free (nname);
+			}
+			closedir(dir);
+		}
+		g_free (cur);
+	}
+
+	return name;
+}
