@@ -81,12 +81,26 @@ static int pop3_get_local_size (CamelFolder *folder);
 
 static void pop3_delete_attachments (CamelFolder *folder, const char *uid);
 
+static void
+check_dir (CamelPOP3Store *store, CamelFolder *folder)
+{
+	if (!store && !folder)
+		return;
+
+	if (!store && folder)
+		store = (CamelPOP3Store *) folder->parent_store;
+
+	if (!g_file_test (store->storage_path, G_FILE_TEST_IS_DIR)) 
+		g_mkdir_with_parents (store->storage_path, S_IRUSR | S_IWUSR | S_IXUSR);
+}
 
 static void
 pop3_finalize (CamelObject *object)
 {
 	CamelFolder *folder = (CamelFolder *) object;
 
+	check_dir (NULL, folder);
+
 	camel_folder_summary_save (folder->summary, NULL);
 
 	return;
@@ -112,9 +126,10 @@ camel_pop3_folder_new (CamelStore *parent, CamelException *ex)
 	folder = CAMEL_FOLDER (camel_object_new (CAMEL_POP3_FOLDER_TYPE));
 	pop3_folder = CAMEL_POP3_FOLDER (folder);
 
-
 	camel_folder_construct (folder, parent, "inbox", "inbox");
 
+	check_dir (p3store, NULL);
+
 	summary_file = g_strdup_printf ("%s/summary.mmap", p3store->storage_path);
 	folder->summary = camel_folder_summary_new (folder);
 	folder->summary->set_extra_flags_func = camel_pop3_summary_set_extra_flags;
@@ -155,6 +170,7 @@ pop3_get_local_size (CamelFolder *folder)
 {
 	CamelPOP3Store *p3store = CAMEL_POP3_STORE (folder->parent_store);
 	int msize = 0;
+	check_dir (p3store, NULL);
 	camel_du (p3store->storage_path , &msize);
 	return msize;
 }
@@ -182,7 +198,7 @@ cmd_builduid(CamelPOP3Engine *pe, CamelPOP3Stream *stream, void *data)
 	case CAMEL_MIME_PARSER_STATE_MULTIPART:
 		h = camel_mime_parser_headers_raw(mp);
 		while (h) {
-			if (g_ascii_strcasecmp(h->name, "status") != 0
+			if (h->name && g_ascii_strcasecmp(h->name, "status") != 0
 			    && g_ascii_strcasecmp(h->name, "x-status") != 0) {
 				md5_update(&md5, (const guchar*) h->name, strlen(h->name));
 				md5_update(&md5, (const guchar*) h->value, strlen(h->value));
@@ -333,6 +349,8 @@ pop3_refresh_info (CamelFolder *folder, CamelException *ex)
 
 	/* Update the summary.mmap file */
 
+	check_dir (pop3_store, NULL);
+
 	camel_folder_summary_prepare_hash (folder->summary);
 
 	camel_pop3_logbook_open (pop3_store->book);
@@ -356,26 +374,41 @@ pop3_refresh_info (CamelFolder *folder, CamelException *ex)
 				continue;
 			}
 
-			if (pop3_store->engine && pop3_store->engine->capa & CAMEL_POP3_CAP_TOP)
+			if (pop3_store->engine && pop3_store->engine->capa & CAMEL_POP3_CAP_TOP) {
 				msg = pop3_get_top (folder, fi->uid, NULL);
-			else if (pop3_store->engine)
-				msg = pop3_get_message (folder, fi->uid, CAMEL_FOLDER_RECEIVE_PARTIAL, -1, NULL);
+				if (!msg && (!(pop3_store->engine && pop3_store->engine->capa & CAMEL_POP3_CAP_TOP)))
+					msg = pop3_get_message (folder, fi->uid, CAMEL_FOLDER_RECEIVE_FULL, -1, NULL);
+			} else if (pop3_store->engine)
+				msg = pop3_get_message (folder, fi->uid, CAMEL_FOLDER_RECEIVE_FULL, -1, NULL);
 
 			g_static_rec_mutex_unlock (pop3_store->eng_lock);
 
 			if (msg)
 			{
-				mi = (CamelMessageInfoBase*) camel_folder_summary_uid (folder->summary, fi->uid);
+
 				if (mi) {
-				    mi->size = (fi->size);
-				   /* TNY TODO: This is a hack! But else we need to parse
-				    * BODYSTRUCTURE (and I'm lazy). It needs fixing though. */
-				    if (mi->size > 102400)
-					mi->flags |= CAMEL_MESSAGE_ATTACHMENTS;
-				    /* ... it does */
-				    camel_message_info_free (mi);
+					struct _camel_header_raw *h;
+					
+					h = ((CamelMimePart *)msg)->headers;
+					if (camel_header_raw_find(&h, "X-MSMail-Priority", NULL) &&
+					    !camel_header_raw_find(&h, "X-MS-Has-Attach", NULL)) {
+						
+						((CamelMessageInfoBase *)mi)->flags &= ~CAMEL_MESSAGE_ATTACHMENTS;
+
+					} else if (!camel_header_raw_find (&h, "X-MS-Has-Attach", NULL)) {
+						
+						mi = (CamelMessageInfoBase*) camel_folder_summary_uid (folder->summary, fi->uid);
+						mi->size = (fi->size);
+						/* TNY TODO: This is a hack! But else we need to parse
+						 * BODYSTRUCTURE (and I'm lazy). It needs fixing though. */
+						if (mi->size > 102400)
+							mi->flags |= CAMEL_MESSAGE_ATTACHMENTS;
+						/* ... it does */
+						camel_message_info_free (mi);
+					}
 				}
 
+
 				camel_object_unref (CAMEL_OBJECT (msg));
 
 				if (!changes)
@@ -455,7 +488,7 @@ pop3_refresh_info (CamelFolder *folder, CamelException *ex)
 		for (t=0; t < pop3_store->uids->len; t++)
 		{
 			CamelPOP3FolderInfo *fi2 = pop3_store->uids->pdata[t];
-			if (fi2 && fi2->uid && !strcmp (fi2->uid, info->uid)) {
+			if (info && fi2 && fi2->uid && info->uid && !strcmp (fi2->uid, info->uid)) {
 				found = TRUE;
 				break;
 			}
@@ -463,7 +496,7 @@ pop3_refresh_info (CamelFolder *folder, CamelException *ex)
 
 		if (!found) {
 			/* Removed remotely (only if not cached locally) */
-			if (!camel_data_cache_exists (pop3_store->cache, "cache", info->uid, NULL)) {
+			if (info && info->uid && !camel_data_cache_exists (pop3_store->cache, "cache", info->uid, NULL)) {
 
 				camel_message_info_ref (info);
 				deleted = g_list_prepend (deleted, info);
@@ -499,6 +532,8 @@ pop3_refresh_info (CamelFolder *folder, CamelException *ex)
 
 
 	camel_pop3_logbook_close (pop3_store->book);
+
+	check_dir (pop3_store, NULL);
 	camel_folder_summary_save (folder->summary, ex);
 	camel_folder_summary_kill_hash (folder->summary);
 
@@ -529,6 +564,8 @@ pop3_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 	pop3_folder = CAMEL_POP3_FOLDER (folder);
 	pop3_store = CAMEL_POP3_STORE (folder->parent_store);
 
+	check_dir (pop3_store, NULL);
+
 	if (!pop3_store->delete_after && !expunge)
 		return;
 
@@ -583,7 +620,7 @@ pop3_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 			}
 
 			g_unlink (expunged_path);
-			cmd = camel_pop3_engine_command_new(pop3_store->engine, 0, NULL, NULL, "DELE %u\r\n", info->uid);
+			cmd = camel_pop3_engine_command_new(pop3_store->engine, 0, NULL, NULL, "DELE %s\r\n", info->uid);
 			while (camel_pop3_engine_iterate(pop3_store->engine, cmd) > 0);
 			if (pop3_store->cache && info->uid)
 				camel_data_cache_remove(pop3_store->cache, "cache", info->uid, NULL);
@@ -602,7 +639,7 @@ pop3_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 			for (t=0; t < pop3_store->uids->len; t++)
 			{
 				CamelPOP3FolderInfo *fi2 = pop3_store->uids->pdata[t];
-				if (fi2 && fi2->uid && !strcmp (fi2->uid, info->uid)) {
+				if (info && fi2 && fi2->uid && info->uid && !strcmp (fi2->uid, info->uid)) {
 					found = TRUE;
 					break;
 				}
@@ -610,7 +647,7 @@ pop3_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 
 			if (!found) {
 				/* Removed remotely (only if not cached locally) */
-				if (!camel_data_cache_exists (pop3_store->cache, "cache", info->uid, NULL)) {
+				if (info && info->uid && !camel_data_cache_exists (pop3_store->cache, "cache", info->uid, NULL)) {
 					camel_message_info_ref (info);
 					deleted = g_list_prepend (deleted, info);
 				}
@@ -650,14 +687,57 @@ pop3_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 }
 
 
+static gboolean
+pop3_get_message_time_from_cache (CamelFolder *folder, const char *uid, time_t *message_time)
+{
+	CamelPOP3Store *pop3_store;
+	CamelStream *stream = NULL;
+	char buffer[1];
+	gboolean res = FALSE;
+
+	g_return_val_if_fail (folder != NULL, FALSE);
+	g_return_val_if_fail (uid != NULL, FALSE);
+	g_return_val_if_fail (message_time != NULL, FALSE);
+
+	pop3_store = CAMEL_POP3_STORE (folder->parent_store);
+
+	g_return_val_if_fail (pop3_store->cache != NULL, FALSE);
+
+	if ((stream = camel_data_cache_get (pop3_store->cache, "cache", uid, NULL)) != NULL
+	    && camel_stream_read (stream, buffer, 1) == 1
+	    && buffer[0] == '#') {
+		CamelMimeMessage *message;
+
+		camel_object_ref ((CamelObject *)stream);
+
+		message = camel_mime_message_new ();
+		if (camel_data_wrapper_construct_from_stream ((CamelDataWrapper *)message, stream) == -1) {
+			g_warning (_("Cannot get message %s: %s"), uid, g_strerror (errno));
+			camel_object_unref ((CamelObject *)message);
+			message = NULL;
+		}
+
+		if (message) {
+			res = TRUE;
+			*message_time = message->date + message->date_offset;
+
+			camel_object_unref ((CamelObject *)message);
+		}
+
+		camel_object_unref ((CamelObject *)stream);
+	}
+
+	return res;
+}
+
 int
-camel_pop3_delete_old(CamelFolder *folder, int days_to_delete, CamelException *ex)
+camel_pop3_delete_old(CamelFolder *folder, int days_to_delete,	CamelException *ex)
 {
 	CamelPOP3Folder *pop3_folder;
 	CamelPOP3FolderInfo *fi;
 	int i;
 	CamelPOP3Store *pop3_store;
-	time_t temp;
+	time_t temp, message_time;
 
 	pop3_folder = CAMEL_POP3_FOLDER (folder);
 	pop3_store = CAMEL_POP3_STORE (CAMEL_FOLDER(pop3_folder)->parent_store);
@@ -676,40 +756,42 @@ camel_pop3_delete_old(CamelFolder *folder, int days_to_delete, CamelException *e
 		}
 	}
 
+	d(printf("%s(%d): pop3_folder->uids->len=[%d]\n", __FILE__, __LINE__, pop3_folder->uids->len));
 	for (i = 0; i < pop3_store->uids->len; i++) {
-		CamelMimeMessage *message = NULL;
-
 		fi = pop3_store->uids->pdata[i];
 
-		if (pop3_store->cache && fi->uid && !camel_data_cache_is_partial(pop3_store->cache, "cache", fi->uid))
-			message = pop3_get_message (folder, fi->uid, CAMEL_FOLDER_RECEIVE_FULL, -1, ex);
-		else if (fi->uid)
-			message = pop3_get_message (folder, fi->uid, CAMEL_FOLDER_RECEIVE_PARTIAL, -1, ex);
+		d(printf("%s(%d): fi->uid=[%s]\n", __FILE__, __LINE__, fi->uid));
+		if (pop3_get_message_time_from_cache (folder, fi->uid, &message_time)) {
+			double time_diff = difftime(temp,message_time);
+			int day_lag = time_diff/(60*60*24);
 
-		if (message)
-		{
-		    time_t message_time = message->date + message->date_offset;
-		    double time_diff = difftime(temp,message_time);
-		    int day_lag = time_diff/(60*60*24);
-		    if (day_lag > days_to_delete)
-		    {
-			if (fi->cmd)
+			d(printf("%s(%d): message_time= [%ld]\n", __FILE__, __LINE__, message_time));
+			d(printf("%s(%d): day_lag=[%d] \t days_to_delete=[%d]\n",
+				 __FILE__, __LINE__, day_lag, days_to_delete));
+
+			if( day_lag > days_to_delete)
 			{
-			    while (camel_pop3_engine_iterate(pop3_store->engine, fi->cmd) > 0)
-				    ;
-			    camel_pop3_engine_command_free(pop3_store->engine, fi->cmd);
-			    fi->cmd = NULL;
+				if (fi->cmd) {
+					while (camel_pop3_engine_iterate(pop3_store->engine, fi->cmd) > 0) {
+						; /* do nothing - iterating until end */
+					}
+
+					camel_pop3_engine_command_free(pop3_store->engine, fi->cmd);
+					fi->cmd = NULL;
+				}
+				d(printf("%s(%d): Deleting old messages\n", __FILE__, __LINE__));
+				fi->cmd = camel_pop3_engine_command_new(pop3_store->engine,
+									0,
+									NULL,
+									NULL,
+									"DELE %u\r\n",
+									fi->id);
+				/* also remove from cache */
+				if (pop3_store->cache && fi->uid) {
+					camel_data_cache_remove(pop3_store->cache, "cache", fi->uid, NULL);
+				}
 			}
-
-			fi->cmd = camel_pop3_engine_command_new(pop3_store->engine, 0, NULL, NULL, "DELE %u\r\n", fi->id);
-			/* also remove from cache */
-			if (pop3_store->cache && fi->uid)
-				camel_data_cache_remove(pop3_store->cache, "cache", fi->uid, NULL);
-		    }
-
-		    camel_object_unref (CAMEL_OBJECT (message));
 		}
-
 	}
 
 	for (i = 0; i < pop3_store->uids->len; i++) {
@@ -726,13 +808,14 @@ camel_pop3_delete_old(CamelFolder *folder, int days_to_delete, CamelException *e
 
 	camel_operation_end(NULL);
 
-	/* camel_pop3_store_expunge (pop3_store, ex); */
+	camel_pop3_store_expunge (pop3_store, ex);
 
 	g_static_rec_mutex_unlock (pop3_store->eng_lock);
 
 	return 0;
 }
 
+
 static int
 cmd_tocache(CamelPOP3Engine *pe, CamelPOP3Stream *stream, void *data)
 {
@@ -793,6 +876,67 @@ done:
 }
 
 
+
+
+static int
+cmd_totop(CamelPOP3Engine *pe, CamelPOP3Stream *stream, void *data)
+{
+	CamelPOP3Store *tstore = (CamelPOP3Store *) pe->store;
+	CamelPOP3FolderInfo *fi = data;
+	char buffer[2048];
+	int w = 0, n;
+	int retval = 1;
+
+	/* What if it fails? */
+	/* We write an '*' to the start of the stream to say its not complete yet */
+	/* This should probably be part of the cache code */
+
+	if ((n = camel_stream_write(fi->stream, "*", 1)) == -1)
+		goto done;
+
+	while ((n = camel_stream_read((CamelStream *)stream, buffer, sizeof(buffer))) > 0) {
+		n = camel_stream_write(fi->stream, buffer, n);
+		if (n == -1)
+			break;
+
+		/* heuristics */
+		if (camel_strstrcase (buffer, "Content-Disposition: attachment") != NULL)
+			fi->has_attachments = TRUE;
+		else if (camel_strstrcase (buffer, "filename=") != NULL &&
+			 strchr (buffer, '.'))
+			fi->has_attachments = TRUE;
+		else if (camel_strstrcase (buffer, "Content-Type: message/rfc822") != NULL)
+			fi->has_attachments = TRUE;
+
+		w += n;
+		if (w > fi->size)
+			w = fi->size;
+/*		if (fi->size != 0)
+			camel_operation_progress(NULL, w , fi->size);*/
+	}
+
+	/* it all worked, output a '#' to say we're a-ok */
+	if (n != -1) {
+		camel_stream_reset(fi->stream);
+		n = camel_stream_write(fi->stream, "#", 1);
+
+		camel_data_cache_set_partial (tstore->cache, "cache", fi->uid, FALSE);
+	}
+done:
+	if (n == -1) {
+		fi->err = errno;
+		retval = 0;
+		g_warning("POP3 retrieval failed: %s", strerror(errno));
+	} else {
+		fi->err = 0;
+	}
+
+	camel_object_unref((CamelObject *)fi->stream);
+	fi->stream = NULL;
+
+	return retval;
+}
+
 static int
 cmd_tocache_partial (CamelPOP3Engine *pe, CamelPOP3Stream *stream, void *data)
 {
@@ -816,15 +960,15 @@ cmd_tocache_partial (CamelPOP3Engine *pe, CamelPOP3Stream *stream, void *data)
 			continue;
 
 		/* heuristics */
-		if (camel_strstrcase ((const char *) buffer, "Content-Disposition: attachment") != NULL)
+		if (buffer && camel_strstrcase ((const char *) buffer, "Content-Disposition: attachment") != NULL)
 			fi->has_attachments = TRUE;
-		else if (camel_strstrcase ((const char *) buffer, "filename=") != NULL &&
+		else if (buffer && camel_strstrcase ((const char *) buffer, "filename=") != NULL &&
 			 strchr ((const char *) buffer, '.'))
 			fi->has_attachments = TRUE;
-		else if (camel_strstrcase ((const char *) buffer, "Content-Type: message/rfc822") != NULL)
+		else if (buffer && camel_strstrcase ((const char *) buffer, "Content-Type: message/rfc822") != NULL)
 			fi->has_attachments = TRUE;
 
-		if (boundary == NULL)
+		if (boundary == NULL && buffer)
 		{
 			   CamelContentType *ct = NULL;
 			   const char *bound=NULL;
@@ -843,7 +987,7 @@ cmd_tocache_partial (CamelPOP3Engine *pe, CamelPOP3Stream *stream, void *data)
 				if (bound && strlen (bound) > 0)
 					boundary = g_strdup (bound);
 			   }
-		} else if (strstr ((const char*) buffer, (const char*) boundary))
+		} else if (buffer && strstr ((const char*) buffer, (const char*) boundary))
 		{
 			if (occurred)
 			{
@@ -857,7 +1001,7 @@ cmd_tocache_partial (CamelPOP3Engine *pe, CamelPOP3Stream *stream, void *data)
 			occurred = TRUE;
 		}
 
-		if (!theend)
+		if (!theend && buffer)
 		{
 		    n = camel_stream_write(fi->stream, (const char*) buffer, len);
 		    if (n == -1 || camel_stream_write(fi->stream, "\n", 1) == -1)
@@ -939,6 +1083,8 @@ pop3_get_message (CamelFolder *folder, const char *uid, CamelFolderReceiveType t
 
 	pop3_debug ("%s requested\n", uid);
 
+	check_dir (pop3_store, NULL);
+
 	stream = camel_data_cache_get (pop3_store->cache, "cache", uid, NULL);
 	if (stream)
 	{
@@ -993,10 +1139,12 @@ pop3_get_message (CamelFolder *folder, const char *uid, CamelFolderReceiveType t
 
 			camel_pop3_store_destroy_lists (pop3_store);
 			pcl = camel_pop3_engine_command_new(pop3_store->engine, CAMEL_POP3_COMMAND_MULTI, cmd_list, folder, "LIST\r\n");
-			if (pop3_store->engine->capa & CAMEL_POP3_CAP_UIDL)
+			if (pop3_store->engine && pop3_store->engine->capa & CAMEL_POP3_CAP_UIDL)
 				pcu = camel_pop3_engine_command_new(pop3_store->engine, CAMEL_POP3_COMMAND_MULTI, cmd_uidl, folder, "UIDL\r\n");
-			while ((i = camel_pop3_engine_iterate(pop3_store->engine, NULL)) > 0)
+			if (pop3_store->engine) {
+			  while ((i = camel_pop3_engine_iterate(pop3_store->engine, NULL)) > 0)
 				;
+			}
 			fi = g_hash_table_lookup(pop3_store->uids_uid, uid);
 		}
 		g_static_rec_mutex_unlock (pop3_store->eng_lock);
@@ -1139,6 +1287,7 @@ rfail:
 		if (pop3_store->cache == NULL
 		    || (stream = camel_data_cache_add(pop3_store->cache, "cache", fi->uid, ex)) == NULL) {
 			/* stream = camel_stream_mem_new(); */
+			g_static_rec_mutex_unlock (pop3_store->uidl_lock);
 			goto do_free_ex;
 		}
 
@@ -1188,6 +1337,7 @@ rfail:
 
 		/* Check to see we have safely written flag set */
 		if (fi->err != 0) {
+			CamelException iex = CAMEL_EXCEPTION_INITIALISER;
 			if (fi->err == EINTR)
 				camel_exception_setv (ex, CAMEL_EXCEPTION_USER_CANCEL,
 					"User canceled message retrieval");
@@ -1195,6 +1345,11 @@ rfail:
 				camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_PROTOCOL,
 					"Failure while retrieving message %s from POP server: %s",
 					uid, g_strerror (fi->err));
+
+			camel_data_cache_remove (pop3_store->cache, "cache", fi->uid, &iex);
+			camel_exception_clear (&iex);
+			message = NULL;
+
 			goto done;
 		}
 
@@ -1321,7 +1476,7 @@ pop3_get_top (CamelFolder *folder, const char *uid, CamelException *ex)
 	/* Sigh, most of the crap in this function is so that the cancel button
 	   returns the proper exception code.  Sigh. */
 
-	camel_operation_start_transient(NULL, "Retrieving summary info for %d", fi->id);
+	/*camel_operation_start_transient(NULL, "Retrieving summary info for %d", fi->id);*/
 
 	/* If we have an oustanding retrieve message running, wait for that to complete
 	   & then retrieve from cache, otherwise, start a new one, and similar */
@@ -1405,11 +1560,17 @@ pop3_get_top (CamelFolder *folder, const char *uid, CamelException *ex)
 		}
 
 		pcr = camel_pop3_engine_command_new(pop3_store->engine, CAMEL_POP3_COMMAND_MULTI,
-			cmd_tocache, fi, "TOP %u 0\r\n", fi->id);
+			cmd_totop, fi, "TOP %u 0\r\n", fi->id);
 		while ((i = camel_pop3_engine_iterate(pop3_store->engine, pcr)) > 0)
 			;
 		if (i == -1)
 			fi->err = errno;
+
+		if (pop3_store->engine && pcr->state == CAMEL_POP3_COMMAND_ERR) {
+			fi->err = -1;
+			pop3_store->engine->capa &= ~CAMEL_POP3_CAP_TOP;
+		}
+
 		camel_pop3_engine_command_free(pop3_store->engine, pcr);
 
 		g_static_rec_mutex_unlock (pop3_store->eng_lock);
@@ -1451,6 +1612,8 @@ pop3_get_top (CamelFolder *folder, const char *uid, CamelException *ex)
 		message = NULL;
 	}
 
+	check_dir (pop3_store, NULL);
+
 	mi = (CamelMessageInfoBase *) camel_folder_summary_uid (summary, uid);
 	if (!mi && !camel_pop3_logbook_is_registered (pop3_store->book, uid)) {
 		mi = (CamelMessageInfoBase *) camel_folder_summary_info_new_from_message (summary, message);
@@ -1464,7 +1627,8 @@ pop3_get_top (CamelFolder *folder, const char *uid, CamelException *ex)
 done:
 	camel_object_unref((CamelObject *)stream);
 fail:
-	camel_operation_end(NULL);
+	
+	/*camel_operation_end(NULL);*/
 
 
 	return message;
