@@ -348,6 +348,47 @@ camel_pop3_store_destroy_lists (CamelPOP3Store *pop3_store)
 
 }
 
+
+
+static void
+camel_pop3_store_destroy_lists_nl (CamelPOP3Store *pop3_store)
+{
+
+	if (pop3_store->uids != NULL)
+	{
+		CamelPOP3FolderInfo **fi = (CamelPOP3FolderInfo **) pop3_store->uids->pdata;
+		int i;
+
+		for (i=0;i<pop3_store->uids->len;i++,fi++) {
+			if (fi[0]->cmd) {
+
+				if (pop3_store->engine == NULL) {
+					g_ptr_array_free(pop3_store->uids, TRUE);
+					g_hash_table_destroy(pop3_store->uids_uid);
+					g_free(fi[0]->uid);
+					g_free(fi[0]);
+					return;
+				}
+
+				while (camel_pop3_engine_iterate(pop3_store->engine, fi[0]->cmd) > 0)
+					;
+				camel_pop3_engine_command_free(pop3_store->engine, fi[0]->cmd);
+				fi[0]->cmd = NULL;
+			}
+
+			g_free(fi[0]->uid);
+			g_free(fi[0]);
+		}
+
+		kill_lists (pop3_store);
+
+		pop3_store->uids = g_ptr_array_new ();
+		pop3_store->uids_uid = g_hash_table_new(g_str_hash, g_str_equal);
+		pop3_store->uids_id = g_hash_table_new(NULL, NULL);
+
+	}
+}
+
 static gpointer
 wait_for_login_delay (gpointer user_data)
 {
@@ -366,14 +407,18 @@ wait_for_login_delay (gpointer user_data)
 
 		sleep (login_delay);
 
-		g_static_rec_mutex_lock (store->eng_lock);
+
 		if (!store->is_refreshing) {
 			CamelException dex = CAMEL_EXCEPTION_INITIALISER;
-			camel_pop3_store_destroy_lists (store);
+			g_static_rec_mutex_lock (store->uidl_lock);
+			g_static_rec_mutex_lock (store->eng_lock);
+			camel_pop3_store_destroy_lists_nl (store);
 			camel_service_disconnect (CAMEL_SERVICE (store), TRUE, &dex);
+			g_static_rec_mutex_unlock (store->eng_lock);
+			g_static_rec_mutex_unlock (store->uidl_lock);
 			killed = TRUE;
 		}
-		g_static_rec_mutex_unlock (store->eng_lock);
+
 	}
 
 	camel_object_unref (store);
@@ -538,7 +583,9 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, int
 
 	camel_object_unref (CAMEL_OBJECT (store->engine));
 	camel_object_unref (CAMEL_OBJECT (tcp_stream));
+	g_static_rec_mutex_lock (store->eng_lock);
 	store->engine = NULL;
+	g_static_rec_mutex_unlock (store->eng_lock);
 	store->connected = FALSE;
 
 	return FALSE;
@@ -636,9 +683,11 @@ query_auth_types (CamelService *service, CamelException *ex)
 		}
 
 		types = g_list_concat(types, g_list_copy(store->engine->auth));
-		g_static_rec_mutex_unlock (store->eng_lock);
 
 		pop3_disconnect (service, TRUE, NULL);
+
+		g_static_rec_mutex_unlock (store->eng_lock);
+
 	} else {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CONNECT,
 				      _("Could not connect to POP server %s"),
@@ -860,6 +909,8 @@ pop3_try_authenticate (CamelService *service, gboolean reprompt, const char *err
 	if (pcu)
 		camel_pop3_engine_command_free(store->engine, pcu);
 
+	/* camel_pop3_engine_reget_capabilities (store->engine); */
+
 	return status;
 }
 
@@ -939,15 +990,19 @@ pop3_connect (CamelService *service, CamelException *ex)
 	return TRUE;
 }
 
+
+// This is called locked by query_auth_types
 static gboolean
 pop3_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 {
 	CamelPOP3Store *store = CAMEL_POP3_STORE (service);
 
+	//g_static_rec_mutex_lock (store->eng_lock);
+
 	store->logged_in = FALSE;
 
 	if (store->engine == NULL) {
-		g_static_rec_mutex_unlock (store->eng_lock);
+		//g_static_rec_mutex_unlock (store->eng_lock);
 		return TRUE;
 	}
 
@@ -960,9 +1015,12 @@ pop3_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 		camel_pop3_engine_command_free(store->engine, pc);
 	}
 
-	camel_object_unref((CamelObject *)store->engine);
+	g_timeout_add (20000, unref_it, store->engine);
+	/* camel_object_unref((CamelObject *)store->engine); */
 	store->engine = NULL;
 
+	//g_static_rec_mutex_unlock (store->eng_lock);
+
 	return TRUE;
 }
 
