@@ -223,6 +223,48 @@ _tny_camel_account_refresh (TnyCamelAccount *self, gboolean recon_if)
 
 		if (urlneedfree)
 			camel_url_free (url);
+	} else if (apriv->url_string) {
+		CamelException uex = CAMEL_EXCEPTION_INITIALISER;
+		CamelURL *url = camel_url_new (apriv->url_string, &uex);
+		if (camel_exception_is_set (&uex))
+			camel_exception_clear (&uex);
+
+		if (url) {
+			if (apriv->proto)
+				g_free (apriv->proto);
+			if (url->protocol)
+				apriv->proto = g_strdup (url->protocol);
+			else
+				apriv->proto = NULL;
+
+			if (apriv->user)
+				g_free (apriv->user);
+			if (url->user)
+				apriv->user = g_strdup (url->user);
+			else
+				apriv->user = NULL;
+
+			if (url->port != -1)
+				apriv->port = url->port;
+
+			if (apriv->host)
+				g_free (apriv->host);
+			if (url->host)
+				apriv->host = g_strdup (url->host);
+			else
+				apriv->host = NULL;
+
+			if (apriv->mech)
+				g_free (apriv->mech);
+			if (url->authmech)
+				apriv->mech = g_strdup (url->authmech);
+			else
+				apriv->mech = NULL;
+
+
+			camel_url_free (url);
+		}
+
 	}
 
 	if (recon_if && (apriv->status != TNY_CONNECTION_STATUS_DISCONNECTED))
@@ -605,7 +647,7 @@ _tny_camel_account_start_camel_operation_n (TnyCamelAccount *self, CamelOperatio
 
 	g_static_rec_mutex_lock (priv->cancel_lock);
 
-	while (priv->inuse_spin); 
+	//while (priv->inuse_spin); 
 
 	priv->inuse_spin = TRUE;
 
@@ -1045,10 +1087,6 @@ tny_camel_account_set_pass_func_default (TnyAccount *self, TnyGetPassFunc get_pa
 	gboolean reconf_if = FALSE;
 	gboolean changed = FALSE;
 
-	/* Ignore this if it is not a change: */
-	if (get_pass_func == priv->get_pass_func)
-		return;
-
 	g_static_rec_mutex_lock (priv->service_lock);
 
 	if (priv->get_pass_func != get_pass_func)
@@ -1592,6 +1630,16 @@ done:
  * The @callback will be invoke as soon as the account is actually online. It's
  * guaranteed that the @callback will happen in the mainloop, if available.
  *
+ * This is a cancelable operation which means that if another cancelable 
+ * operation executes, this operation will be aborted. Being aborted means that
+ * the callback will still be called, but with cancelled=TRUE.
+ *
+ * Only one instance of @tny_camel_account_set_online for folder @self can run
+ * at the same time. If you call for another, the first will be aborted. This
+ * means that it's callback will be called with cancelled=TRUE. If the 
+ * TnyDevice's online state changes while operation is taking place, the 
+ * behaviour is undefined. Although the @callback will always happen and in that
+ * case with cancelled=TRUE.
  **/
 void 
 tny_camel_account_set_online (TnyCamelAccount *self, gboolean online, TnyCamelSetOnlineCallback callback, gpointer user_data)
@@ -1987,8 +2035,13 @@ tny_camel_account_finalize (GObject *object)
 
 	g_static_rec_mutex_lock (priv->service_lock);
 
-	if (priv->service && CAMEL_IS_OBJECT (priv->service))
+	if (priv->service && CAMEL_IS_OBJECT (priv->service)) {
+		if (priv->service->url) {
+			/* known leak to enforce creating a new service */
+			priv->service->url->user = g_strdup ("non existing dummy user");
+		}
 		camel_object_unref (CAMEL_OBJECT (priv->service));
+	}
 
 	if (G_LIKELY (priv->cache_location))
 		g_free (priv->cache_location);
