@@ -33,7 +33,7 @@ struct _TnyCamelFolderPriv
 	GType headers_list_type;
 	guint folder_changed_id;
 	guint reason_to_live;
-	GMutex *reason_lock;
+	GStaticRecMutex *reason_lock;
 	GStaticRecMutex *folder_lock, *obs_lock;
 	CamelFolder *folder;
 	gchar *folder_name;
@@ -53,6 +53,7 @@ struct _TnyCamelFolderPriv
 	TnyFolderCaps caps; 
 	CamelException load_ex;
 	GList *obs, *sobs;
+	gboolean cant_reuse_iter;
 };
 
 CamelFolder* _tny_camel_folder_get_camel_folder (TnyCamelFolder *self);
