@@ -62,8 +62,57 @@ static GObjectClass *parent_class = NULL;
 #include <camel/camel-mime-filter-charset.h>
 #include <camel/camel-mime-filter-windows.h>
 
+#include <libedataserver/e-iconv.h>
+
 static ssize_t camel_stream_format_text (CamelDataWrapper *dw, CamelStream *stream);
 
+
+
+static char*
+decode_it_2 (CamelMimePart *part, const char *str, gboolean is_addr)
+{
+	struct _camel_header_raw *h = part->headers;
+	const char *content, *charset = NULL;
+	CamelContentType *ct = NULL;
+
+	if (!str)
+		return NULL;
+	
+	if ((content = camel_header_raw_find(&h, "Content-Type", NULL))
+	     && (ct = camel_content_type_decode(content))
+	     && (charset = camel_content_type_param(ct, "charset"))
+	     && (g_ascii_strcasecmp(charset, "us-ascii") == 0))
+		charset = NULL;
+
+	charset = charset ? e_iconv_charset_name (charset) : NULL;
+
+	while (isspace ((unsigned) *str))
+		str++;
+
+	if (is_addr) {
+		char *ret;
+		struct _camel_header_address *addr;
+		addr = camel_header_address_decode (str, charset);
+		if (addr) {
+			ret = camel_header_address_list_format (addr);
+			camel_header_address_list_clear (&addr);
+		} else {
+			ret = g_strdup (str);
+		}
+
+		if (ct)
+			camel_content_type_unref (ct);
+
+		return ret;
+	}
+
+	if (ct)
+		camel_content_type_unref (ct);
+
+	return camel_header_decode_string (str, charset);
+}
+
+
 static void 
 tny_camel_mime_part_set_header_pair (TnyMimePart *self, const gchar *name, const gchar *value)
 {
@@ -120,6 +169,7 @@ tny_camel_mime_part_get_parts_default (TnyMimePart *self, TnyList *list)
 	CamelDataWrapper *containee;
 	gboolean is_related = FALSE;
 	CamelContentType *content_type = NULL;
+	gboolean has_attachments = FALSE;
 
 	g_assert (TNY_IS_LIST (list));
 
@@ -148,10 +198,15 @@ tny_camel_mime_part_get_parts_default (TnyMimePart *self, TnyList *list)
 			CamelMimePart *tpart = camel_multipart_get_part (CAMEL_MULTIPART (containee), i);
 			TnyMimePart *newpart=NULL;
 			CamelContentType *type;
+			const gchar *disposition;
 
 			if (!tpart || !CAMEL_IS_MIME_PART (tpart))
 				continue;
 
+			disposition = camel_mime_part_get_disposition (tpart);
+			if (disposition && camel_strstrcase (disposition, "attachment") != NULL)
+				has_attachments = TRUE;
+
 			type = camel_mime_part_get_content_type (tpart);
 			if (CAMEL_IS_MIME_MESSAGE (tpart))
 			{
@@ -195,6 +250,18 @@ tny_camel_mime_part_get_parts_default (TnyMimePart *self, TnyList *list)
 		}
 	}
 
+	if (TNY_IS_MSG (self)) {
+		TnyHeader *header;
+
+		header = tny_msg_get_header (TNY_MSG (self));
+		if (header) {
+			if (has_attachments && 
+			    !(tny_header_get_flags (header) & TNY_HEADER_FLAG_ATTACHMENTS))
+				tny_header_set_flag (header, TNY_HEADER_FLAG_ATTACHMENTS);
+			g_object_unref (header);
+		}
+	}
+
 	g_mutex_unlock (priv->part_lock);
 
 	return;
@@ -285,22 +352,35 @@ recreate_part (TnyMimePart *orig)
 	if (TNY_IS_MSG (orig)) {
 		TnyHeader *hdr = NULL;
 		TnyHeader *dest_header;
+		gchar *str;
 
 		hdr = tny_msg_get_header (TNY_MSG (orig));
 		retval = TNY_MIME_PART (tny_camel_msg_new ());
  		dest_header = tny_msg_get_header (TNY_MSG (retval));
-		if (tny_header_get_bcc (hdr))
-			tny_header_set_bcc (dest_header, tny_header_get_bcc (hdr));
-		if (tny_header_get_cc (hdr))
-			tny_header_set_cc (dest_header, tny_header_get_cc (hdr));
-		if (tny_header_get_from (hdr))
-			tny_header_set_from (dest_header, tny_header_get_from (hdr));
-		if (tny_header_get_replyto (hdr))
-			tny_header_set_replyto (dest_header, tny_header_get_replyto (hdr));
-		if (tny_header_get_subject (hdr))
-			tny_header_set_subject (dest_header, tny_header_get_subject (hdr));
-		if (tny_header_get_to (hdr))
-			tny_header_set_to (dest_header, tny_header_get_to (hdr));
+		if (str = tny_header_dup_bcc (hdr)) {
+			tny_header_set_bcc (dest_header, str);
+			g_free (str);
+		}
+		if (str = tny_header_dup_cc (hdr)) {
+			tny_header_set_cc (dest_header, str);
+			g_free (str);
+		}
+		if (str = tny_header_dup_from (hdr)) {
+			tny_header_set_from (dest_header, str);
+			g_free (str);
+		}
+		if (str = tny_header_dup_replyto (hdr)) {
+			tny_header_set_replyto (dest_header, str);
+			g_free (str);
+		}
+		if (str = tny_header_dup_subject (hdr)) {
+			tny_header_set_subject (dest_header, str);
+			g_free (str);
+		}
+		if (str = tny_header_dup_to (hdr)) {
+			tny_header_set_to (dest_header, str);
+			g_free (str);
+		}
 		/* tny_header_set_priority (dest_header, tny_header_get_priority (hdr)); */
 		g_object_unref (hdr);
 		g_object_unref (dest_header);
@@ -585,8 +665,8 @@ tny_camel_mime_part_write_to_stream_default (TnyMimePart *self, TnyStream *strea
 	wrapper = camel_medium_get_content_object (medium);
 
 	if (!wrapper) {
-		g_error (_("Mime part does not yet have a source stream, use "
-			"tny_mime_part_construct first"));
+		g_warning (_("Mime part does not yet have a source stream, use "
+			     "tny_mime_part_construct first"));
 		camel_object_unref (cstream);
 		g_set_error (err, TNY_MIME_ERROR, 
 				TNY_MIME_ERROR_STATE,
@@ -628,7 +708,7 @@ camel_stream_format_text (CamelDataWrapper *dw, CamelStream *stream)
 
 	if (dw->mime_type && (charset = camel_content_type_param 
 			(dw->mime_type, "charset")) && 
-		g_ascii_strncasecmp(charset, "iso-8859-", 9) == 0) 
+		g_ascii_strncasecmp(charset, "iso-8859+-", 9) == 0) 
 	{
 		CamelStream *null;
 
@@ -679,6 +759,7 @@ tny_camel_mime_part_decode_to_stream_default (TnyMimePart *self, TnyStream *stre
 	CamelDataWrapper *wrapper;
 	CamelMedium *medium;
 	CamelStream *cstream;
+	gssize bytes = -1;
 
 	g_assert (TNY_IS_STREAM (stream));
 
@@ -692,13 +773,11 @@ tny_camel_mime_part_decode_to_stream_default (TnyMimePart *self, TnyStream *stre
 	wrapper = camel_medium_get_content_object (medium);
 
 	if (G_UNLIKELY (!wrapper)) {
-		g_error (_("Mime part does not yet have a source stream, use "
-			"tny_mime_part_construct first"));
+		g_warning (_("Mime part does not yet have a source stream, use "
+			     "tny_mime_part_construct first"));
 		camel_object_unref (CAMEL_OBJECT (cstream));
 		return -1;
 	}
-
-	gssize bytes = -1;
 	
 	if (camel_content_type_is (wrapper->mime_type, "text", "*"))
 		bytes = (gssize) camel_stream_format_text (wrapper, cstream);
@@ -846,6 +925,7 @@ tny_camel_mime_part_get_decoded_stream_default (TnyMimePart *self)
 	CamelDataWrapper *wrapper;
 	CamelMedium *medium;
 	CamelStream *stream = camel_stream_mem_new ();
+	gssize bytes = -1;
 
 	g_mutex_lock (priv->part_lock);
 	medium =  CAMEL_MEDIUM (priv->part);
@@ -864,15 +944,17 @@ tny_camel_mime_part_get_decoded_stream_default (TnyMimePart *self)
 		camel_stream_reset (wrapper->stream);
 
 		if (camel_content_type_is (wrapper->mime_type, "text", "*"))
-			camel_stream_format_text (wrapper, stream);
+			bytes = camel_stream_format_text (wrapper, stream);
 		else
-			camel_data_wrapper_decode_to_stream (wrapper, stream);
+			bytes = camel_data_wrapper_decode_to_stream (wrapper, stream);
 	}
 
-	retval = TNY_STREAM (tny_camel_stream_new (stream));
-	camel_object_unref (stream);
+	if (bytes >= 0) {
+		retval = TNY_STREAM (tny_camel_stream_new (stream));
+		tny_stream_reset (retval);
+	}
 
-	tny_stream_reset (retval);
+	camel_object_unref (stream);
 	camel_object_unref (medium);
 
 	return retval;
@@ -959,6 +1041,10 @@ _tny_camel_mime_part_set_part (TnyCamelMimePart *self, CamelMimePart *part)
 
 	g_mutex_lock (priv->part_lock);
 
+	if (priv->cached_filename)
+		g_free (priv->cached_filename);
+	priv->cached_filename = NULL;
+
 	if (priv->cached_content_type)
 		g_free (priv->cached_content_type);
 	priv->cached_content_type = NULL;
@@ -1008,13 +1094,21 @@ static const gchar*
 tny_camel_mime_part_get_filename_default (TnyMimePart *self)
 {
 	TnyCamelMimePartPriv *priv = TNY_CAMEL_MIME_PART_GET_PRIVATE (self);
-	const gchar *retval;
 
 	g_mutex_lock (priv->part_lock);
-	retval = camel_mime_part_get_filename (priv->part);
+	if (!priv->cached_filename) {
+		const char *str = camel_mime_part_get_filename (priv->part);
+
+		if (str) {
+			if (!g_utf8_validate (str, -1, NULL))
+				priv->cached_filename = decode_it_2 (priv->part, str, FALSE);
+			else
+				priv->cached_filename = g_strdup (str);
+		}
+	}
 	g_mutex_unlock (priv->part_lock);
-	
-	return retval;
+
+	return priv->cached_filename;
 }
 
 static const gchar*
@@ -1192,9 +1286,15 @@ tny_camel_mime_part_set_content_type_default (TnyMimePart *self, const gchar *co
 
 	g_mutex_lock (priv->part_lock);
 	camel_mime_part_set_content_type (priv->part, content_type);
+
+	if (priv->cached_filename)
+		g_free (priv->cached_filename);
+	priv->cached_filename = NULL;
+
 	if (priv->cached_content_type)
 		g_free (priv->cached_content_type);
 	priv->cached_content_type = NULL;
+
 	g_mutex_unlock (priv->part_lock);
 
 	return;
@@ -1207,9 +1307,15 @@ tny_camel_mime_part_finalize (GObject *object)
 	TnyCamelMimePartPriv *priv = TNY_CAMEL_MIME_PART_GET_PRIVATE (self);
 
 	g_mutex_lock (priv->part_lock);
+
+	if (priv->cached_filename)
+		g_free (priv->cached_filename);
+	priv->cached_filename = NULL;
+
 	if (priv->cached_content_type)
 		g_free (priv->cached_content_type);
 	priv->cached_content_type = NULL;
+
 	if (priv->part && CAMEL_IS_OBJECT (priv->part))
 		camel_object_unref (CAMEL_OBJECT (priv->part));
 	g_mutex_unlock (priv->part_lock);
@@ -1350,6 +1456,9 @@ tny_camel_mime_part_instance_init (GTypeInstance *instance, gpointer g_class)
 
 	priv->part_lock = g_mutex_new ();
 
+	priv->cached_filename = NULL;
+	priv->cached_content_type = NULL;
+
 	return;
 }
 
