@@ -329,15 +329,72 @@ emit_queue_control_signals (TnySendQueue *self, guint signal_id)
 typedef struct {
 	TnySendQueue *self;
 	TnyDevice *device;
+	TnyAccount *outbox_account, *sentbox_account;
+	TnyFolder *outbox, *sentbox;
 } MainThreadInfo;
 
+typedef struct {
+	TnyFolder *folder;
+	TnyList *list;
+	gboolean refresh;
+	GError **err;
+
+	GCond* condition;
+	gboolean had_callback;
+	GMutex *mutex;
+
+} GetHeadersSync;
+
+static void 
+get_headers_async (TnyFolder *self, gboolean cancelled, TnyList *headers, GError *err, gpointer user_data)
+{
+	GetHeadersSync *info = (GetHeadersSync *) user_data;
+
+	if (err && info->err)
+		*info->err = g_error_copy (err);
+
+	g_mutex_lock (info->mutex);
+	g_cond_broadcast (info->condition);
+	info->had_callback = TRUE;
+	g_mutex_unlock (info->mutex);
+}
+
+static void
+get_headers_sync (TnyFolder *folder, TnyList *list, gboolean refresh, GError **err)
+{
+	GetHeadersSync *info = g_slice_new0 (GetHeadersSync);
+
+	info->mutex = g_mutex_new ();
+	info->condition = g_cond_new ();
+	info->had_callback = FALSE;
+
+	info->folder = g_object_ref (folder);
+	info->list = g_object_ref (list);
+	info->refresh = refresh;
+	info->err = err;
+
+	tny_folder_get_headers_async (folder, list, refresh, 
+			get_headers_async, NULL, info);
+
+	g_mutex_lock (info->mutex);
+	if (!info->had_callback)
+		g_cond_wait (info->condition, info->mutex);
+	g_mutex_unlock (info->mutex);
+
+	g_mutex_free (info->mutex);
+	g_cond_free (info->condition);
+
+	g_object_unref (info->folder);
+	g_object_unref (info->list);
+	g_slice_free (GetHeadersSync, info);
+}
+
 static gpointer
 thread_main (gpointer data)
 {
 	MainThreadInfo *info = (MainThreadInfo *) data;
 	TnySendQueue *self = info->self;
 	TnyCamelSendQueuePriv *priv = TNY_CAMEL_SEND_QUEUE_GET_PRIVATE (self);
-	TnyFolder *sentbox = NULL, *outbox = NULL;
 	guint i = 0, length = 0;
 	TnyList *list = NULL;
 	TnyDevice *device = info->device;
@@ -346,41 +403,24 @@ thread_main (gpointer data)
 	priv->is_running = TRUE;
 	list = tny_simple_list_new ();
 
-	g_mutex_lock (priv->todo_lock);
+	g_static_rec_mutex_lock (priv->todo_lock);
 	{
 		GError *terror = NULL;
-		sentbox = get_sentbox (self);
-		outbox = tny_send_queue_get_outbox (self);
-
-		if (!outbox || !sentbox) {
-			if (outbox)
-				g_object_unref (outbox);
-			if (sentbox)
-				g_object_unref (sentbox);
-			g_warning ("tny_send_queue_get_outbox and "
-				"tny_send_queue_get_sentbox are not allowed to "
-				"return NULL. This indicates a problem in the "
-				"software.");
-			priv->is_running = FALSE;
-			g_mutex_unlock (priv->todo_lock);
-			goto errorhandler;
-		}
-
-		tny_folder_get_headers (outbox, list, TRUE, &terror);
+		get_headers_sync (info->outbox, list, TRUE, &terror);
 
 		if (terror != NULL)
 		{
 			emit_error (self, NULL, NULL, terror, i, priv->total);
 			g_error_free (terror);
 			g_object_unref (list);
-			g_mutex_unlock (priv->todo_lock);
+			g_static_rec_mutex_unlock (priv->todo_lock);
 			goto errorhandler;
 		}
 
 		length = tny_list_get_length (list);
 		priv->total = length;
 	}
-	g_mutex_unlock (priv->todo_lock);
+	g_static_rec_mutex_unlock (priv->todo_lock);
 
 	g_object_unref (list);
 
@@ -391,9 +431,9 @@ thread_main (gpointer data)
 		TnyHeader *header = NULL;
 		TnyMsg *msg = NULL;
 
-		g_mutex_lock (priv->sending_lock);
+		g_static_rec_mutex_lock (priv->sending_lock);
 
-		g_mutex_lock (priv->todo_lock);
+		g_static_rec_mutex_lock (priv->todo_lock);
 		{
 			GError *ferror = NULL;
 			TnyIterator *hdriter = NULL;
@@ -401,12 +441,12 @@ thread_main (gpointer data)
 			GList *to_remove = NULL, *copy;
 			TnyIterator *giter = NULL;
 
-			if (tny_device_is_online (device))
-				tny_folder_get_headers (outbox, headers, TRUE, &ferror);
+			if (tny_device_is_online (device)) 
+				get_headers_sync (info->outbox, headers, TRUE, &ferror);
 			else {
 				priv->is_running = FALSE;
-				g_mutex_unlock (priv->todo_lock);
-				g_mutex_unlock (priv->sending_lock);
+				g_static_rec_mutex_unlock (priv->todo_lock);
+				g_static_rec_mutex_unlock (priv->sending_lock);
 				goto errorhandler;
 			}
 
@@ -416,8 +456,8 @@ thread_main (gpointer data)
 				emit_error (self, NULL, NULL, ferror, i, priv->total);
 				g_error_free (ferror);
 				g_object_unref (headers);
-				g_mutex_unlock (priv->todo_lock);
-				g_mutex_unlock (priv->sending_lock);
+				g_static_rec_mutex_unlock (priv->todo_lock);
+				g_static_rec_mutex_unlock (priv->sending_lock);
 				goto errorhandler;
 			}
 
@@ -427,12 +467,17 @@ thread_main (gpointer data)
 			{
 				TnyHeader *curhdr = TNY_HEADER (tny_iterator_get_current (giter));
 				TnyHeaderFlags flags = tny_header_get_flags (curhdr);
+				gchar *uid;
+
+				uid = tny_header_dup_uid (curhdr);
 
 				if ((flags & TNY_HEADER_FLAG_SUSPENDED) ||
+				    (flags & TNY_HEADER_FLAG_ANSWERED) ||
 				    (g_hash_table_lookup_extended (failed_headers, 
-								   tny_header_get_uid (curhdr),
+								   uid,
 								   NULL, NULL)))
 					to_remove = g_list_prepend (to_remove, curhdr);
+				g_free (uid);
 
 				g_object_unref (curhdr);
 				tny_iterator_next (giter);
@@ -456,8 +501,8 @@ thread_main (gpointer data)
 			{
 				priv->is_running = FALSE;
 				g_object_unref (headers);
-				g_mutex_unlock (priv->todo_lock);
-				g_mutex_unlock (priv->sending_lock);
+				g_static_rec_mutex_unlock (priv->todo_lock);
+				g_static_rec_mutex_unlock (priv->sending_lock);
 				break;
 			}
 
@@ -467,7 +512,7 @@ thread_main (gpointer data)
 			g_object_unref (hdriter);
 			g_object_unref (headers);
 		}
-		g_mutex_unlock (priv->todo_lock);
+		g_static_rec_mutex_unlock (priv->todo_lock);
 
 		if (header && TNY_IS_HEADER (header) && tny_device_is_online (device))
 		{
@@ -475,7 +520,7 @@ thread_main (gpointer data)
 			GError *err = NULL;
 
 			tny_list_prepend (hassent, G_OBJECT (header));
-			msg = tny_folder_get_msg (outbox, header, &err);
+			msg = tny_folder_get_msg (info->outbox, header, &err);
 
 			/* Emits msg-sending signal to inform a new msg is being sent */
 			emit_control (self, header, msg, TNY_SEND_QUEUE_MSG_SENDING, i, priv->total);
@@ -485,37 +530,40 @@ thread_main (gpointer data)
 				_tny_camel_account_start_camel_operation (TNY_CAMEL_ACCOUNT (priv->trans_account),
 									  NULL, NULL, "Sending message");
 
+				g_static_rec_mutex_unlock (priv->sending_lock);
 				tny_transport_account_send (priv->trans_account, msg, &err);
+				g_static_rec_mutex_lock (priv->sending_lock);
 
 				_tny_camel_account_stop_camel_operation (TNY_CAMEL_ACCOUNT (priv->trans_account));
 
 				if (err != NULL) {
 					emit_error (self, header, msg, err, i, priv->total);
 					g_hash_table_insert (failed_headers, 
-							     g_strdup (tny_header_get_uid (header)), NULL);
+							     tny_header_dup_uid (header), NULL);
 				}
 			} else {
 				emit_error (self, header, msg, err, i, priv->total);
 				g_hash_table_insert (failed_headers, 
-						     g_strdup (tny_header_get_uid (header)), NULL);
+						     tny_header_dup_uid (header), NULL);
 			}
 
-			g_mutex_lock (priv->todo_lock);
+			g_static_rec_mutex_lock (priv->todo_lock);
 			{
 				if (err == NULL)
 				{
 					GError *newerr = NULL;
 					priv->cur_i = i;
 					tny_header_set_flag (header, TNY_HEADER_FLAG_SEEN);
-					tny_folder_transfer_msgs (outbox, hassent, sentbox, TRUE, &newerr);
+					tny_header_set_flag (header, TNY_HEADER_FLAG_ANSWERED);
+					tny_folder_transfer_msgs (info->outbox, hassent, info->sentbox, TRUE, &newerr);
 					if (newerr != NULL) {
 						emit_error (self, header, msg, newerr, i, priv->total);
 						g_error_free (newerr);
-					}
+					} 
 					priv->total--;
 				}
 			}
-			g_mutex_unlock (priv->todo_lock);
+			g_static_rec_mutex_unlock (priv->todo_lock);
 
 			/* Emits msg-sent signal to inform msg has been sent */
 			/* This now happens in on_msg_sent_get_msg! */
@@ -536,11 +584,11 @@ thread_main (gpointer data)
 			length = 0;
 			if (header && G_IS_OBJECT (header))
 				g_object_unref (header);
-			g_mutex_unlock (priv->sending_lock);
+			g_static_rec_mutex_unlock (priv->sending_lock);
 			break;
 		}
 
-		g_mutex_unlock (priv->sending_lock);
+		g_static_rec_mutex_unlock (priv->sending_lock);
 
 	}
 
@@ -551,13 +599,19 @@ errorhandler:
 	if (failed_headers)
 		g_hash_table_destroy (failed_headers);
 
-	g_object_unref (sentbox);
-	g_object_unref (outbox);
-
 	priv->thread = NULL;
 
 	g_object_unref (info->device);
 	g_object_unref (info->self);
+
+	if (info->outbox_account)
+		g_object_unref (info->outbox_account);
+	if (info->sentbox_account)
+		g_object_unref (info->sentbox_account);
+
+	g_object_unref (info->outbox);
+	g_object_unref (info->sentbox);
+
 	g_slice_free (MainThreadInfo, info);
 
 	/* Emit the queue-stop signal */
@@ -566,8 +620,10 @@ errorhandler:
 	return NULL;
 }
 
+#define FOLDERSNOTREADY 	_("tny_send_queue_get_outbox and tny_send_queue_get_sentbox are not allowed to return NULL. This indicates a problem in the software.")
+
 static void 
-create_worker (TnySendQueue *self)
+create_worker (TnySendQueue *self, GError **err)
 {
 	TnyCamelSendQueuePriv *priv = TNY_CAMEL_SEND_QUEUE_GET_PRIVATE (self);
 
@@ -583,6 +639,28 @@ create_worker (TnySendQueue *self)
 			info->self = g_object_ref (self);
 			info->device = g_object_ref (spriv->device);
 
+			info->outbox = tny_send_queue_get_outbox (self);
+			info->sentbox = get_sentbox (self);
+
+			if (!info->outbox || !info->sentbox) {
+
+				g_set_error (err, TNY_SERVICE_ERROR, 
+					TNY_SERVICE_ERROR_ADD_MSG, FOLDERSNOTREADY);
+				g_warning (FOLDERSNOTREADY);
+
+				g_object_unref (info->self);
+				g_object_unref (info->device);
+				if (info->sentbox)
+					g_object_unref (info->sentbox);
+				if (info->outbox)
+					g_object_unref (info->outbox);
+				g_slice_free (MainThreadInfo, info);
+				return;
+			}
+
+			info->sentbox_account = tny_folder_get_account (info->sentbox);
+			info->outbox_account = tny_folder_get_account (info->outbox);
+
 			emit_queue_control_signals (self, TNY_SEND_QUEUE_START);
 
 			priv->thread = g_thread_create (thread_main, info, FALSE, NULL);
@@ -610,7 +688,7 @@ tny_camel_send_queue_cancel_default (TnySendQueue *self, TnySendQueueCancelActio
 	TnyList *headers = tny_simple_list_new ();
 	TnyIterator *iter;
 
-	g_mutex_lock (priv->sending_lock);
+	g_static_rec_mutex_lock (priv->sending_lock);
 
 	priv->is_running = FALSE;
 
@@ -622,7 +700,7 @@ tny_camel_send_queue_cancel_default (TnySendQueue *self, TnySendQueueCancelActio
 	{
 		g_object_unref (headers);
 		g_object_unref (outbox);
-		g_mutex_unlock (priv->sending_lock);
+		g_static_rec_mutex_unlock (priv->sending_lock);
 		return;
 	}
 
@@ -643,7 +721,7 @@ tny_camel_send_queue_cancel_default (TnySendQueue *self, TnySendQueueCancelActio
 			g_object_unref (iter);
 			g_object_unref (headers);
 			g_object_unref (outbox);
-			g_mutex_unlock (priv->sending_lock);
+			g_static_rec_mutex_unlock (priv->sending_lock);
 			return;
 		}
 
@@ -656,7 +734,7 @@ tny_camel_send_queue_cancel_default (TnySendQueue *self, TnySendQueueCancelActio
 	tny_folder_sync (outbox, TRUE, err);
 	g_object_unref (outbox);
 
-	g_mutex_unlock (priv->sending_lock);
+	g_static_rec_mutex_unlock (priv->sending_lock);
 
 	return;
 }
@@ -689,14 +767,22 @@ on_added (TnyFolder *folder, gboolean cancelled, GError *err, gpointer user_data
 {
 	OnAddedInfo *info = (OnAddedInfo *) user_data;
 	TnyCamelSendQueuePriv *priv = TNY_CAMEL_SEND_QUEUE_GET_PRIVATE (info->self);
+	GError *new_err = NULL;
+
+	if (!err)
+		priv->total++;
+
+	if (!err & !priv->is_running)
+		create_worker (info->self, &new_err);
 
 	/* Call user callback after msg has beed added to OUTBOX, waiting to be sent*/
 	if (info->callback)
-		info->callback (info->self, info->cancelled, info->msg, info->user_data, err);
+		info->callback (info->self, info->cancelled, info->msg, info->user_data, 
+			new_err?new_err:err);
+
+	if (new_err)
+		g_error_free (new_err);
 
-	priv->total++;
-	if (priv->total >= 1 && !priv->is_running)
-		create_worker (info->self);
 	if (info->self)
 		g_object_unref (info->self);
 	if (info->msg)
@@ -714,11 +800,16 @@ tny_camel_send_queue_add_default (TnySendQueue *self, TnyMsg *msg, GError **err)
 
 	g_assert (TNY_IS_CAMEL_MSG (msg));
 
-	g_mutex_lock (priv->todo_lock);
+	g_static_rec_mutex_lock (priv->todo_lock);
 	{
 		TnyFolder *outbox;
 		TnyList *headers = tny_simple_list_new ();
 		OnAddedInfo *info = NULL;
+		TnyHeader *hdr = tny_msg_get_header (msg);
+
+		tny_header_unset_flag (hdr, TNY_HEADER_FLAG_ANSWERED);
+
+		g_object_unref (hdr);
 
 		outbox = tny_send_queue_get_outbox (self);
 
@@ -730,7 +821,7 @@ tny_camel_send_queue_add_default (TnySendQueue *self, TnyMsg *msg, GError **err)
 				"because it does not have a valid outbox. "
 				"This problem indicates a bug in the software."));
 			g_object_unref (headers);
-			g_mutex_unlock (priv->todo_lock);
+			g_static_rec_mutex_unlock (priv->todo_lock);
 			return;
 		}
 
@@ -740,7 +831,7 @@ tny_camel_send_queue_add_default (TnySendQueue *self, TnyMsg *msg, GError **err)
 		{
 			g_object_unref (headers);
 			g_object_unref (outbox);
-			g_mutex_unlock (priv->todo_lock);
+			g_static_rec_mutex_unlock (priv->todo_lock);
 			return;
 		}
 
@@ -756,7 +847,7 @@ tny_camel_send_queue_add_default (TnySendQueue *self, TnyMsg *msg, GError **err)
 
 		g_object_unref (outbox);
 	}
-	g_mutex_unlock (priv->todo_lock);
+	g_static_rec_mutex_unlock (priv->todo_lock);
 
 	return;
 }
@@ -774,54 +865,33 @@ tny_camel_send_queue_add_async_default (TnySendQueue *self, TnyMsg *msg, TnySend
 {
 	TnyCamelSendQueuePriv *priv = TNY_CAMEL_SEND_QUEUE_GET_PRIVATE (self);
 	GError *err = NULL;
+	OnAddedInfo *info = NULL;
+	TnyFolder *outbox;
 
 	g_assert (TNY_IS_CAMEL_MSG (msg));
 
-	g_mutex_lock (priv->todo_lock);
-	{
-		TnyFolder *outbox;
-		TnyList *headers = tny_simple_list_new ();
-		OnAddedInfo *info = NULL;
+	outbox = tny_send_queue_get_outbox (TNY_SEND_QUEUE (self));
 
-		outbox = tny_send_queue_get_outbox (TNY_SEND_QUEUE (self));
+	if (!outbox || !TNY_IS_FOLDER (outbox)) {
+		g_set_error (&err, TNY_SERVICE_ERROR, 
+			TNY_SERVICE_ERROR_ADD_MSG,
+			_("Operating can't continue: send queue not ready "
+			"because it does not have a valid outbox. "
+			"This problem indicates a bug in the software."));
 
-		if (!outbox || !TNY_IS_FOLDER (outbox))
-		{
-			g_set_error (&err, TNY_SERVICE_ERROR, 
-				TNY_SERVICE_ERROR_ADD_MSG,
-				_("Operating can't continue: send queue not ready "
-				"because it does not have a valid outbox. "
-				"This problem indicates a bug in the software."));
-			g_object_unref (headers);
-			g_mutex_unlock (priv->todo_lock);
-			return;
-		}
-
-		tny_folder_get_headers (outbox, headers, TRUE, &err);
-
-		if (err!= NULL)
-		{
-			g_object_unref (headers);
-			g_object_unref (outbox);
-			g_mutex_unlock (priv->todo_lock);
-			return;
-		}
-
-		priv->total = tny_list_get_length (headers);
-		g_object_unref (headers);
-
-		info = g_slice_new0 (OnAddedInfo);
-
-		info->msg = TNY_MSG (g_object_ref (msg));
-		info->self = TNY_SEND_QUEUE (g_object_ref (self));
-		info->callback = callback;
-		info->user_data = user_data;
-
-		tny_folder_add_msg_async (outbox, msg, on_added, on_status, info);
-
-		g_object_unref (outbox);
+		return;
 	}
-	g_mutex_unlock (priv->todo_lock);
+
+	info = g_slice_new0 (OnAddedInfo);
+
+	info->msg = TNY_MSG (g_object_ref (msg));
+	info->self = TNY_SEND_QUEUE (g_object_ref (self));
+	info->callback = callback;
+	info->user_data = user_data;
+
+	tny_folder_add_msg_async (outbox, msg, on_added, on_status, info);
+
+	g_object_unref (outbox);
 
 	return;
 }
@@ -845,7 +915,7 @@ create_maildir (TnySendQueue *self, const gchar *name)
 
 	g_assert (aname);
 
-	full_path = g_strdup_printf ("maildir://%s/%s/maildir", session->storage_path, aname);
+	full_path = g_strdup_printf ("maildir://%s/sendqueue/%s/maildir", session->storage_path, aname);
 
 	mdstore = camel_session_get_store(session, full_path, &ex);
 
@@ -964,7 +1034,7 @@ tny_camel_send_queue_finalize (GObject *object)
 		_tny_session_camel_unreg_queue (apriv->session, (TnyCamelSendQueue *) self);
 	}
 
-	g_mutex_lock (priv->todo_lock);
+	g_static_rec_mutex_lock (priv->todo_lock);
 
 	if (priv->signal != -1)
 		g_signal_handler_disconnect (G_OBJECT (priv->trans_account), 
@@ -974,10 +1044,14 @@ tny_camel_send_queue_finalize (GObject *object)
 	if (priv->outbox_cache)
 		g_object_unref (priv->outbox_cache);
 
-	g_mutex_unlock (priv->todo_lock);
+	g_static_rec_mutex_unlock (priv->todo_lock);
+
+	g_free (priv->todo_lock);
+	priv->todo_lock = NULL;
+
+	g_free (priv->sending_lock);
+	priv->sending_lock = NULL;
 
-	g_mutex_free (priv->todo_lock);
-	g_mutex_free (priv->sending_lock);
 	g_object_unref (priv->trans_account);
 
 	(*parent_class->finalize) (object);
@@ -1089,8 +1163,11 @@ tny_camel_send_queue_get_transport_account (TnyCamelSendQueue *self)
 void
 tny_camel_send_queue_flush (TnyCamelSendQueue *self)
 {
+	GError *err = NULL;
 	g_return_if_fail (TNY_IS_CAMEL_SEND_QUEUE(self));
-	create_worker (TNY_SEND_QUEUE (self));
+	create_worker (TNY_SEND_QUEUE (self), &err);
+	if (err)
+		g_error_free (err);
 }
 
 
@@ -1141,8 +1218,12 @@ tny_camel_send_queue_instance_init (GTypeInstance *instance, gpointer g_class)
 	priv->signal = -1;
 	priv->sentbox_cache = NULL;
 	priv->outbox_cache = NULL;
-	priv->todo_lock = g_mutex_new ();
-	priv->sending_lock = g_mutex_new ();
+
+	priv->todo_lock = g_new0 (GStaticRecMutex, 1);
+	g_static_rec_mutex_init (priv->todo_lock);
+	priv->sending_lock = g_new0 (GStaticRecMutex, 1);
+	g_static_rec_mutex_init (priv->sending_lock);
+
 	priv->is_running = FALSE;
 	priv->thread = NULL;
 	priv->pending_send_notifies = 0;
