@@ -484,13 +484,42 @@ tny_camel_store_account_prepare (TnyCamelAccount *self, gboolean recon_if, gbool
 			apriv->type, apriv->ex);
 
 		if (apriv->service && CAMEL_IS_SERVICE (apriv->service) && 
-		  new_service && !camel_exception_is_set (apriv->ex)) 
-		{
+		  new_service && !camel_exception_is_set (apriv->ex)) {
 			camel_object_unref (apriv->service);
 			apriv->service = NULL;
-		} 
+		}
 
 		if (new_service && !camel_exception_is_set (apriv->ex)) {
+			char *str = camel_service_get_url (new_service);
+			if (str) {
+				char *urlsfilep = camel_session_get_storage_path ((CamelSession *)apriv->session, new_service, NULL);
+				if (urlsfilep) {
+					
+					char *urlsfile = g_strdup_printf ("%s/url_string", urlsfilep);
+					char *namefile = g_strdup_printf ("%s/name", urlsfilep);
+					FILE *f1 = fopen (urlsfile, "w");
+					FILE *f2 = fopen (namefile, "w");
+					if (f1) {
+						fputs (str, f1);
+						fclose (f1);
+					}
+					if (f2) {
+						if (apriv->name)
+							fputs (apriv->name, f2);
+						else
+							fputs ("No name", f2);
+						fclose (f2);
+					}
+					g_free (namefile);
+					g_free (urlsfile);
+					g_free (urlsfilep);
+				}
+				g_free (str);
+			}
+
+			if (apriv->service && CAMEL_IS_SERVICE (apriv->service))
+				camel_object_unref (apriv->service);
+
 			apriv->service = new_service;
 			apriv->service->data = self;
 			apriv->service->connecting = (con_op) connection;
@@ -1191,6 +1220,8 @@ tny_camel_store_account_create_folder_async_destroyer (gpointer thr_user_data)
 
 	_tny_session_stop_operation (info->session);
 
+	camel_object_unref (info->session);
+
 	return;
 }
 
@@ -1210,6 +1241,7 @@ tny_camel_store_account_create_folder_async_default (TnyFolderStore *self, const
 	/* Idle info for the callbacks */
 	info = g_slice_new (CreateFolderInfo);
 	info->session = apriv->session;
+	camel_object_ref (info->session);
 	info->self = self;
 	info->name = g_strdup (name);
 	info->callback = callback;
@@ -1259,6 +1291,19 @@ tny_camel_store_account_factor_folder (TnyCamelStoreAccount *self, const gchar *
 	return TNY_CAMEL_STORE_ACCOUNT_GET_CLASS (self)->factor_folder(self, full_name, was_new);
 }
 
+
+static void 
+notify_factory_del (TnyCamelStoreAccount *self, GObject *folder)
+{
+	if (self && TNY_IS_CAMEL_STORE_ACCOUNT (self)) {
+		TnyCamelStoreAccountPriv *priv = TNY_CAMEL_STORE_ACCOUNT_GET_PRIVATE (self);
+		g_static_rec_mutex_lock (priv->factory_lock);
+		priv->managed_folders = g_list_remove_all (priv->managed_folders, folder);
+		g_static_rec_mutex_unlock (priv->factory_lock);
+	}
+}
+
+
 static TnyFolder * 
 tny_camel_store_account_factor_folder_default (TnyCamelStoreAccount *self, const gchar *full_name, gboolean *was_new)
 {
@@ -1286,8 +1331,8 @@ tny_camel_store_account_factor_folder_default (TnyCamelStoreAccount *self, const
 
 	if (!folder) {
 		folder = TNY_CAMEL_FOLDER (_tny_camel_folder_new ());
+		g_object_weak_ref (G_OBJECT (folder), (GWeakNotify) notify_factory_del, self);
 		priv->managed_folders = g_list_prepend (priv->managed_folders, folder);
-
 		*was_new = TRUE;
 	}
 
@@ -1303,7 +1348,7 @@ tny_camel_store_account_get_folders_default (TnyFolderStore *self, TnyList *list
 	TnyCamelStoreAccountPriv *priv = TNY_CAMEL_STORE_ACCOUNT_GET_PRIVATE (self);    
 	CamelException ex = CAMEL_EXCEPTION_INITIALISER;    
 	CamelFolderInfo *iter=NULL; guint32 flags; CamelStore *store;
-
+	
 	g_assert (TNY_IS_LIST (list));
 	g_assert (CAMEL_IS_SESSION (apriv->session));
 
@@ -1436,6 +1481,8 @@ tny_camel_store_account_get_folders_async_destroyer (gpointer thr_user_data)
 	if (info->err)
 		g_error_free (info->err);
 
+	camel_object_unref (info->session);
+
 	return;
 }
 
@@ -1481,6 +1528,8 @@ tny_camel_store_account_get_folders_async_cancelled_destroyer (gpointer thr_user
 	if (info->err)
 		g_error_free (info->err);
 
+	camel_object_unref (info->session);
+
 	return;
 }
 
@@ -1514,6 +1563,8 @@ tny_camel_store_account_get_folders_async_default (TnyFolderStore *self, TnyList
 	/* Idle info for the callbacks */
 	info = g_slice_new0 (GetFoldersInfo);
 	info->session = apriv->session;
+	camel_object_ref (info->session);
+
 	info->err = NULL;
 	info->self = self;
 	info->list = list;
@@ -1906,6 +1957,7 @@ tny_camel_store_account_queue_going_online_destroy (gpointer user_data)
 
 	camel_object_unref (info->service);
 	g_object_unref (info->self);
+
 	camel_object_unref (info->session);
 
 	return;
@@ -1936,6 +1988,7 @@ tny_camel_store_account_queue_going_online_cancelled_destroy (gpointer user_data
 	/* thread reference */
 	camel_object_unref (info->service);
 	g_object_unref (info->self);
+
 	camel_object_unref (info->session);
 
 	return;
