@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with self library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -48,6 +48,7 @@
 #include <tny-status.h>
 #define TINYMAIL_ENABLE_PRIVATE_API
 #include "tny-common-priv.h"
+#include "tny-camel-common-priv.h"
 #undef TINYMAIL_ENABLE_PRIVATE_API
 
 #include "tny-session-camel-priv.h"
@@ -118,7 +119,7 @@ get_password_idle_func (gpointer user_data)
 }
 
 static void
-get_password_destroy_func (gpointer user_data)
+get_password_destroy(gpointer user_data)
 {
 	HadToWaitForPasswordInfo *info = (HadToWaitForPasswordInfo *) user_data;
 
@@ -182,10 +183,10 @@ tny_session_camel_get_password (CamelSession *session, CamelService *service, co
 		if (g_main_depth () == 0) {
 			g_idle_add_full (G_PRIORITY_DEFAULT, 
 				get_password_idle_func, info, 
-				get_password_destroy_func);
+				get_password_destroy);
 		} else {
 			get_password_idle_func (info);
-			get_password_destroy_func (info);
+			get_password_destroy(info);
 		}
 
 		/* Wait on the queue for the mainloop callback to be finished */
@@ -210,12 +211,14 @@ tny_session_camel_get_password (CamelSession *session, CamelService *service, co
 
 	if (cancel || retval == NULL) 
 	{
-		TnyCamelAccountPriv *apriv = TNY_CAMEL_ACCOUNT_GET_PRIVATE (account);
+		if (account) {
+			TnyCamelAccountPriv *apriv = TNY_CAMEL_ACCOUNT_GET_PRIVATE (account);
 
-		if (CAMEL_IS_DISCO_STORE (apriv->service)) {
-			CamelException tex = CAMEL_EXCEPTION_INITIALISER;
-			camel_disco_store_set_status (CAMEL_DISCO_STORE (apriv->service),
-					CAMEL_DISCO_STORE_OFFLINE, &tex);
+			if (CAMEL_IS_DISCO_STORE (apriv->service)) {
+				CamelException tex = CAMEL_EXCEPTION_INITIALISER;
+				camel_disco_store_set_status (CAMEL_DISCO_STORE (apriv->service),
+						CAMEL_DISCO_STORE_OFFLINE, &tex);
+			}
 		}
 
 		camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
@@ -272,7 +275,7 @@ forget_password_idle_func (gpointer user_data)
 }
 
 static void
-forget_password_destroy_func (gpointer user_data)
+forget_password_destroy(gpointer user_data)
 {
 	HadToWaitForForgetInfo *info = (HadToWaitForForgetInfo *) user_data;
 
@@ -319,10 +322,10 @@ tny_session_camel_forget_password (CamelSession *session, CamelService *service,
 		if (g_main_depth () == 0) {
 			g_idle_add_full (G_PRIORITY_DEFAULT, 
 				forget_password_idle_func, info, 
-				forget_password_destroy_func);
+				forget_password_destroy);
 		} else {
 			forget_password_idle_func (info); 
-			forget_password_destroy_func (info);
+			forget_password_destroy(info);
 		}
 
 		/* Wait on the queue for the mainloop callback to be finished */
@@ -371,7 +374,7 @@ alert_idle_func (gpointer user_data)
 	info->retval = tny_account_store_alert (
 		(TnyAccountStore*) self->priv->account_store, 
 		account, info->tnytype, info->question, 
-		(const GError *) info->err);
+		info->err);
 
 	tny_lockable_unlock (self->priv->ui_lock);
 
@@ -379,7 +382,7 @@ alert_idle_func (gpointer user_data)
 }
 
 static void
-alert_destroy_func (gpointer user_data)
+alert_destroy(gpointer user_data)
 {
 	HadToWaitForAlertInfo *info = (HadToWaitForAlertInfo *) user_data;
 
@@ -418,10 +421,10 @@ tny_session_camel_do_an_error (TnySessionCamel *self, TnyAccount *account, TnyAl
 	if (g_main_depth () == 0) {
 		g_idle_add_full (G_PRIORITY_DEFAULT, 
 			alert_idle_func, info, 
-			alert_destroy_func);
+			alert_destroy);
 	} else {
 		alert_idle_func (info); 
-		alert_destroy_func (info);
+		alert_destroy(info);
 	}
 
 	/* Wait on the queue for the mainloop callback to be finished */
@@ -489,14 +492,7 @@ tny_session_camel_alert_user (CamelSession *session, CamelSessionAlertType type,
 			break;
 		}
 
-		if (ex->id == CAMEL_EXCEPTION_USER_CANCEL)
-			g_set_error (&err, TNY_ACCOUNT_STORE_ERROR, 
-				TNY_ACCOUNT_STORE_ERROR_CANCEL_ALERT, 
-				"The connecting was canceled");
-		else
-			g_set_error (&err, TNY_ACCOUNT_STORE_ERROR, 
-				_tny_camel_account_get_tny_error_code_for_camel_exception_id (ex), 
-				camel_exception_get_description (ex));
+		_tny_camel_exception_to_tny_error (ex, &err);
 
 		retval = tny_session_camel_do_an_error (self, account, tnytype, 
 			TRUE, err);
@@ -538,12 +534,12 @@ mail_tool_uri_to_folder (CamelSession *session, const char *uri, guint32 flags,
 	CamelURL *url;
 	CamelStore *store = NULL;
 	CamelFolder *folder = NULL;
-	int offset = 0;
+	/*int offset = 0;*/
 	char *curi = NULL;
 
 	g_return_val_if_fail (uri != NULL, NULL);
 	
-	url = camel_url_new (uri + offset, ex);
+	url = camel_url_new (uri /*+ offset*/, ex);
 
 	if (G_UNLIKELY (!url))
 	{
@@ -551,7 +547,7 @@ mail_tool_uri_to_folder (CamelSession *session, const char *uri, guint32 flags,
 		return NULL;
 	}
 
-	store = (CamelStore *)camel_session_get_service(session, uri+offset, CAMEL_PROVIDER_STORE, ex);
+	store = (CamelStore *)camel_session_get_service(session, uri /* + offset */, CAMEL_PROVIDER_STORE, ex);
 	if (G_LIKELY (store))
 	{
 		const char *name;
@@ -566,7 +562,7 @@ mail_tool_uri_to_folder (CamelSession *session, const char *uri, guint32 flags,
 				name = "";
 		}
 
-		if (offset) 
+		/*if (offset) 
 		{
 			if (offset == 7)
 				folder = (CamelFolder*)camel_store_get_trash (store, ex);
@@ -574,7 +570,7 @@ mail_tool_uri_to_folder (CamelSession *session, const char *uri, guint32 flags,
 				folder = (CamelFolder*)camel_store_get_junk (store, ex);
 			else
 				g_assert (FALSE);
-		} else
+		} else*/
 			folder = (CamelFolder*)camel_store_get_folder (store, name, flags, ex);
 		camel_object_unref (store);
 	}
@@ -704,8 +700,9 @@ _tny_session_camel_register_account (TnySessionCamel *self, TnyCamelAccount *acc
 }
 
 static void
-on_account_connect_done (TnySessionCamel *self, TnyCamelAccount *account, GError *err, gpointer user_data)
+on_account_connect_done (TnyCamelAccount *account, gboolean canceled, GError *err, gpointer user_data)
 {
+	TnySessionCamel *self = (TnySessionCamel *) user_data;
 	TnyCamelAccountPriv *apriv = TNY_CAMEL_ACCOUNT_GET_PRIVATE (account);
 
 	/* This one happens when a account is finished with connecting. On 
@@ -723,13 +720,17 @@ on_account_connect_done (TnySessionCamel *self, TnyCamelAccount *account, GError
 
 	apriv->is_ready = TRUE;
 
-	if (err) {
-		if (strcmp (err->message, "cancel")) {
-			tny_session_camel_do_an_error (self, TNY_ACCOUNT (account), 
-				TNY_ALERT_TYPE_ERROR, FALSE, err);
-		}
+	if (!canceled && err) {
+
+		/* It seems err is forgotten here ... if the disk is full ! */
+		tny_account_store_alert (
+			(TnyAccountStore*) self->priv->account_store, 
+			TNY_ACCOUNT (account), TNY_ALERT_TYPE_ERROR, FALSE, 
+			err);
 	}
 
+	camel_object_unref (self);
+
 	return;
 }
 
@@ -769,10 +770,12 @@ tny_session_queue_going_online_for_account (TnySessionCamel *self, TnyCamelAccou
 	 * go online to the account's queue. This is implemented in a protected
 	 * method in TnyCamelStoreAccount. Go take a look! */
 
-	if (TNY_IS_CAMEL_STORE_ACCOUNT (account)) 
+	if (TNY_IS_CAMEL_STORE_ACCOUNT (account)) {
+		camel_object_ref (self);
 		_tny_camel_store_account_queue_going_online (
 			TNY_CAMEL_STORE_ACCOUNT (account), self, online, 
-			on_account_connect_done, NULL);
+			on_account_connect_done, self);
+	}
 
 	/* Else, if it's a transport account, we don't have any transport 
 	 * account implementations that actually need to go online at this 
@@ -1095,7 +1098,10 @@ tny_session_camel_finalise (CamelObject *object)
 		g_free (priv->camel_dir);
 
 	g_mutex_free (priv->queue_lock);
-	g_static_rec_mutex_free (priv->current_accounts_lock);
+
+	/* g_static_rec_mutex_free (priv->current_accounts_lock); */
+	g_free (priv->current_accounts_lock);
+	priv->current_accounts_lock = NULL;
 
 	g_slice_free (TnySessionCamelPriv, self->priv);
 
