@@ -16,16 +16,16 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with self library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 typedef struct _TnyGnomeDevicePriv TnyGnomeDevicePriv;
 
 struct _TnyGnomeDevicePriv
 {
-	gboolean forced, fset;
-    
+	gboolean forced, fset, current_state;
+
 #ifdef GNOME
 	gboolean invnm;
 	guint callback_id;
