@@ -154,13 +154,13 @@ gconf_listener_account_changed (GConfClient *client, guint cnxn_id,
 	TnyMaemoAccountStorePriv *priv = TNY_MAEMO_ACCOUNT_STORE_GET_PRIVATE (self);
 
 	gchar *key = entry->key ? g_strdup (entry->key) : g_strdup ("");
-	gchar *ptr = strrchr (key, '/'); 
+	gchar *ptr = strrchr (key, '/');
 
-	if (ptr)
+	if (ptr) {
 		ptr++;
-
-	if (!strcmp (ptr, "count"))
-		kill_stored_accounts (priv);
+		if (!strcmp (ptr, "count"))
+			kill_stored_accounts (priv);
+	}
 
 	g_free (key);
 
@@ -340,19 +340,14 @@ tny_maemo_account_store_find_account (TnyAccountStore *self, const gchar *url_st
 	if (!priv->accounts)
 		load_accounts (self);
 
-	if (priv->accounts)
-	{
+	if (priv->accounts) {
 		GList *copy = priv->accounts;
-		while (copy)
-		{
+		while (copy) {
 			TnyAccount *account = copy->data;
-
-			if (tny_account_matches_url_string (account, url_string))
-			{
+			if (tny_account_matches_url_string (account, url_string)) {
 				found = TNY_ACCOUNT (g_object_ref (G_OBJECT (account)));
 				break;
 			}
-
 			copy = g_list_next (copy);
 		}
 	}
@@ -366,21 +361,7 @@ tny_maemo_account_store_get_cache_dir (TnyAccountStore *self)
 {
 	TnyMaemoAccountStorePriv *priv = TNY_MAEMO_ACCOUNT_STORE_GET_PRIVATE (self);
 
-	if (G_UNLIKELY (!priv->cache_dir))
-	{
-		/* Note that there's no listener for this key. If it changes,
-		   the camelsession should be destroyed and rebuild from scratch.
-		   Which basically means reloading the accounts aswell. 
-		  
-		   So say you're a nut who wants this key to be updatable at 
-		   runtime, you'll have to unload all the accounts here, and of
-		   course reload them. All the functionality for that is already
-		   available. Perhaps I should just do it ... hmm, maybe another
-		   day. Soon. Perhaps. I don't know. Probably . . . . bleh. 
-
-		   Oh and, not to forget! You should probably also move the old
-		   cache location to the new one. Or cleanup the old one. */
-
+	if (!priv->cache_dir) {
 		gchar *cache_dir = gconf_client_get_string (priv->client, 
 			"/apps/tinymail/cache_dir", NULL);
 		priv->cache_dir = g_build_filename (g_get_home_dir (), 
@@ -405,20 +386,15 @@ tny_maemo_account_store_get_accounts (TnyAccountStore *self, TnyList *list, TnyG
 	if (priv->accounts)
 	{
 		GList *copy = priv->accounts;
-		while (copy)
-		{
+		while (copy) {
 			TnyAccount *account = copy->data;
-
-			if (types == TNY_ACCOUNT_STORE_BOTH || types == TNY_ACCOUNT_STORE_STORE_ACCOUNTS)
-			{
+			if (types == TNY_ACCOUNT_STORE_BOTH || types == TNY_ACCOUNT_STORE_STORE_ACCOUNTS) {
 				if (TNY_IS_STORE_ACCOUNT (account))
 					tny_list_prepend (list, (GObject*)account);
-			} else if (types == TNY_ACCOUNT_STORE_BOTH || types == TNY_ACCOUNT_STORE_TRANSPORT_ACCOUNTS)
-			{
+			} else if (types == TNY_ACCOUNT_STORE_BOTH || types == TNY_ACCOUNT_STORE_TRANSPORT_ACCOUNTS) {
 				if (TNY_IS_TRANSPORT_ACCOUNT (account))
 					tny_list_prepend (list, (GObject*)account);
 			}
-
 			copy = g_list_next (copy);
 		}
 	}
@@ -459,52 +435,12 @@ or
 
 */
 
-#if 0 /* Not used. */
-static void
-tny_maemo_account_store_add_account (TnyAccountStore *self, TnyAccount *account, const gchar *type)
-{
-	TnyMaemoAccountStorePriv *priv = TNY_MAEMO_ACCOUNT_STORE_GET_PRIVATE (self);
-	gchar *key = NULL;
-	gint count = gconf_client_get_int (priv->client, "/apps/tinymail/accounts/count", NULL);
-
-	g_assert (TNY_IS_ACCOUNT (account));
-
-	count++;
-
-	key = g_strdup_printf ("/apps/tinymail/accounts/%d/hostname", count);
-	gconf_client_set_string (priv->client, (const gchar*) key, 
-		tny_account_get_hostname (account), NULL);
-	g_free (key); 
-
-	key = g_strdup_printf ("/apps/tinymail/accounts/%d/proto", count);
-	gconf_client_set_string (priv->client, (const gchar*) key, 
-		tny_account_get_proto (account), NULL);
-	g_free (key); 
-
-	key = g_strdup_printf ("/apps/tinymail/accounts/%d/type", count);
-	gconf_client_set_string (priv->client, (const gchar*) key, type, NULL);
-	g_free (key); 
-
-	key = g_strdup_printf ("/apps/tinymail/accounts/%d/user", count);
-	gconf_client_set_string (priv->client, (const gchar*) key, 
-		tny_account_get_user (account), NULL);
-	g_free (key); 
-
-	gconf_client_set_int (priv->client, "/apps/tinymail/accounts/count", 
-		count, NULL);
-
-	return;
-}
-#endif
-
-
 
 static TnyDevice*
 tny_maemo_account_store_get_device (TnyAccountStore *self)
 {
 	TnyMaemoAccountStorePriv *priv = TNY_MAEMO_ACCOUNT_STORE_GET_PRIVATE (self);
-
-	return g_object_ref (G_OBJECT (priv->device));
+	return g_object_ref (priv->device);
 }
 
 /**
@@ -533,6 +469,7 @@ tny_maemo_account_store_instance_init (GTypeInstance *instance, gpointer g_class
 	TnyMaemoAccountStorePriv *priv = TNY_MAEMO_ACCOUNT_STORE_GET_PRIVATE (self);
 	TnyPlatformFactory *platfact;
 
+	priv->cache_dir = NULL;
 	priv->accounts = NULL;
 	priv->client = gconf_client_get_default ();
 
@@ -561,7 +498,7 @@ tny_maemo_account_store_finalize (GObject *object)
 
 	kill_stored_accounts (priv);
 
-	if (G_LIKELY (priv->cache_dir))
+	if (priv->cache_dir)
 		g_free (priv->cache_dir);
 
 	(*parent_class->finalize) (object);
