@@ -85,15 +85,18 @@ conic_emit_status_idle (gpointer user_data)
 	
 	info  = (EmitStatusInfo *) user_data;
 
+	g_debug ("%s: destroying %p (idle)", __FUNCTION__, user_data);
+
 	/* We lock the gdk thread because tinymail wants implementations to do
 	 * this before emitting signals from within a g_idle_add_full callback.
 	 * See http://www.tinymail.org/trac/tinymail/wiki/HowTnyLockable */
-
 	gdk_threads_enter ();
 	g_signal_emit (info->self, tny_device_signals [TNY_DEVICE_CONNECTION_CHANGED],
 		0, info->status);
 	gdk_threads_leave ();
 
+	g_debug ("%s: emitted tny-device-connection-changed signal", __FUNCTION__);
+
 	return FALSE;
 }
 
@@ -106,10 +109,14 @@ conic_emit_status_destroy (gpointer user_data)
 
 	info = (EmitStatusInfo *) user_data;
 	
-	if (G_IS_OBJECT(info->self))
+	g_debug ("%s: destroying status info (%p)", __FUNCTION__, user_data);
+
+	if (G_IS_OBJECT(info->self)) {
 		g_object_unref (info->self);
-	
-	g_slice_free (EmitStatusInfo, info);
+		g_slice_free (EmitStatusInfo, info);
+		g_debug ("%s: destroyed %p", __FUNCTION__, user_data);
+	} else
+		g_warning ("%s: BUG: not a valid info", __FUNCTION__);
 }
 
 static void 
@@ -129,6 +136,9 @@ conic_emit_status (TnyDevice *self, gboolean status)
 	if (!dnsmasq_has_resolv())
 		time = 5000;
 
+	g_debug ("%s: emitting status (%s, %s) in %d ms", 
+		   __FUNCTION__, info, status ? "true" : "false", time);
+
 	g_timeout_add_full (G_PRIORITY_DEFAULT, time, conic_emit_status_idle,
 		info, conic_emit_status_destroy);
 }
@@ -199,9 +209,13 @@ handle_connect (TnyMaemoConicDevice *self, int con_err, int con_state)
 		if (err)
 			g_error_free (err);
 
-		g_object_unref (info->self);
-		g_free (info->iap_id);
-		g_slice_free (ConnectInfo, info);
+		if (G_IS_OBJECT(info->self)) {
+			g_object_unref (info->self);
+			g_free (info->iap_id);
+			info->iap_id = NULL;
+			g_slice_free (ConnectInfo, info);
+		} else
+			g_warning ("%s: BUG: info seems b0rked", __FUNCTION__);
 	}
 }
 
@@ -211,7 +225,7 @@ typedef struct {
 	int con_state;
 } HandleConnInfo;
 
-static gboolean /* FIXME: this function will always return FALSE, is that correct? */
+static gboolean 
 handle_con_idle (gpointer data)
 {
 	HandleConnInfo *info;
@@ -233,10 +247,11 @@ handle_con_idle_destroy (gpointer data)
 	
 	info = (HandleConnInfo *) data;
 
-	if (G_IS_OBJECT(info->self))
+	if (G_IS_OBJECT(info->self)) {
 		g_object_unref (info->self);
-	
-	g_slice_free (HandleConnInfo, data); 
+		g_slice_free (HandleConnInfo, data);
+	} else
+		g_warning ("%s: BUG: data seems b0rked", __FUNCTION__);
 }
 
 
@@ -321,13 +336,16 @@ on_connection_event (ConIcConnection *cnx, ConIcConnectionEvent *event, gpointer
 				 handle_con_idle_destroy);
 	}
 
-	if (emit)
+	if (emit) {
+		g_debug ("%s: emiting is_online (%s)",
+			 __FUNCTION__, is_online ? "true" : "false");
 		conic_emit_status (TNY_DEVICE (device), is_online);
+	}
 }
 
 
 /**
- * tny_maemo_conic_device_connect:
+ * tny_maemo_conic_device_connect_async:
  * @self: a #TnyDevice object
  * @iap_id: the id of the Internet Access Point (IAP), or NULL for 'any;
  * @user_requested: whether or not the connection was automatically requested or by an user action
@@ -386,11 +404,16 @@ tny_maemo_conic_device_connect_async (TnyMaemoConicDevice *self,
 
 	if (request_failed) {
 		priv->connect_slot = NULL;
-		if (info->callback)
-			info->callback (info->self, iap_id, FALSE, err, info->user_data);
-		g_free (info->iap_id);
-		g_object_unref (info->self);
-		g_slice_free (ConnectInfo, info);
+		if (G_IS_OBJECT(info->self)) {
+			if (info->callback)
+				info->callback (info->self, iap_id, FALSE, err, info->user_data);
+			g_free (info->iap_id);
+			info->iap_id = NULL;
+			g_object_unref (info->self);
+			info->self = NULL;
+			g_slice_free (ConnectInfo, info);
+		} else 
+			g_warning ("%s: BUG: info seems b0rked", __FUNCTION__);
 	}
 }
 
@@ -417,12 +440,13 @@ tny_maemo_conic_device_disconnect (TnyMaemoConicDevice *self, const gchar* iap_i
 
 	if (iap_id) {
 		if (!con_ic_connection_disconnect_by_id (priv->cnx, iap_id)) {
-			g_warning ("Could not send disconnect_by_id dbus message");
+			g_warning ("%s: disconnect_by_id failed (%s)", 
+				   __FUNCTION__, iap_id);
 			return FALSE;
 		}
 	} else {
-		/* don't try to disconnect if iap_id==NULL, or conic will crash... */
-		g_warning ("Could not send disconnect dbus message");
+		if (!con_ic_connection_disconnect (priv->cnx))
+			g_warning ("Could not send disconnect dbus message");
 		return FALSE;
 	}
 
@@ -509,6 +533,16 @@ tny_maemo_conic_device_get_iap_list (TnyMaemoConicDevice *self)
 	return con_ic_connection_get_all_iaps (priv->cnx);
 }
 
+static void
+unref_gobject (GObject *obj)
+{
+	if (G_IS_OBJECT(obj))
+		g_object_unref(obj);
+	else
+		g_warning ("%s: not a valid GObject (%p)",
+			   __FUNCTION__, obj);
+}
+
 
 /**
  * tny_maemo_conic_device_free_iap_list:
@@ -520,7 +554,7 @@ tny_maemo_conic_device_get_iap_list (TnyMaemoConicDevice *self)
 void
 tny_maemo_conic_device_free_iap_list (TnyMaemoConicDevice *self, GSList* cnx_list)
 {
-	g_slist_foreach (cnx_list, (GFunc)g_object_unref, NULL);
+	g_slist_foreach (cnx_list, (GFunc)unref_gobject, NULL);
 	g_slist_free (cnx_list);
 }
 
@@ -543,8 +577,11 @@ tny_maemo_conic_device_force_online (TnyDevice *device)
 	priv->is_online = TRUE;
 
 	/* Signal if it changed: */
-	if (!already_online)
+	if (!already_online) {
+		g_debug ("%s: emiting connection-changed signal",
+			 __FUNCTION__);
 		g_signal_emit (device, tny_device_signals [TNY_DEVICE_CONNECTION_CHANGED], 0, TRUE);
+	}
 }
 
 
@@ -586,15 +623,18 @@ tny_maemo_conic_device_instance_init (GTypeInstance *instance, gpointer g_class)
 
 	/* We should not have a real is_online, based on what libconic has told us: */
 
-	priv->forced = FALSE;
-	priv->iap = NULL;
-	priv->is_online = dnsmasq_has_resolv ();
+	priv->forced       = FALSE;
+	priv->iap          = NULL;
+	priv->is_online    = FALSE;
 	priv->connect_slot = NULL;
-	
+	priv->loop         = NULL;
+
 	priv->cnx = con_ic_connection_new ();
 
-	if (!priv->cnx)
-		g_warning ("con_ic_connection_new failed. The TnyMaemoConicDevice will be useless.");
+	if (!priv->cnx) {	
+		g_warning ("%s: con_ic_connection_new failed.", __FUNCTION__);
+		return;
+	}
 
 	/* This might be necessary to make the connection object actually emit 
 	 * the signal, though the documentation says that they should be sent 
@@ -607,10 +647,10 @@ tny_maemo_conic_device_instance_init (GTypeInstance *instance, gpointer g_class)
 
 	/* This will get us in connected state only if there is already a connection.
 	 * thus, this will setup our state correctly when we receive the signals. */
-	if (!con_ic_connection_connect (priv->cnx, CON_IC_CONNECT_FLAG_AUTOMATICALLY_TRIGGERED))
-		g_warning ("could not send connect dbus message");
-
-	return;
+	if (!con_ic_connection_connect (priv->cnx, 
+					CON_IC_CONNECT_FLAG_AUTOMATICALLY_TRIGGERED))
+		g_warning ("%s: could not send connect dbus message",
+			__FUNCTION__);	
 }
 
 
@@ -630,21 +670,26 @@ tny_maemo_conic_device_finalize (GObject *obj)
 {
 	TnyMaemoConicDevicePriv *priv;
 
-	g_return_if_fail (obj && G_IS_OBJECT(obj));
+	g_return_if_fail (TNY_IS_MAEMO_CONIC_DEVICE(obj));
+
+	g_debug ("%s: shutting the device down...", __FUNCTION__);
 	
 	priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (obj);
-	g_signal_handler_disconnect (obj, priv->signal1);
+
+	if (g_signal_handler_is_connected (priv->cnx, priv->signal1))
+		g_signal_handler_disconnect (priv->cnx, priv->signal1);
 
 	if (priv->cnx && CON_IC_IS_CONNECTION(priv->cnx)) {
-		tny_maemo_conic_device_disconnect (TNY_MAEMO_CONIC_DEVICE(obj),priv->iap);
+		tny_maemo_conic_device_disconnect (TNY_MAEMO_CONIC_DEVICE(obj),
+						   priv->iap);
 		g_object_unref (priv->cnx);
 		priv->cnx = NULL;
-	}
-
-	if (priv->iap) {
-		g_free (priv->iap);
-		priv->iap = NULL;
-	}
+	} else
+		g_warning ("%s: BUG: priv->cnx is not a valid connection",
+			   __FUNCTION__);
+	
+	g_free (priv->iap);
+	priv->iap = NULL;
 
 	(*parent_class->finalize) (obj);
 }
@@ -745,6 +790,9 @@ tny_maemo_conic_device_connect (TnyMaemoConicDevice *self,
 	gboolean request_failed = FALSE;
 	ConIcConnectFlags flags;
 
+	g_warning ("%s: you should tny_maemo_conic_device_connect_async",
+		   __FUNCTION__);
+
 	g_return_val_if_fail (TNY_IS_DEVICE(self), FALSE);
 	priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (self);
 
@@ -770,7 +818,7 @@ tny_maemo_conic_device_connect (TnyMaemoConicDevice *self,
 	}
 
 	if (request_failed) {
-		g_object_unref (priv->loop);
+		g_main_loop_unref (priv->loop);
 		priv->loop = NULL;
 	}
 	
@@ -784,8 +832,10 @@ tny_maemo_conic_device_connect (TnyMaemoConicDevice *self,
 		g_main_loop_run (priv->loop);
 	GDK_THREADS_ENTER();
 
-	g_main_loop_unref (priv->loop);
-	priv->loop = NULL;
+	if (priv->loop) {
+		g_main_loop_unref (priv->loop);
+		priv->loop = NULL;
+	}
 
 	return priv->is_online;
 }
