@@ -156,33 +156,31 @@ tny_folder_monitor_update (TnyFolderObserver *self, TnyFolderChange *change)
 static gboolean 
 uid_matcher (TnyList *list, GObject *item, gpointer match_data)
 {
-	const char *uid = tny_header_get_uid ((TnyHeader *) item);
+	gboolean result = FALSE;
+	char *uid = tny_header_dup_uid ((TnyHeader *) item);
 
 	if (uid && !strcmp (uid, (const char*) match_data))
- 		return TRUE;
+ 		result = TRUE;
 
-	return FALSE;
+	g_free (uid);
+
+	return result;
 }
 
 
 static void
-foreach_list_add_header (TnyFolderMonitorPriv *priv, TnyHeader *header, gint length)
+foreach_list_add_header (TnyFolderMonitorPriv *priv, TnyHeader *header, gint length, gboolean check_duplicates)
 {
-	TnyIterator *iter;
-	const gchar *uid = (length<100)?tny_header_get_uid (header):NULL;
+	TnyIterator *iter = tny_list_create_iterator (priv->lists);
 
-	iter = tny_list_create_iterator (priv->lists);
 	while (!tny_iterator_is_done (iter))
 	{
 		TnyList *list = TNY_LIST (tny_iterator_get_current (iter));
+		gchar *uid = tny_header_dup_uid (header);
 
-		/* If the length is larger than 100, we're bulk receiving
-		 * a lot of headers. It's probably not an IDLE situation and
-		 * we can therefore assume that we'll for sure wont have 
-		 * duplicates. */
-
-		if (uid)
+		if (check_duplicates && uid)
 			tny_list_remove_matches (list, uid_matcher, (gpointer) uid); 
+		g_free (uid);
 
 		tny_list_prepend (list, (GObject *) header);
 
@@ -250,7 +248,8 @@ tny_folder_monitor_update_default (TnyFolderObserver *self, TnyFolderChange *cha
 		while (!tny_iterator_is_done (iter))
 		{
 			TnyHeader *header = TNY_HEADER (tny_iterator_get_current (iter));
-			foreach_list_add_header (priv, header, length);
+			foreach_list_add_header (priv, header, length, 
+				tny_folder_change_get_check_duplicates (change));
 			g_object_unref (header);
 			tny_iterator_next (iter);
 		}
@@ -271,11 +270,9 @@ tny_folder_monitor_update_default (TnyFolderObserver *self, TnyFolderChange *cha
 		{
 			const gchar *uid;
 			TnyHeader *header = TNY_HEADER (tny_iterator_get_current (iter));
-			uid = tny_header_get_uid (header);
-			if (uid) {
-				gchar *tuid = g_strdup (uid);
-				g_ptr_array_add (array, tuid);
-			}
+			uid = tny_header_dup_uid (header);
+			if (uid)
+				g_ptr_array_add (array, (gpointer) uid);
 			g_object_unref (header);
 			tny_iterator_next (iter);
 		}
