@@ -339,6 +339,7 @@ tny_folder_set_msg_receive_strategy (TnyFolder *self, TnyMsgReceiveStrategy *st)
  * If @new_name already exists in @into and @err is not NULL, then an error will
  * be set in @err and no further action will be performed.
  * 
+ * Deprecated: 1.0: Use tny_folder_copy_async in stead
  * returns: (null-ok) (caller-owns): a new folder instance to whom was copied or NULL
  * since: 1.0
  * audience: application-developer
@@ -568,6 +569,7 @@ tny_folder_set_msg_remove_strategy (TnyFolder *self, TnyMsgRemoveStrategy *st)
  * g_object_unref (folder);
  * </programlisting></informalexample>
  *
+ * Deprecated: 1.0: Use tny_folder_sync_async in stead
  * since: 1.0
  * audience: application-developer
  **/
@@ -600,6 +602,10 @@ tny_folder_sync (TnyFolder *self, gboolean expunge, GError **err)
  * messages (the ones marked with TNY_HEADER_FLAG_DELETED) as well if @expunge
  * is TRUE.
  *
+ * This is a cancelable operation which means that if another cancelable 
+ * operation executes, this operation will be aborted. Being aborted means that
+ * the callback will still be called, but with cancelled=TRUE.
+ *
  * since: 1.0
  * audience: application-developer
  **/
@@ -676,6 +682,7 @@ tny_folder_add_msg_async (TnyFolder *self, TnyMsg *msg, TnyFolderCallback callba
  * Folder observers of @self will get a header-added event caused by this
  * action.
  *
+ * Deprecated: 1.0: Use tny_folder_add_msg_async in stead
  * since: 1.0
  * audience: application-developer
  **/
@@ -732,6 +739,7 @@ tny_folder_add_msg (TnyFolder *self, TnyMsg *msg, GError **err)
  * }
  * </programlisting></informalexample>
  *
+ * Deprecated: 1.0: Use tny_folder_remove_msgs_async in stead
  * since: 1.0
  * audience: application-developer
  **/
@@ -763,6 +771,7 @@ tny_folder_remove_msg (TnyFolder *self, TnyHeader *header, GError **err)
  * Folder observers of @self will get only one header-removed trigger caused
  * by this action (with all the removed headers in the #TnyFolderChange).
  *
+ * Deprecated: 1.0: Use tny_folder_remove_msgs_async in stead
  * since: 1.0
  * audience: application-developer
  **/
@@ -780,6 +789,41 @@ tny_folder_remove_msgs (TnyFolder *self, TnyList *headers, GError **err)
 	return;
 }
 
+
+
+/**
+ * tny_folder_remove_msgs_async:
+ * @self: a #TnyFolder
+ * @headers: a #TnyList with #TnyHeader items of the messages to remove
+ * @callback: (null-ok): a #TnyFolderCallback or NULL
+ * @status_callback: (null-ok): a #TnyStatusCallback or NULL
+ * @user_data: (null-ok): user data that will be passed to the callbacks
+ *
+ * Remove messages from a folder. It will use the #TnyMsgRemoveStrategy of 
+ * @self to perform the removal itself. For more details, check out the 
+ * documentation of the used #TnyMsgRemoveStrategy type. The default 
+ * implementation for libtinymail-camel is the #TnyCamelMsgRemoveStrategy.
+ *
+ * Folder observers of @self will get only one header-removed trigger caused
+ * by this action (with all the removed headers in the #TnyFolderChange).
+ *
+ * since: 1.0
+ * audience: application-developer
+ **/
+void 
+tny_folder_remove_msgs_async (TnyFolder *self, TnyList *headers, TnyFolderCallback callback, TnyStatusCallback status_callback, gpointer user_data)
+{
+#ifdef DBC /* require */
+	g_assert (TNY_IS_FOLDER (self));
+	g_assert (headers);
+	g_assert (TNY_IS_LIST (headers));
+	g_assert (TNY_FOLDER_GET_IFACE (self)->remove_msgs_async!= NULL);
+#endif
+
+	TNY_FOLDER_GET_IFACE (self)->remove_msgs_async(self, headers, callback, status_callback, user_data);
+	return;
+}
+
 /**
  * tny_folder_refresh_async:
  * @self: a #TnyFolder
@@ -835,6 +879,10 @@ tny_folder_remove_msgs (TnyFolder *self, TnyList *headers, GError **err)
  *          status_update_cb, NULL); 
  * </programlisting></informalexample>
  *
+ * This is a cancelable operation which means that if another cancelable 
+ * operation executes, this operation will be aborted. Being aborted means that
+ * the callback will still be called, but with cancelled=TRUE.
+ *
  * since: 1.0
  * audience: application-developer
  **/
@@ -872,6 +920,7 @@ tny_folder_refresh_async (TnyFolder *self, TnyFolderCallback callback, TnyStatus
  * This function can take a very long time. You are advised to consider using
  * tny_folder_refresh_async() in stead.
  *
+ * Deprecated: 1.0: Use tny_folder_refresh_async in stead
  * since: 1.0
  * audience: application-developer
  **/
@@ -1026,6 +1075,7 @@ tny_folder_get_account (TnyFolder *self)
  * with this API. You must only use instances that you got from 
  * tny_folder_get_headers().
  *
+ * Deprecated: 1.0: Use tny_folder_transfer_msgs_async in stead
  * since: 1.0
  * audience: application-developer
  **/
@@ -1063,6 +1113,10 @@ tny_folder_transfer_msgs (TnyFolder *self, TnyList *headers, TnyFolder *folder_d
  * with this API. You must only use instances that you got from 
  * tny_folder_get_headers().
  *
+ * This is a cancelable operation which means that if another cancelable 
+ * operation executes, this operation will be aborted. Being aborted means that
+ * the callback will still be called, but with cancelled=TRUE.
+ *
  * since: 1.0
  * audience: application-developer
  **/
@@ -1110,6 +1164,7 @@ tny_folder_transfer_msgs_async (TnyFolder *self, TnyList *header_list, TnyFolder
  * other #TnyFolder methods. The message's instance nor its header will receive 
  * updates from the observable folder.
  * 
+ * Deprecated: 1.0: Use tny_folder_get_msg_async in stead
  * returns: (null-ok) (caller-owns): The message instance or NULL
  * since: 1.0
  * audience: application-developer
@@ -1318,6 +1373,10 @@ tny_folder_get_headers (TnyFolder *self, TnyList *headers, gboolean refresh, GEr
  * to a query object will be placed. This will introduce both an API and ABI 
  * breakage.
  *
+ * Only one instance of @tny_folder_get_headers_async for folder @self can run
+ * at the same time. If you call for another, the first will be aborted. This
+ * means that it's callback will be called with cancelled=TRUE.
+ *
  * since: 1.0
  * audience: application-developer
  **/
