@@ -20,6 +20,7 @@
 #include <config.h>
 #include <glib.h>
 #include <glib/gi18n-lib.h>
+#include <string.h>
 
 #include <tny-gnome-keyring-password-getter.h>
 
@@ -73,6 +74,8 @@ tny_gnome_keyring_password_getter_get_password (TnyPasswordGetter *self, const g
 
 			retval = gnome_password_dialog_get_password (dialog);
 
+			mlock (retval, strlen (retval));
+
 			r = gnome_password_dialog_get_remember (dialog);
 
 			if (r == GNOME_PASSWORD_DIALOG_REMEMBER_FOREVER)
