@@ -152,7 +152,6 @@ tny_gtk_header_list_model_get_column_type (GtkTreeModel *self, gint column)
 	return retval;
 }
 
-
 static gboolean
 tny_gtk_header_list_model_get_iter (GtkTreeModel *self, GtkTreeIter *iter, GtkTreePath *path)
 {
@@ -261,11 +260,14 @@ tny_gtk_header_list_model_received_date_sort_func (GtkTreeModel *model, GtkTreeI
 	 * the GPtrArray. This must be quite efficient, as this is called lots
 	 * of times while you are sorting things. */
 
+	g_static_rec_mutex_lock (priv->iterator_lock);
+
 	hdr_a = priv->items->pdata[(gint)a->user_data];
 	hdr_b = priv->items->pdata[(gint)b->user_data];
 
 	recv_a = tny_header_get_date_received (hdr_a);
 	recv_b = tny_header_get_date_received (hdr_b);
+	g_static_rec_mutex_unlock (priv->iterator_lock);
 
 	return (recv_a - recv_b);
 }
@@ -295,12 +297,16 @@ tny_gtk_header_list_model_sent_date_sort_func (GtkTreeModel *model, GtkTreeIter
 	 * the GPtrArray. This must be quite efficient, as this is called lots
 	 * of times while you are sorting things. */
 
+	g_static_rec_mutex_lock (priv->iterator_lock);
+
 	hdr_a = priv->items->pdata[(gint)a->user_data];
 	hdr_b = priv->items->pdata[(gint)b->user_data];
 
 	recv_a = tny_header_get_date_sent (hdr_a);
 	recv_b = tny_header_get_date_sent (hdr_b);
 
+	g_static_rec_mutex_unlock (priv->iterator_lock);
+
 	return (recv_a - recv_b);
 }
 
@@ -346,7 +352,7 @@ tny_gtk_header_list_model_get_value (GtkTreeModel *self, GtkTreeIter *iter, gint
 {
 	TnyGtkHeaderListModelPriv *priv = TNY_GTK_HEADER_LIST_MODEL_GET_PRIVATE (self);
 
-	const gchar *str;
+	gchar *str;
 	gchar *rdate = NULL;
 	gint i;
 
@@ -393,9 +399,9 @@ tny_gtk_header_list_model_get_value (GtkTreeModel *self, GtkTreeIter *iter, gint
 	{
 		case TNY_GTK_HEADER_LIST_MODEL_CC_COLUMN:
 			g_value_init (value, G_TYPE_STRING);
-			str = tny_header_get_cc ((TnyHeader*) priv->items->pdata[i]);
+			str = tny_header_dup_cc ((TnyHeader*) priv->items->pdata[i]);
 			if (str)
-				g_value_set_string (value, str);
+				g_value_take_string (value, str);
 			break;
 		case TNY_GTK_HEADER_LIST_MODEL_DATE_SENT_COLUMN:
 			g_value_init (value, G_TYPE_STRING);
@@ -434,21 +440,21 @@ tny_gtk_header_list_model_get_value (GtkTreeModel *self, GtkTreeIter *iter, gint
 			break;
 		case TNY_GTK_HEADER_LIST_MODEL_TO_COLUMN:
 			g_value_init (value, G_TYPE_STRING);
-			str = tny_header_get_to ((TnyHeader*) priv->items->pdata[i]);
+			str = tny_header_dup_to ((TnyHeader*) priv->items->pdata[i]);
 			if (str)
-				g_value_set_string (value, str);
+				g_value_take_string (value, str);
 			break;
 		case TNY_GTK_HEADER_LIST_MODEL_SUBJECT_COLUMN:
 			g_value_init (value, G_TYPE_STRING);
-			str = tny_header_get_subject ((TnyHeader*) priv->items->pdata[i]);
+			str = tny_header_dup_subject ((TnyHeader*) priv->items->pdata[i]);
 			if (str)
-				g_value_set_string (value, str);
+				g_value_take_string (value, str);
 			break;
 		case TNY_GTK_HEADER_LIST_MODEL_FROM_COLUMN:
 			g_value_init (value, G_TYPE_STRING);
-			str = tny_header_get_from ((TnyHeader*) priv->items->pdata[i]);
+			str = tny_header_dup_from ((TnyHeader*) priv->items->pdata[i]);
 			if (str)
-				g_value_set_string (value, str);
+				g_value_take_string (value, str);
 			break;
 		case TNY_GTK_HEADER_LIST_MODEL_FLAGS_COLUMN:
 			g_value_init (value, G_TYPE_INT);
@@ -660,10 +666,13 @@ notify_views_add (gpointer data)
 	GtkTreePath *path;
 	GtkTreeIter iter;
 
+	g_static_rec_mutex_lock (priv->iterator_lock);
+
 	g_mutex_lock (priv->ra_lock);
 	priv->updating_views++;
 	if (priv->registered >= priv->items->len) {
 		g_mutex_unlock (priv->ra_lock);
+		g_static_rec_mutex_unlock (priv->iterator_lock);
 		return FALSE;
 	}
 
@@ -682,6 +691,7 @@ notify_views_add (gpointer data)
 		needmore = TRUE;
 
 	g_mutex_unlock (priv->ra_lock);
+	g_static_rec_mutex_unlock (priv->iterator_lock);
 
 	gdk_threads_enter();
 
@@ -700,6 +710,20 @@ notify_views_add (gpointer data)
 }
 
 
+static gboolean 
+uid_matcher (TnyList *list, GObject *item, gpointer match_data)
+{
+	gboolean result = FALSE;
+	char *uid = tny_header_dup_uid ((TnyHeader *) item);
+
+	if (uid && !strcmp (uid, (const char*) match_data))
+ 		result = TRUE;
+
+	g_free (uid);
+
+	return result;
+}
+
 
 /* This will be called often while you are in tny_folder_refresh(_async). It can
  * and will be called from a thread, so we must cope with that in case we want 
@@ -711,6 +735,14 @@ tny_gtk_header_list_model_prepend (TnyList *self, GObject* item)
 
 	g_static_rec_mutex_lock (priv->iterator_lock);
 
+	if (priv->no_duplicates) {
+		gchar *uid = tny_header_dup_uid ((TnyHeader *) item);
+		if (uid) {
+			tny_list_remove_matches (self, uid_matcher, (gpointer) uid); 
+			g_free (uid);
+		}
+	}
+
 	/* Prepend something to the list itself. The get_length will auto update
 	 * because that one uses GPtrArray's len property. We are reusing the 
 	 * iterator_lock for locking this out, by the way. */
@@ -948,10 +980,13 @@ tny_gtk_header_list_model_remove_matches (TnyList *self, TnyListMatcher matcher,
 	TnyGtkHeaderListModelPriv *priv = TNY_GTK_HEADER_LIST_MODEL_GET_PRIVATE (self);
 	int i; GList *items = NULL;
 
+	g_static_rec_mutex_lock (priv->iterator_lock);
+
 	for (i=0; i < priv->items->len; i++) {
 		if (matcher (self, priv->items->pdata[i], match_data))
 			items = g_list_append (items, g_object_ref (priv->items->pdata[i]));
 	}
+	g_static_rec_mutex_unlock (priv->iterator_lock);
 
 
 	if (items) 
@@ -1059,12 +1094,30 @@ static void forea (gpointer u, gpointer o)
 }
 #endif
 
+static gboolean
+free_items (gpointer user_data)
+{
+	GPtrArray *copy = user_data;
+
+#ifdef DEBUG_EXTRA
+	g_ptr_array_foreach (copy, (GFunc) forea, NULL);
+#else
+	g_ptr_array_foreach (copy, (GFunc) g_object_unref, NULL);
+#endif
+	g_ptr_array_free (copy, TRUE);
+
+	return FALSE;
+}
+
+static inline void 
+copy_them (gpointer data, gpointer user_data) { g_ptr_array_add ((GPtrArray *) user_data, data); }
 
 static void
 tny_gtk_header_list_model_finalize (GObject *object)
 {
 	TnyGtkHeaderListModel *self = (TnyGtkHeaderListModel *) object;
 	TnyGtkHeaderListModelPriv *priv = TNY_GTK_HEADER_LIST_MODEL_GET_PRIVATE (self);
+	GPtrArray *copy = g_ptr_array_new ();
 
 	g_static_rec_mutex_lock (priv->iterator_lock);
 
@@ -1079,11 +1132,11 @@ tny_gtk_header_list_model_finalize (GObject *object)
 
 	remove_del_timeouts (self);
 
-#ifdef DEBUG_EXTRA
-	g_ptr_array_foreach (priv->items, (GFunc) forea, NULL);
-#else
-	g_ptr_array_foreach (priv->items, (GFunc) g_object_unref, NULL);
-#endif
+	g_ptr_array_foreach (priv->items, (GFunc) copy_them, copy);
+
+	free_items (copy);
+	// g_timeout_add (5*100, free_items, copy);
+
 
 	if (priv->folder)
 		g_object_unref (priv->folder);
@@ -1132,6 +1185,7 @@ tny_gtk_header_list_model_init (TnyGtkHeaderListModel *self)
 					   TnyGtkHeaderListModelPriv));
 	self->priv = priv;
 
+	priv->no_duplicates = FALSE;
 	priv->folder = NULL;
 	priv->iterator_lock = g_new0 (GStaticRecMutex, 1);
 	g_static_rec_mutex_init (priv->iterator_lock);
@@ -1150,6 +1204,48 @@ tny_gtk_header_list_model_init (TnyGtkHeaderListModel *self)
 }
 
 
+
+/**
+ * tny_gtk_header_list_model_get_no_duplicates:
+ * @self: a #TnyGtkHeaderListModel
+ *
+ * Gets whether or not @self allows duplicates of #TnyHeader instances to be
+ * added. The duplicates will be tested by tny_header_dup_uid uniqueness.
+ * 
+ * returns: whether or not to allow duplicates
+ * since: 1.0
+ * audience: application-developer
+ **/
+gboolean 
+tny_gtk_header_list_model_get_no_duplicates (TnyGtkHeaderListModel *self)
+{
+	TnyGtkHeaderListModelPriv *priv = TNY_GTK_HEADER_LIST_MODEL_GET_PRIVATE (self);
+	return priv->no_duplicates;
+}
+
+/**
+ * tny_gtk_header_list_model_set_no_duplicates:
+ * @self: a #TnyGtkHeaderListModel
+ * @setting: whether or not to allow duplicates
+ *
+ * Sets whether or not @self allows duplicates of #TnyHeader instances to be
+ * added. The duplicates will be tested by tny_header_dup_uid uniqueness.
+ * Setting this property to TRUE will negatively impact performance of @self.
+ * It'll also influence behaviour of tny_list_prepend and tny_list_append.
+ *
+ * Default value, therefore, is FALSE.
+ * 
+ * since: 1.0
+ * audience: application-developer
+ **/
+void
+tny_gtk_header_list_model_set_no_duplicates (TnyGtkHeaderListModel *self, gboolean setting)
+{
+	TnyGtkHeaderListModelPriv *priv = TNY_GTK_HEADER_LIST_MODEL_GET_PRIVATE (self);
+	priv->no_duplicates = setting;
+	return;
+}
+
 /**
  * tny_gtk_header_list_model_set_folder:
  * @self: a #TnyGtkHeaderListModel
@@ -1170,6 +1266,7 @@ tny_gtk_header_list_model_set_folder (TnyGtkHeaderListModel *self, TnyFolder *fo
 	TnyGtkHeaderListModelPriv *priv = TNY_GTK_HEADER_LIST_MODEL_GET_PRIVATE (self);
 	GtkTreeIter iter;
 	GtkTreePath *path;
+	GPtrArray *copy_items;
 
 	g_static_rec_mutex_lock (priv->iterator_lock);
 
@@ -1185,14 +1282,12 @@ tny_gtk_header_list_model_set_folder (TnyGtkHeaderListModel *self, TnyFolder *fo
 	 * assertion in gtk_tree_model_sort_build_level (I have no idea why the
 	 * assertion is placed there, in stead of a normal return, though) */
  
-	g_ptr_array_foreach (priv->items, (GFunc) g_object_unref, NULL);
+	copy_items = priv->items;
+	priv->registered = 0;
+	priv->items = g_ptr_array_sized_new (tny_folder_get_all_count (folder));
 	if (priv->folder)
 		g_object_unref (priv->folder);
 	priv->folder = TNY_FOLDER (g_object_ref (folder));
-	priv->registered = 0;
-	g_ptr_array_free (priv->items, TRUE);
-	priv->items = g_ptr_array_sized_new (tny_folder_get_all_count (folder));
-
 	g_static_rec_mutex_unlock (priv->iterator_lock);
 
 	/* Get a new list of headers */
@@ -1205,6 +1300,9 @@ tny_gtk_header_list_model_set_folder (TnyGtkHeaderListModel *self, TnyFolder *fo
 
 	g_static_rec_mutex_lock (priv->iterator_lock);
 
+	g_ptr_array_foreach (copy_items, (GFunc) g_object_unref, NULL);
+	g_ptr_array_free (copy_items, TRUE);
+
 	/* Reference the new folder instance */
 
 	/* Notify the observers (the GtkTreeView) */
