@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with self library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <glib.h>
@@ -27,6 +27,7 @@ G_BEGIN_DECLS
 gboolean _mozilla_preference_set (const char *preference_name, const char *new_value);
 gboolean _mozilla_preference_set_boolean (const char *preference_name, gboolean new_boolean_value);
 gboolean _mozilla_preference_set_int (const char *preference_name, gint new_int_value);
+void _mozilla_preference_init ();
 
 G_END_DECLS
 
