@@ -228,21 +228,26 @@ on_send_queue_error_happened (TnySendQueue *self, TnyHeader *header, TnyMsg *msg
 {
 	if (header)
 	{
-		gchar *str = g_strdup_printf ("%s. Do you want to remove the message (%s)?",
-			err->message, tny_header_get_subject (header));
+		gchar *subject;
+		gchar *str;
+		
+		subject = tny_header_dup_subject (header);
+		str = g_strdup_printf ("%s. Do you want to remove the message (%s)?",
+			err->message, subject);
+		g_free (subject);
 		OnResponseInfo *info = g_slice_new (OnResponseInfo);
 
 		if (!rem_dialog)
 		{
 			rem_dialog = gtk_message_dialog_new (NULL, 0,
 				GTK_MESSAGE_ERROR, GTK_BUTTONS_YES_NO, str);
-			g_free (str);
 			info->self = g_object_ref (user_data);
 			info->header = g_object_ref (header);
 			g_signal_connect (G_OBJECT (rem_dialog), "response",
 				G_CALLBACK (on_response), info);
 			gtk_widget_show_all (rem_dialog);
 		}
+		g_free (str);
 	}
 	return;
 }
@@ -632,11 +637,16 @@ on_header_view_key_press_event (GtkTreeView *header_view, GdkEventKey *event, gp
 
 			if (header)
 			{
-				GtkWidget *dialog = gtk_message_dialog_new (NULL, 
+				gchar *subject;
+				GtkWidget *dialog;
+
+				subject = tny_header_dup_subject (header);
+				dialog = gtk_message_dialog_new (NULL, 
 					GTK_DIALOG_MODAL,
 					GTK_MESSAGE_WARNING, GTK_BUTTONS_YES_NO, 
 					_("This will remove the message with subject \"%s\""),
-					tny_header_get_subject (header));
+					subject);
+				g_free (subject);
 
 				if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
 				{
@@ -680,11 +690,16 @@ on_header_view_key_press_event (GtkTreeView *header_view, GdkEventKey *event, gp
 
 			if (header)
 			{
-				GtkWidget *dialog = gtk_message_dialog_new (NULL, 
-					GTK_DIALOG_MODAL,
-					GTK_MESSAGE_WARNING, GTK_BUTTONS_YES_NO, 
-					_("This will uncache the message with subject \"%s\""),
-					tny_header_get_subject (header));
+				gchar *subject;
+				GtkWidget *dialog;
+
+				subject = tny_header_dup_subject (header);
+				dialog = gtk_message_dialog_new (NULL, 
+								 GTK_DIALOG_MODAL,
+								 GTK_MESSAGE_WARNING, GTK_BUTTONS_YES_NO, 
+								 _("This will uncache the message with subject \"%s\""),
+								 subject);
+				g_free (subject);
 
 				if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
 				{
@@ -1100,6 +1115,90 @@ on_header_view_tree_row_activated (GtkTreeView *treeview, GtkTreePath *path,
 }
 
 
+static void
+on_rename (TnyFolder *self, gboolean cancelled, TnyFolderStore *into, TnyFolder *new_folder, GError *err, gpointer user_data)
+{
+	GObject *mself = user_data;
+
+	if (err != NULL)
+	{
+		GtkWidget *edialog;
+		edialog = gtk_message_dialog_new (
+			  GTK_WINDOW (gtk_widget_get_parent (GTK_WIDGET (mself))),
+			  GTK_DIALOG_DESTROY_WITH_PARENT,
+			  GTK_MESSAGE_ERROR,
+			  GTK_BUTTONS_CLOSE,
+			  err->message);
+		g_signal_connect_swapped (edialog, "response",
+			G_CALLBACK (gtk_widget_destroy), edialog);
+		gtk_widget_show_all (edialog);
+	} else {
+	}
+
+	g_object_unref (mself);
+}
+
+static void
+fetch_folder_store_from_path (const gchar *path, TnyFolder *origin, gchar **name, TnyFolderStore **into)
+{
+	TnyFolderStore *origin_store;
+	*name = NULL;
+	*into = NULL;
+
+	if (path == NULL || path[0] == '\0') {
+		return;
+	}
+
+	if (path[0] == '/') {
+		origin_store = TNY_FOLDER_STORE (tny_folder_get_account (origin));
+		path++;
+	} else {
+		origin_store = tny_folder_get_folder_store (origin);
+	}
+
+	while (*path != '\0') {
+		char *slash_pos;
+		if (strncmp (path, "../", 3)== 0) {
+			if (!TNY_IS_ACCOUNT (origin_store)) {
+				TnyFolderStore *tmp;
+				tmp = tny_folder_get_folder_store (TNY_FOLDER (origin_store));
+				if (tmp) {
+					g_object_unref (origin_store);
+					origin_store = tmp;
+				}
+			}
+			path += 3;
+		} else if ((slash_pos = strstr (path, "/")) != NULL) {
+			TnyFolderStoreQuery *query;
+			TnyList *list;
+			gchar *folder_name;
+			
+			query = tny_folder_store_query_new ();
+			folder_name = g_strndup (path, slash_pos - path);
+			tny_folder_store_query_add_item (query, folder_name, TNY_FOLDER_STORE_QUERY_OPTION_MATCH_ON_NAME);
+			g_free (folder_name);
+			list = tny_simple_list_new ();
+			tny_folder_store_get_folders (origin_store, list, query, NULL);
+			g_object_unref (query);
+			if (tny_list_get_length (list) == 1) {
+				TnyIterator *iter;
+				TnyFolderStore *tmp;
+				iter = tny_list_create_iterator (list);
+				tmp = TNY_FOLDER_STORE (tny_iterator_get_current (iter));
+				g_object_unref (origin_store);
+				origin_store = tmp;
+				g_object_unref (iter);
+			}
+			g_object_unref (list);
+			path = slash_pos + 1;
+		} else {
+			*name = g_strdup (path);
+			*into = origin_store;
+			break;
+		}
+	}
+}
+
 static void 
 on_rename_folder_activate (GtkMenuItem *mitem, gpointer user_data)
 {
@@ -1151,39 +1250,29 @@ on_rename_folder_activate (GtkMenuItem *mitem, gpointer user_data)
 				{
 					gboolean move = (result == GTK_RESPONSE_ACCEPT);
 					GError *err = NULL;
-					const gchar *newname = gtk_entry_get_text (GTK_ENTRY (entry));
+					gchar *entry_value = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
+					gchar *new_name;
 					TnyFolderStore *into;
-					TnyFolder *nfol;
 
 					clear_header_view (priv);
 
-					into = tny_folder_get_folder_store (folder);
-					nfol = tny_folder_copy (folder, into, newname, move, &err);
+					
+					fetch_folder_store_from_path (entry_value, folder, &new_name, &into);
 
-					if (err != NULL)
-					{
-						GtkWidget *edialog;
+					gtk_widget_destroy (dialog);
+					dialog = NULL;
 
-						gtk_widget_destroy (dialog);
-						dialog = NULL;
+					if (new_name != NULL) {
 
-						edialog = gtk_message_dialog_new (
-										  GTK_WINDOW (gtk_widget_get_parent (GTK_WIDGET (self))),
-										  GTK_DIALOG_DESTROY_WITH_PARENT,
-										  GTK_MESSAGE_ERROR,
-										  GTK_BUTTONS_CLOSE,
-										  err->message);
-						g_signal_connect_swapped (edialog, "response",
-							G_CALLBACK (gtk_widget_destroy), edialog);
-						gtk_widget_show_all (edialog);
-						g_error_free (err);
+						tny_folder_copy_async (folder, into, new_name, move, 
+								       on_rename, NULL, g_object_ref (self));
+
+						g_free (new_name);
 					}
 
 					if (into)
 						g_object_unref (into);
 
-					if (nfol)
-						g_object_unref (nfol);
 
 				}
 				break;
@@ -1433,7 +1522,25 @@ cancel_button_clicked (GtkWidget *button, gpointer user_data)
 	}
 }
 
+static void
+create_cb (TnyFolderStore *store, gboolean cancelled, TnyFolder *new_folder, GError *err, gpointer user_data)
+{
+	TnyDemouiSummaryView *self = user_data;
+	if (err != NULL) {
+		GtkWidget *edialog;
+		edialog = gtk_message_dialog_new (
+						  GTK_WINDOW (gtk_widget_get_parent (GTK_WIDGET (self))),
+						  GTK_DIALOG_DESTROY_WITH_PARENT,
+						  GTK_MESSAGE_ERROR,
+						  GTK_BUTTONS_CLOSE,
+						  err->message);
+		g_signal_connect_swapped (edialog, "response",
+			G_CALLBACK (gtk_widget_destroy), edialog);
+		gtk_widget_show_all (edialog);
+	}
 
+	g_object_unref (self);
+}
 static void 
 on_create_folder_activate (GtkMenuItem *mitem, gpointer user_data)
 {
@@ -1483,29 +1590,10 @@ on_create_folder_activate (GtkMenuItem *mitem, gpointer user_data)
 					GError *err = NULL;
 					const gchar *newname = gtk_entry_get_text (GTK_ENTRY (entry));
 
-					TnyFolder *created = tny_folder_store_create_folder (folderstore, newname, &err);
+					tny_folder_store_create_folder_async (folderstore, newname, 
+						create_cb, status_update, 
+						g_object_ref (self));
 
-					if (created) /* Can be NULL on failure */
-						g_object_unref (G_OBJECT (created));
-
-					if (err != NULL)
-					{
-						GtkWidget *edialog;
-
-						gtk_widget_destroy (dialog);
-						dialog = NULL;
-
-						edialog = gtk_message_dialog_new (
-										  GTK_WINDOW (gtk_widget_get_parent (GTK_WIDGET (self))),
-										  GTK_DIALOG_DESTROY_WITH_PARENT,
-										  GTK_MESSAGE_ERROR,
-										  GTK_BUTTONS_CLOSE,
-										  err->message);
-						g_signal_connect_swapped (edialog, "response",
-							G_CALLBACK (gtk_widget_destroy), edialog);
-						gtk_widget_show_all (edialog);
-						g_error_free (err);
-					}
 				}
 				break;
 
@@ -1515,7 +1603,7 @@ on_create_folder_activate (GtkMenuItem *mitem, gpointer user_data)
 
 			if (dialog)
 				gtk_widget_destroy (dialog);
-			g_object_unref (G_OBJECT (folderstore));
+			g_object_unref (folderstore);
 		}
 
 	}
@@ -1552,11 +1640,15 @@ on_move_to_folder_activate (GtkMenuItem *mitem, gpointer user_data)
 
 	if (folder) 
 	{
+		gchar *subject;
 		TnyList *headers = tny_simple_list_new ();
 
+		subject = tny_header_dup_subject (info->header);
+
 		g_print ("Transfering: %s to %s\n",
-			tny_header_get_subject (info->header), 
+			subject, 
 			tny_folder_get_name (info->to_folder));
+		g_free (subject);
 
 		tny_list_prepend (headers, (GObject *) info->header);
 		tny_folder_transfer_msgs_async (folder, headers, info->to_folder, 
